
WITH zg_glob1;
USE  zg_glob1;
WITH zg_glob2;
USE  zg_glob2;
WITH zg_glob3;  
USE  zg_glob3;
WITH zg_glob4;
USE  zg_glob4;

WITH calendar;
USE  calendar;

procedure tk_at01 is

  PRAGMA suppress(access_check);
  PRAGMA suppress(discriminant_check);
  PRAGMA suppress(index_check);
  PRAGMA suppress(length_check);
  PRAGMA suppress(range_check);
  PRAGMA suppress(division_check);
  PRAGMA suppress(overflow_check);
  PRAGMA suppress(elaboration_check);
  PRAGMA suppress(storage_check);

    type Transfer_States is ( Unknown, Aborted, Completed );
    State       : Transfer_States := Unknown;
    Interval	: Float6 := 0.0;      -- Time in second for an entire test run
    Min_Delay	: Float6 := 2.0 ** (-10);
    Target      : Float6 := 1.0;
    Count       : Float6;
    My_Interval : Float6;
    Elapsed_Time: Float6;
    First_Time,
    Start_Time  : Calendar.Time;
    Min_Time    : Float6;
    Transfer_Taken : exception;
    Transfer_Not_Taken : exception;

BEGIN

    -- Make sure we have a loop that will burn at least the minimum delay
    -- compiler can handle

    -- First, synch the clock
    First_Time := Calendar.Clock;
    Start_Time := First_Time;
    while First_Time = Start_Time loop
      Start_Time := Calendar.Clock;
    end loop;
    -- Clock has just ticked.

    delay Duration(Min_Delay);
    Min_Time := Float6(Calendar.Clock - Start_Time);

    Elapsed_Time := Min_Time;
    while Elapsed_Time <= Min_Time loop
       Min_Delay := 2.0 * Min_Delay;
       if ZG_Glob3.Boolean_False then
         ZG_Glob2.Put_Comment_Optional ( 
            "Ignore (fooling optimizer) " & Float6'Image (Min_Delay) );
         ZG_Glob2.Put_New_Line_Optional;
         Proc_Spoil;
       end if;

       -- Synch the clock
       First_Time := Calendar.Clock;
       Start_Time := First_Time;
       while First_Time = Start_Time loop
         Start_Time := Calendar.Clock;
       end loop;
    -- Clock has just ticked.

       delay Duration(Min_Delay);
       Elapsed_Time := Float6(Calendar.Clock - Start_Time);
    end loop;
    -- Now we have a minimum delay time (Min_Delay) that is achievable.

    -- Find Interval and Target so that repeating test code Target times
    -- takes at least Interval seconds, and Interval is large enough
    -- that Interval/5.0 is an achievable delay ( >= Min_Delay).
    while Interval  < 5.0 * Min_Delay loop

      Count := ZG_Glob4.One;
      Target := 2.0 * Target;
      if ZG_Glob3.Boolean_False then
         zg_glob2.put_comment ("Ignore (fooling optimizer) " & 
                               Float6'Image (Target));
         zg_glob2.put_new_line;
         Proc_Spoil;
      end if;
      -- Synch the clock
      First_Time := Calendar.Clock;
      Start_Time := First_Time;
      while First_Time = Start_Time loop
        Start_Time := Calendar.Clock;
      end loop;
      -- Clock has just ticked.
      while Count < Target loop
        Count := Count + ZG_Glob4.One;
        if Count > 2.0 * ZG_Glob4.One * TARGET then
           Proc_Spoil;
        end if;
      end loop;
      Interval := Float6 (Calendar.Clock - Start_Time);
    end loop;
    -- Now, executing the loop from One to Target takes at least Interval
    -- seconds, and Interval/5.0 is at least Min_Delay.

TK_AT_ATC_TAKEN_01:
begin
    --
    -- Test Name         :  tk_at_atc_taken_01
    -- Prime Purpose     :  Allow estimate of the difference in execution
    --			    speed with and without transfer of control.
    --			    Transfer DOES take place in this test
    -- Optimization      :  
    -- Related tests     :  tk_at_atc_not_taken_02
    -- Author            :  Daniel M. Roy
    -- Reviewer          :  Phil Brashear
    -- Date              :  January 20, 1995
    -- Source            :
    -- Dependencies      :
    -- Other Information :
    -- Revisions         :  23 AUG 95, MCH
    --                      Renamed test from a9_ms_atc_taken
    --                                     to tk_at_atc_taken_01.

    zg_glob2.put_test_name("tk_at_atc_taken_01");


    My_Interval := Interval / 5.0;    -- Short enough to ensure that delay
                                      -- expires before loop completes.

    PRAGMA include("startime");
    -- Code to time begins below
    Count := ZG_Glob4.One;
    select
      delay calendar.day_duration (My_Interval);
      while Count < Target loop
        Count := Count + ZG_Glob4.One;
        if Count > 2.0 * ZG_Glob4.One * TARGET then
           Proc_Spoil;
        end if;
      end loop;
      State := Aborted;
    then abort
      Count := Count * ZG_Glob4.One;
      while Count < Target loop
        Count := Count + ZG_Glob4.One;
        if Count > 2.0 * ZG_Glob4.One * Target then
           Proc_Spoil;
        end if;
      end loop;
      State := Completed;
    end select;
    -- Code to time ends above
    PRAGMA include("stoptime0");
    if State /= Aborted then
      raise Transfer_Not_Taken;
    end if;
    zg_glob2.put_comment_optional(
       "   Asynchronous Select Statement, asynchronous transfer TAKEN"); 
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional(
       "   -- cf tk_at_atc_not_taken_02");
    zg_glob2.put_new_line_optional;
    PRAGMA include("stoptime2");
    --------------------------------------------------------------------------
  exception
    when Transfer_Not_Taken =>
       zg_glob2.put_comment(
          "Asynchronous transfer not taken.  This behavior is incorrect.  It");
      zg_glob2.put_new_line;
      zg_glob2.put_comment(
        "   is not meaningful to report the execution time of this problem");
      zg_glob2.put_new_line;
      zg_glob2.put_comment(
        "   when the behavior is not correct.");
      zg_glob2.put_new_line;
      zg_glob2.put_error_code( err_at_execution_time );
END tk_at_atc_taken_01;

TK_AT_ATC_NOT_TAKEN_02:
begin

    --
    -- Test Name         :  tk_at_atc_not_taken_02
    -- Prime Purpose     :  Allow estimate of the difference in execution
    --				speed with and without transfer of control
    --				ATC does NOT take place in this test
    -- Optimization      :  
    -- Related tests     :  tk_at_atc_taken_01
    -- Author            :  Daniel M. Roy
    -- Reviewer          :  Phil Brashear
    -- Date              :  January 20, 1995
    -- Source            :
    -- Dependencies      :
    -- Other Information :
    -- Revisions         :  23 AUG 95, MCH
    --                      Renamed test from a9_ms_atc_not_taken
    --                                     to tk_at_atc_not_taken_02.


    zg_glob2.put_test_name("tk_at_atc_not_taken_02");

    My_Interval := 5.0 * Interval;  -- Long enough to ensure completion
                                    -- of loop before delay expires

    -- Now, for the real thing
    PRAGMA include("startime");
    -- Code to time begins below
    Count := ZG_Glob4.One;
    select
      delay calendar.day_duration (My_Interval);
      while Count < Target loop
        Count := Count + ZG_Glob4.One;
        if Count > 2.0 * ZG_Glob4.One * TARGET then
           Proc_Spoil;
        end if;
      end loop;
      State := Aborted;
    then abort
      Count := Count * ZG_Glob4.One;
      while Count < Target loop
        Count := Count + ZG_Glob4.One;
        if Count > 2.0 * ZG_Glob4.One * Target then
           Proc_Spoil;
        end if;
      end loop;
      State := Completed;
    end select;
    -- Code to time ends above
    PRAGMA include("stoptime0");
    if State /= Completed then
      raise Transfer_Taken;
    end if;
    zg_glob2.put_comment_optional(
	"   Asynchronous Select Statement, asynchronous transfer NOT taken "); 
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional(
	"   -- cf tk_at_atc_taken_01");
    zg_glob2.put_new_line_optional;
    PRAGMA include("stoptime2");
    ---------------------------------------------------------------------------
  exception
    when Transfer_Taken =>
       zg_glob2.put_comment(
          "Asynchronous transfer was taken.  This behavior is incorrect.  It");
      zg_glob2.put_new_line;
      zg_glob2.put_comment(
        "   is not meaningful to report the execution time of this problem");
      zg_glob2.put_new_line;
      zg_glob2.put_comment(
        "   when the behavior is not correct.");
      zg_glob2.put_new_line;
      zg_glob2.put_error_code( err_at_execution_time );

END tk_at_atc_not_taken_02;


end tk_at01;
