-- +++ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
WITH zg_glob1;
USE  zg_glob1;
WITH zg_glob2;
USE  zg_glob2;
WITH zg_glob3;
USE  zg_glob3;
WITH zg_glob4;
USE  zg_glob4;

WITH starlet;             --  VMS specific
USE  starlet;
WITH calendar;
USE  calendar;
WITH system;
 
PROCEDURE tk_in_int_02 IS
 
--  This test uses DEC Ada AST (asynchronous system trap) from the package
--  Starlet.  To use with a system which actually supports interrupts, two
--  changes will have to be made:
--    1)  Use low_level_IO.send_control to tie the output from the program
--           to an interrupt port.  This permits the program to generate
--           an external interrupt.  This will replace the entry calls on
--           RECEIVE_AST.
--    2)  An ADDRESS clause must be added to tie the interrupt handler to the
--           interrupt port. (LRM 13.5)
 
  interrupt_service_flag : boolean;
  time1                  : time;
 
  -- pragma suppress(access_check);
  -- pragma suppress(discriminant_check);
  -- pragma suppress(index_check);
  -- pragma suppress(length_check);
  -- pragma suppress(range_check);
  -- pragma suppress(division_check);
  -- pragma suppress(overflow_check);
  -- pragma suppress(elaboration_check);
  -- pragma suppress(storage_check);
 
  
  TASK low_priority IS
         --+ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
      PRAGMA priority( zg_glob1.priority_1 );
  END low_priority;
 
  TASK handle_int2 IS
 
    ENTRY receive_ast;                --  This entry is also VMS specific
                                      -- for RECEIVE_AST use at ????????; --
                                      -- this will be modified to match target
         --+ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
      PRAGMA priority( zg_glob1.priority_3 );
    PRAGMA ast_entry(receive_ast);    -- goes away on new target
 
  END handle_int2;
 
  TASK higher_priority IS
         --+ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
      PRAGMA priority( zg_glob1.priority_2 );
    ENTRY high;
  END higher_priority;
 
  TASK BODY higher_priority IS
  BEGIN
    LOOP
      ACCEPT high;
      zg_glob4.proc0;
    END LOOP;
  END higher_priority;
 
  TASK BODY handle_int2 IS
  BEGIN
    LOOP
      ACCEPT receive_ast DO
        interrupt_service_flag := true;
        higher_priority.high;
      END receive_ast;
    END LOOP;
  END handle_int2;
 
  TASK BODY low_priority IS
  BEGIN
 
    -- Test Name         :  tk_in_int_02
    -- Prime Purpose     :  Task switching.  An interrupt enables a task with
    --                      a higher priority than the task which was running
    --                      when the interrupt occurred. After the interrupt has
    --                      been serviced, the higher priority task, not the one
    --                      running when the interrupt occurred, will be
    --                      scheduled. 
    -- LRM Version       :  Revision A
    -- Optimization      :
    -- Related Tests     :  tk_in_int_01 .. tk_in_int_09
    -- Author            :  Maralee Turner
    -- Reviewer          :  T. Leavitt, 11-oct-89
    -- Date              :
    -- Source            :
    -- Dependencies      :
    -- Other Information :  This test will have to be modified to run on
    --                      systems which actually support interrupts.
    --                      Many non-embedded systems do no support interrupts;
    --                      these tests may be omitted in many cases.
 
  
    ----------------------------------------------------------------------------
    zg_glob2.put_test_name("tk_in_int_02");
 
    PRAGMA include("startime");

    interrupt_service_flag := false;
    handle_int2.receive_ast;         -- replace by LOW_LEVEL_IO.SEND_CONTROL 

    time1 := clock;
    WHILE clock <= time1 + 1.0 AND NOT interrupt_service_flag LOOP 
                                              --  prevents possibility
                                              --  of infinite wait

      NULL;
    END LOOP;
    IF NOT interrupt_service_flag THEN
      zg_glob2.put_test_name("tk_in_int_02");
      zg_glob2.put_comment
	("   test failed - did not return from the interrupt routine.");
      zg_glob2.put_new_line;
      zg_glob2.put_error_code(zg_glob1.err_at_execution_time);--+tcl 11-oct-89
      zg_glob4.die;
    END IF;
 
    PRAGMA include("stoptime0");

    zg_glob2.put_comment_optional
	("   - An interrupt enables a task with a higher priority ");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("   than the task which was running when the interrupt occurred.");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("   The interrupt task is initiated from the main task, the ");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("   higher priority task is initiated from the interrupt. After");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("   the interrupt has been serviced, the higher priority task, not");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("   the one running when the interrupt occurred, will be scheduled.");
    zg_glob2.put_new_line_optional;

    PRAGMA include("stoptime2");
 
    ABORT higher_priority;
    ABORT handle_int2;
    ABORT low_priority;
  EXCEPTION
    WHEN OTHERS =>
      NULL;
  END low_priority;
 
BEGIN
  NULL;
EXCEPTION
  WHEN OTHERS =>
    NULL;
END tk_in_int_02;
