-- +++ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
WITH zg_glob1;
USE  zg_glob1;
WITH zg_glob2;
USE  zg_glob2;
WITH zg_glob3;
USE  zg_glob3;
WITH zg_glob4;
USE  zg_glob4;

WITH starlet;                   -- VMS specific
USE  starlet;
WITH calendar;
USE  calendar;
WITH system;
 
PROCEDURE tk_in_int_03 IS
 
--  This test uses DEC Ada AST (asynchronous system trap) from the package
--  Starlet.  To use with a system which actually supports interrupts, two
--  changes will have to be made:
--    1)  Use low_level_IO.send_control to tie the output from the program
--           to an interrupt port.  This permits the program to generate
--           an external interrupt.  This will replace the entry calls on
--           RECEIVE_AST.
--    2)  An ADDRESS clause must be added to tie the interrupt handler to the
--           interrupt port. (LRM 13.5)
 
  minutes                : CONSTANT := 60.0;
 
  time1                  : time;
 
  interrupt_service_flag : boolean;
 
  -- pragma suppress(access_check);
  -- pragma suppress(discriminant_check);
  -- pragma suppress(index_check);
  -- pragma suppress(length_check);
  -- pragma suppress(range_check);
  -- pragma suppress(division_check);
  -- pragma suppress(overflow_check);
  -- pragma suppress(elaboration_check);
  -- pragma suppress(storage_check);
 
  
  TASK TYPE several_tasks IS    -- use of a task type to create
    ENTRY start_task;           -- several tasks
  END several_tasks;
 
  t1, t2, t3, t4, t5, t6 : several_tasks;
 
  TASK BODY several_tasks IS
  BEGIN
    LOOP
      DELAY 5 * minutes;    -- wait queue
      ACCEPT start_task;
    END LOOP;
  END several_tasks;
 
  TASK time_task IS
         --+ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
      PRAGMA priority( zg_glob1.priority_1 );
  END time_task;
 
  TASK handle_int3 IS
    ENTRY receive_ast;                --  This entry is also VMS specific
                                      -- for RECEIVE_AST use at ????????; --
                                      -- this will be modified to match target
         --+ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
      PRAGMA priority( zg_glob1.priority_2 );
    PRAGMA ast_entry(receive_ast);    -- goes away on new target
  END handle_int3;
 
  TASK BODY handle_int3 IS
  BEGIN
    LOOP
      ACCEPT receive_ast DO
        interrupt_service_flag := true;
      END receive_ast;
    END LOOP;
  END handle_int3;
 
  TASK BODY time_task IS
  BEGIN
 
 
    -- Test Name         :  tk_in_int_03
    -- Prime Purpose     :  Timing of a simple interrupt is complicated by
    -- having several tasks on a wait queue.
    -- LRM Version       :  Revision A
    -- Optimization      :
    -- Related Tests     :  tk_in_int_01 .. tk_in_int_09
    -- Author            :  Maralee Turner
    -- Reviewer          :  T. Leavitt, 11-oct-89
    -- Date              :
    -- Source            :
    -- Dependencies      :
    -- Other Information :  This test will have to be modified to run on
    --                      systems which actually support interrupts.
    --                      Many non-embedded systems do no support interrupts;
    --                      these tests may be omitted in many cases.
 
  
    ----------------------------------------------------------------------------
    zg_glob2.put_test_name("tk_in_int_03");

    PRAGMA include("startime");
 
    interrupt_service_flag := false;
    handle_int3.receive_ast;    -- replace by LOW_LEVEL_IO.SEND_CONTROL 

    time1 := clock;
    WHILE clock <= time1 + 1.0 AND NOT interrupt_service_flag LOOP   
                                              --  prevents possibility
                                              --  of infinite wait
      NULL;
    END LOOP;
    IF NOT interrupt_service_flag THEN
      zg_glob2.put_comment
	("   test failed - did not return from interrupt routine.");
      zg_glob2.put_new_line;
      zg_glob2.put_error_code(zg_glob1.err_at_execution_time); --+tcl 11-oct-89
      zg_glob4.die;
    END IF;
 
    PRAGMA include("stoptime0");

    zg_glob2.put_comment_optional
	("   - several tasks are placed in a delay queue to test");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("   scheduling overheads associated with restarting execution");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional("   after processing an interrupt.");
    zg_glob2.put_new_line_optional;

    PRAGMA include("stoptime2");
 
    ABORT t1, t2, t3, t4, t5, t6;
    ABORT handle_int3;
    ABORT time_task;
  EXCEPTION
    WHEN OTHERS =>
      NULL;
  END time_task;
 
BEGIN
  NULL;
EXCEPTION
  WHEN OTHERS =>
    NULL;
END tk_in_int_03;
