-- +++ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
WITH zg_glob1;
USE  zg_glob1;
WITH zg_glob2;
USE  zg_glob2;
WITH zg_glob3;
USE  zg_glob3;
WITH zg_glob4;
USE  zg_glob4;

WITH starlet;
USE  starlet;
WITH calendar;
USE  calendar;
WITH system;
 
--  This test uses DEC Ada AST (asynchronous system trap) from the package
--  Starlet.  To use with a system which actually supports interrupts, two
--  changes will have to be made:
--    1)  Use low_level_IO.send_control to tie the output from the program
--           to an interrupt port.  This permits the program to generate
--           an external interrupt.  This will replace the entry calls on
--           RECEIVE_AST.
--    2)  An ADDRESS clause must be added to tie the interrupt handler to the
--           interrupt port. (LRM 13.5)
 
PROCEDURE tk_in_int_05 IS
 
  time1                  : time;
  interrupt_service_flag : boolean;
 
  -- pragma suppress(access_check);
  -- pragma suppress(discriminant_check);
  -- pragma suppress(index_check);
  -- pragma suppress(length_check);
  -- pragma suppress(range_check);
  -- pragma suppress(division_check);
  -- pragma suppress(overflow_check);
  -- pragma suppress(elaboration_check);
  -- pragma suppress(storage_check);
 
  
  TASK time_task IS
         --+ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
      PRAGMA priority( zg_glob1.priority_1 );
  END time_task;
 
 
  TASK BODY time_task IS
  BEGIN
 
    -- Test Name         :  tk_in_int_05
    -- Prime Purpose     :  An exception is raised within the rendezvous of the
    --                      interrupt entry call.  This problem tests for the
    --                      performance impact of raising exceptions inside
    --                      the rendezvous.
    -- LRM Version       :  Revision A
    -- Optimization      :
    -- Related Tests     :  tk_in_int_01 .. tk_in_int_09
    -- Author            :  Maralee Turner
    -- Reviewer          :  T. Leavitt, 11-oct-89
    -- Date              :
    -- Source            :
    -- Dependencies      :
    -- Other Information :  This test will have to be modified to run on
    --                      systems which actually support interrupts.
    --                      Many non-embedded systems do no support interrupts;
    --                      these tests may be omitted in many cases.
  
    ----------------------------------------------------------------------------
    zg_glob2.put_test_name("tk_in_int_05");

    PRAGMA include("startime");
 
    exc_block :
      DECLARE
        except_in_interrupt : EXCEPTION;
        TASK handle_int5 IS
          ENTRY receive_ast;                -- This entry is also VMS specific
                                            -- for RECEIVE_AST use at ????????;
                                            -- this will be modified to match
                                            -- target
          --+ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
          PRAGMA priority( zg_glob1.priority_2 );
          PRAGMA ast_entry(receive_ast);    -- goes away on new target
        END handle_int5;
 
        TASK BODY handle_int5 IS
        BEGIN
          ACCEPT receive_ast DO
            interrupt_service_flag := true;
            IF zg_glob4.ll /= zg_glob4.mm   -- this condition is always true
	    THEN			    
              RAISE except_in_interrupt;    -- an interrupt is raised within
            END IF;                         -- the handler of another
          END receive_ast;
        EXCEPTION
          WHEN except_in_interrupt =>
            NULL;
        END handle_int5;
 
      BEGIN
        interrupt_service_flag := false;
        handle_int5.receive_ast;        -- replace by LOW_LEVEL_IO.SEND_CONTROL

        time1 := clock;
        WHILE clock <= time1 + 1.0 AND NOT	-- prevents possibility of
         interrupt_service_flag LOOP            -- infinite wait
          NULL;
        END LOOP;
        IF NOT interrupt_service_flag THEN
          zg_glob2.put_comment
		("   test failed - did not return from interrupt routine.");
          zg_glob2.put_new_line;
          zg_glob2.put_error_code(zg_glob1.err_at_execution_time);
						--+tcl 11-oct-89 review
          zg_glob4.die;
        END IF;
      EXCEPTION
        WHEN except_in_interrupt =>
          NULL;
      END exc_block;
 
    PRAGMA include("stoptime0");

    zg_glob2.put_comment_optional
	("   - an exception is raised within the rendezvous");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("   of the interrupt entry call.  This test determines");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("   the performance of exception handling and the");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("   propagation of an exception from the rendezvous.");
    zg_glob2.put_new_line_optional;

    PRAGMA include("stoptime2");
    ----------------------------------------------------------------------------
 
  EXCEPTION
    WHEN OTHERS =>
      NULL;
  END time_task;
 
BEGIN
  NULL;
EXCEPTION
  WHEN OTHERS =>
    NULL;
END tk_in_int_05;
