-- +++ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
WITH zg_glob1;
USE  zg_glob1;
WITH zg_glob2;
USE  zg_glob2;
WITH zg_glob3;
USE  zg_glob3;
WITH zg_glob4;
USE  zg_glob4;

WITH starlet;                                                 -- VMS specific
USE  starlet;
WITH calendar;
USE  calendar;
WITH system;
 
--  This test uses DEC Ada AST (asynchronous system trap) from the package
--  Starlet.  To use with a system which actually supports interrupts, two
--  changes will have to be made:
--    1)  Use low_level_IO.send_control to tie the output from the program
--           to an interrupt port.  This permits the program to generate
--           an external interrupt.  This will replace the entry calls on
--           RECEIVE_AST.
--    2)  An ADDRESS clause must be added to tie the interrupt handler to the
--           interrupt port. (LRM 13.5)
 
PROCEDURE tk_in_int_08 IS
 
  time1             : time;
 
  int1_service_flag, 
  int2_service_flag : boolean;
  serve_1, serve_2  : boolean := false;    --  boolean variables to flag which
  start_2           : boolean;             --  interrupt is serviced.
 
  -- pragma suppress(access_check);
  -- pragma suppress(discriminant_check);
  -- pragma suppress(index_check);
  -- pragma suppress(length_check);
  -- pragma suppress(range_check);
  -- pragma suppress(division_check);
  -- pragma suppress(overflow_check);
  -- pragma suppress(elaboration_check);
  -- pragma suppress(storage_check);
 
  
  TASK time_task IS
         --+ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
      PRAGMA priority( zg_glob1.priority_1 );
  END time_task;
 
  TASK inter1 IS
 
    ENTRY receive_ast;    --  This entry is also VMS specific
                          -- for RECEIVE_AST use at ????????; -- this will be
                          -- modified to match target
         --+ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
      PRAGMA priority( zg_glob1.priority_2 );
    PRAGMA ast_entry(receive_ast);
 
  END inter1;
 
  TASK inter2 IS
 
    ENTRY receive_ast;    --  This entry is also VMS specific
                          -- for RECEIVE_AST use at ????????; -- this will be
                          -- modified to match target
         --+ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
      PRAGMA priority( zg_glob1.priority_3 );
    PRAGMA ast_entry(receive_ast);
 
  END inter2;
 
  TASK BODY inter2 IS
  BEGIN
    LOOP
      ACCEPT receive_ast DO
        int2_service_flag := true;
        start_2 := true;    -- set flag to true indicating the interrupt
      END receive_ast;      --   handler was interrupted.
    END LOOP;
  END inter2;
 
  TASK BODY inter1 IS
  BEGIN
    LOOP
      ACCEPT receive_ast DO
        int1_service_flag := true;
        inter2.receive_ast;    -- replace by LOW_LEVEL_IO.SEND_CONTROL
        IF start_2 THEN        -- SERVE_2 indicates that the second interrupt
          serve_2 := true;     --   is serviced before the first completes.
        ELSE                   -- SERVE_1 indicated that the first interrupt
          serve_1 := true;     --   is completely handled before servicing
        END IF;                --   the second.
      END receive_ast;
    END LOOP;
  END inter1;
 
 
  TASK BODY time_task IS
  BEGIN
 
    -- Test Name         :  tk_in_int_08
    -- Prime Purpose     :  tk_in_int_08 tests the response time when an 
    --                      interrupt occurs during an interrupt handler.  
    --                      tk_in_int_08 is similar to tk_in_int_07 
    --                      except the second interrupt task has a
    --                      higher priority than the executing interrupt. This
    --                      test will determine whether priorities are
    --                      recognized by the handler.
    -- LRM Version       :  Revision A
    -- Optimization      :
    -- Related Tests     :  tk_in_int_01 .. tk_in_int_09
    -- Author            :  Maralee Turner
    -- Reviewer          :  T. Leavitt, 11-oct-89
    -- Date              :
    -- Source            :
    -- Dependencies      :
    -- Other Information :  This test will have to be modified to run on
    --                      systems which actually support interrupts.
    --                      Many non-embedded systems do no support interrupts;
    --                      these tests may be omitted in many cases.
 
  
    ----------------------------------------------------------------------------
    zg_glob2.put_test_name("tk_in_int_08");

    PRAGMA include("startime");
 
    int1_service_flag := false;
    int2_service_flag := false;
    start_2 := false;
    inter1.receive_ast;                            -- replace by
                                                   -- LOW_LEVEL_IO.SEND_CONTROL
 
    time1 := clock;
    WHILE clock <= time1 + 1.0 AND NOT int1_service_flag LOOP
      NULL;
    END LOOP;
    IF NOT int1_service_flag THEN
      zg_glob2.put_comment
	("   test failed - didn't return from 1st interrupt routine.");
      zg_glob2.put_new_line;
      zg_glob2.put_error_code(zg_glob1.err_at_execution_time); --+tcl 11-oct-89

      zg_glob4.die;
    END IF;
    IF NOT int2_service_flag THEN
      zg_glob2.put_comment
	("   test failed - didn't return from 2nd interrupt routine.");
      zg_glob2.put_new_line;
      zg_glob2.put_error_code(zg_glob1.err_at_execution_time); --+tcl 11-oct-89  
      zg_glob4.die;
    END IF;
 
    PRAGMA include("stoptime0");

    zg_glob2.put_comment_optional
	("   - tests the response time when an interrupt occurs");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("   during an interrupt handler.  tk_in_int_08 is similar");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("   to tk_in_int_07 except the second interrupt task has a");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("   higher priority than the executing interrupt. This");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("   test will determine whether priorities are recognized");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional("   by the handler.");
    zg_glob2.put_new_line_optional;
 
    IF serve_1 AND NOT serve_2 THEN
      zg_glob2.put_comment
	("  The first interrupt always completes before the second");
      zg_glob2.put_new_line;
      zg_glob2.put_comment("   interrupt is processed.");
      zg_glob2.put_new_line;
    ELSIF serve_2 AND NOT serve_1 THEN
      zg_glob2.put_comment
	("   The second interrupt is processed, then control returns");
      zg_glob2.put_new_line;
      zg_glob2.put_comment("   to the first interrupt.");
      zg_glob2.put_new_line;
    ELSE
      zg_glob2.put_comment
	("   The results are indefinite as to whether an interrupt");
      zg_glob2.put_new_line;
      zg_glob2.put_comment
	("   will always be processed immediately if raised in ");
      zg_glob2.put_new_line;
      zg_glob2.put_comment("   an interrupt handler..");
      zg_glob2.put_new_line;
    END IF;

    PRAGMA include("stoptime2");
    ----------------------------------------------------------------------------
 
    ABORT inter2;
    ABORT inter1;
    ABORT time_task;
  EXCEPTION
    WHEN OTHERS =>
      NULL;
  END time_task;
 
BEGIN
  NULL;
EXCEPTION
  WHEN OTHERS =>
    NULL;
END tk_in_int_08;
