with zg_glob1;
use  zg_glob1;
with zg_glob2;
with zg_glob3;
with zg_glob4;
WITH starlet;                                                 -- VMS specific
USE  starlet;
--with system;                    --+ tcl 7 April 1992, PCR #100 part 3
                                  --+    A "WITH SYSTEM;" may be necessary on
                                  --+    some compilers to use PRAGMA PRIORITY
                                  --+    even though Ada Interpretation
                                  --+    AI-00197 clearly states that it should
                                  --+    NOT be required.

with calendar;
use calendar;
 
--  This test uses DEC Ada AST (asynchronous system trap) from the package
--  STARLET.  To use with a system which actually supports interrupts, two
--  changes will have to be made:
--    1)  Use LOW_LEVEL_IO.SEND_CONTROL to tie the output from the program
--           to an interrupt port.  This permits the program to generate
--           an external interrupt.  This will replace the entry calls on
--           RECEIVE_AST. The only occurrence of calls is in procedure 
--           SIGNAL_INTERRUPT.
--    2)  An ADDRESS clause must be added to tie the interrupt handler to the
--           interrupt port. (LRM 13.5)
 
PROCEDURE tk_in_int_10 IS
 
  iteration_done : boolean := false;
  program_done   : boolean := false;
  count_of_switches  : zg_glob1.integer16 ;
  calling_task       : zg_glob1.integer16 ;
  count_during_current_iteration : zg_glob1.integer16 ;
  count_for_problem  : constant zg_glob1.integer16 := 100 ;

  TASK driver IS
    PRAGMA priority( zg_glob1.priority_2 );
    entry start_program; 
    entry stop_program;
    entry stop_iteration;
  END driver ; 

  TASK calling_task_1 IS
    PRAGMA priority( zg_glob1.priority_1 );
    entry start_iteration; 
    entry stop_problem ;
  END calling_task_1 ;

  TASK calling_task_2 IS
    PRAGMA priority( zg_glob1.priority_1 );
    entry start_iteration ;
    entry stop_problem ;
  END calling_task_2 ;
 
  TASK receiving_task IS
    ENTRY receive_ast;    --  This entry is VMS specific
                          --  for RECEIVE_AST use at ????????; 
                          --  Modified to match target requirements
    PRAGMA priority( zg_glob1.priority_2 );
    PRAGMA ast_entry(receive_ast);
  END receiving_task ;
 
  procedure set_caller ( id : zg_glob1.integer16 ) is
  begin
    if id /= calling_task then
      calling_task := id;
      count_of_switches := count_of_switches + 1;
    end if;
  end set_caller;


  procedure signal_interrupt is
  begin
    receiving_task.receive_ast; -- replace by call on LOW_LEVEL_IO.SEND_CONTROL 
  end signal_interrupt ;

  TASK BODY receiving_task IS
  BEGIN
    LOOP
      ACCEPT receive_ast 
      DO
        count_during_current_iteration := count_during_current_iteration + 1;
        if count_during_current_iteration >= count_for_problem 
        then
          iteration_done := true;
        end if;
      END receive_ast;

      exit when program_done;

    END LOOP;
  END receiving_task ;

  TASK body calling_task_1 IS
    exit_loop : boolean := false ;
  begin
   loop
     select 
       accept start_iteration ;
       loop
         set_caller(1); 
         zg_glob4.proc0;
         signal_interrupt;
         exit when iteration_done ;
       end loop;
       driver.stop_iteration;
     or
       accept stop_problem
         do
           exit_loop := true; 
         end;
     end select;
     exit when exit_loop;
  end loop;
  
  END calling_task_1 ;

  TASK body calling_task_2 IS
    exit_loop : boolean := false ;
  begin
   loop
     select 
       accept start_iteration ;
       loop
         set_caller(2); 
         zg_glob4.proc0;
         signal_interrupt; 
         exit when iteration_done ;
       end loop;
       driver.stop_iteration;
     or
       accept stop_problem 
         do
           exit_loop := true; 
         end;
     end select;
     exit when exit_loop;
  end loop;
  
  END calling_task_2 ;


  TASK BODY driver IS
  BEGIN
  
    accept start_program;
 
----------------------------------------------------------------------------
  -- Test Name         :  tk_in_int_10 
  -- Prime Purpose     :  tk_in_int_10 Tests whether a compilation system 
  --                      performs scheduling operations after an interrupt. 
  --                      It checks whether the task which resumes operations 
  --                      after an interrupt is the same as the task which was
  --                      executing before the interrupt or whether the system
  --                      sometimes schedules another task of equal priority.
  -- Optimization      :
  -- Related Tests     :  
  -- Author            :  T. Leavitt 
  -- Reviewer          :  B. Decker-Lindsey - 28 February 1992
  -- Date              :  December 1991
  -- Source            :  Test for this feature was recommended in CECOM 
  --                      report "Evaluation of the ACEC Benchmark Suite 
  --                      for Real-Time Applications." CIN:C02 092LY 0003 00, 
  --                      23 July 1990
  -- Dependencies      :  Tying tasks to interrupts
  -- Other Information :  This test will have to be modified to run on
  --                      systems which actually support interrupts.
  --                      Many non-embedded systems do not support interrupts;
  --                      these tests may be omitted in many cases.
  -- Revisions         :  PCR #100
  -- Author            :  Thomas C. Leavitt
  -- Reviewer          :  Julie Leastman - 14 April 1992
  -- Date              :  7 April 1992
  -- Changes           :  Commented out reference to WITH SYSTEM with comments 
  --                      stating that some systems may require it to use a 
  --                      PRAGMA PRIORITY directive. Corrected typo in output
  --                      of test description ("wither" replaced by "whether").

----------------------------------------------------------------------------

    count_of_switches := 0;
    count_during_current_iteration := 0 ;
    set_caller (0); 

    signal_interrupt; 

    zg_glob2.put_test_name("tk_in_int_10 ");
    PRAGMA include("startime");
      iteration_done := false ;
      count_during_current_iteration := 0 ;
      count_of_switches := 0;
      calling_task_1.start_iteration;
      calling_task_2.start_iteration;
      accept stop_iteration;         -- accept call from both CALLING_TASKs
      accept stop_iteration;
    PRAGMA include("stoptime0");
    zg_glob2.put_comment_optional(
       "   Tests whether a compilation system performs ");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional(
       "   scheduling operations after an interrupt. It checks ");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional(
       "   whether the task which resumes operations after ");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional(
       "   an interrupt is the same as the task which was ");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional(
                 --+ tcl 7 April 1992, PCR #100 part 4
                 --+       corrected typo, replaced "wither" with "whether"
       "   preempted by the interrupt or whether the system ");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional(
       "   sometimes schedules another task of equal priority. ");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional(
       "   Test performs 100 interrupts and 4 ""normal"" rendezvous.");
    zg_glob2.put_new_line_optional;
    PRAGMA include("stoptime2");

    IF count_of_switches <= 2 THEN
      zg_glob2.put_ancillary(
         " >>> tk_in_int_10 system always resumed preempted task" &
         " after an interrupt");
      zg_glob2.put_new_line;
    ELSIF count_of_switches >= 100 THEN
      zg_glob2.put_ancillary(
         " >>> tk_in_int_10 system always switched tasks after an interrupt");
      zg_glob2.put_new_line;
    ELSE
      zg_glob2.put_ancillary(
          " >>> tk_in_int_10 system sometimes switched tasks after" &
          " an interrupt");
      zg_glob2.put_new_line;
      zg_glob2.put_ancillary(
          " >>> tk_in_int_10 Number of switches is ");
      zg_glob2.put_value( count_of_switches );
      zg_glob2.put_comment(" out of ");
      zg_glob2.put_value( count_for_problem );
      zg_glob2.put_new_line;
    END IF;

    program_done := true;
    signal_interrupt;
    calling_task_1.stop_problem ;
    calling_task_2.stop_problem ;
    accept stop_program;
  end driver ;

BEGIN


-- This program intentionally does NOT specify any suppression pragmas.
-- It will not operate as intended if predefined constraint checking is
-- suppressed.
 
  DECLARE
    SUBTYPE mytype IS zg_glob1.integer16 RANGE 15000 .. 15001;
    a : mytype;
  BEGIN
    a := mytype(zg_glob4.ei);       -- not in range, ei has value "1"
                                    -- this assignment should raise
                                    -- CONSTRAINT_ERROR
 
    zg_glob2.put_comment(
       " The system is NOT performing range checks. ");
    zg_glob2.put_new_line;
    zg_glob2.put_comment(
       " This is either an implementation error or the program was ");
    zg_glob2.put_new_line;
    zg_glob2.put_comment(
       " improperly compiled with options requesting suppression of ");
    zg_glob2.put_new_line;
    zg_glob2.put_comment(
       " constraint checking. ");
    zg_glob2.put_new_line;
    zg_glob2.put_new_line;
    zg_glob2.put_comment(
       " User should verify compiler option settings and recompile if needed.");
    zg_glob2.put_new_line(2);
 
    zg_glob2.put_test_name(
       "tk_in_int_10 - system is suppressing all range checking ");
    zg_glob2.put_error_code(err_at_execution_time);

    zg_glob1.bool := false;
  EXCEPTION
    WHEN constraint_error =>
      zg_glob1.bool := true;
  END;
 
  IF NOT zg_glob1.bool THEN         -- if system has suppressed range checking
    abort driver, calling_task_1, calling_task_2, receiving_task ;
    RETURN;
  END IF;
                                    -- no point in proceeding

  driver.start_program;
  driver.stop_program;

END tk_in_int_10 ;
