-- +++ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
WITH zg_glob1;
USE  zg_glob1;
WITH zg_glob2;
USE  zg_glob2;
WITH zg_glob3;
USE  zg_glob3;
WITH zg_glob4;

WITH Text_IO;
WITH calendar;
USE  calendar;

PRAGMA elaborate
	(Text_IO, zg_glob1, zg_glob2, zg_glob3, zg_glob4, calendar );

PACKAGE pkg01_tk_la02 IS
 
  TASK background IS
         --+ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
      PRAGMA priority( zg_glob1.priority_2 );
    ENTRY initialization_complete;
    ENTRY start_problem;
    ENTRY stop_problem;
    ENTRY stop_program;
  END background;
 
  TASK main IS
         --+ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
      PRAGMA priority( zg_glob1.priority_1 );
    entry start_program ;
  END main;
END pkg01_tk_la02;

PACKAGE BODY pkg01_tk_la02 IS
 
  PRAGMA suppress(access_check);
  PRAGMA suppress(discriminant_check);
  PRAGMA suppress(index_check);
  PRAGMA suppress(length_check);
  PRAGMA suppress(range_check);
  PRAGMA suppress(division_check);
  PRAGMA suppress(overflow_check);
  PRAGMA suppress(elaboration_check);
  PRAGMA suppress(storage_check);
  
  TASK BODY background IS
  BEGIN
    ACCEPT initialization_complete;
 
    ACCEPT start_problem DO
      zg_glob4.char := 'Y';
      zg_glob2.put_comment
	("   Background task will now execute a GET ");
      zg_glob2.put_new_line;
      zg_glob2.put_comment
	("   The intent of test problem tk_la_async_04 is to observe ");
      zg_glob2.put_new_line;
      zg_glob2.put_comment
	("   performance while a background task waits for asynchronous input");
      zg_glob2.put_new_line;
      zg_glob2.put_comment
	("   If the program does not terminate in 10 minutes, enter an 'X' to");
      zg_glob2.put_new_line;
      zg_glob2.put_comment
	("   permit it to stop normally. ");
      zg_glob2.put_new_line;
    END start_problem;
 
    BEGIN
      Text_IO.get (zg_glob4.char);    -- this may hang up program!
    EXCEPTION
      WHEN OTHERS =>
        zg_glob2.put_new_line;
        zg_glob2.put_comment
		(" Exception was raised when background task executed a GET ");
        zg_glob2.put_new_line;
        zg_glob4.char := 'X';
    END;
 
    ACCEPT stop_problem;
 
    ACCEPT stop_program;
  END background;
 
  TASK BODY main IS
  BEGIN
 
 
    --
    -- Test Name         :  tk_la_async_04
    -- Prime Purpose     :  Perform 10_000 procedure calls while another task is
    --                      waiting on a terminal GET. Need to perform a
    --                      fairly long test problem to permit possible
    --                      overheads from polling to show up.
    -- LRM Version       :  A
    -- Optimization      :
    -- Related tests     :  tk_la_async_03
    -- Author            :  Thomas C. Leavitt
    -- Reviewer          :  Kermit E. Terrell
    -- Date              :
    -- Source            :
    -- Dependencies      :
    -- Other Information :
    --
 
    accept start_program ;

    background.initialization_complete;
 
    background.start_problem;
 
    ------------------------------------------------------------------------
    zg_glob2.put_test_name("tk_la_async_04");

    PRAGMA include("startime");

    FOR i IN 1 .. 10_000 LOOP
      zg_glob4.proc0;
    END LOOP;

    if zg_glob4.char = 'x' or zg_glob4.char = 'X' then 
      zg_glob2.put_error_code (zg_glob1.err_dependent_test);
      zg_glob2.put_new_line; 
      raise program_error; 
    end if;

    PRAGMA include("stoptime0");

    IF not ( zg_glob4.char = 'x' OR zg_glob4.char = 'X') THEN
      zg_glob2.put_comment_optional
	("   Perform 10_000 procedure calls while another task is waiting on");
      zg_glob2.put_new_line_optional;
      zg_glob2.put_comment_optional
	("   a terminal GET. Need to perform a fairly long test problem to");
      zg_glob2.put_new_line_optional;
      zg_glob2.put_comment_optional
	("   permit possible overheads from polling to show up. ");
      zg_glob2.put_new_line_optional;
    ELSE
      zg_glob2.put_comment
	("Implementation dependent test for support of async I/O ");
      zg_glob2.put_new_line;
      zg_glob2.put_error_code (zg_glob1.err_dependent_test);
    END IF;

    PRAGMA include("stoptime2");
    ------------------------------------------------------------------------
 
    zg_glob2.put_new_line;
    zg_glob2.put_comment(
       "  Enter any character other than 'X' to permit normal termination. ");
    zg_glob2.put_new_line(2);
    
    background.stop_problem;
 
    background.stop_program;
 
  END main;
END pkg01_tk_la02;
 
-----------------------------------
 
WITH pkg01_tk_la02;

PRAGMA elaborate(pkg01_tk_la02);

PROCEDURE tk_la_async_04 IS
BEGIN
  pkg01_tk_la02.main.start_program; 
END tk_la_async_04;
