-- +++ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
WITH zg_glob1;
USE  zg_glob1;
WITH zg_glob2;
USE  zg_glob2;
WITH zg_glob3;
USE  zg_glob3;
WITH zg_glob4;

WITH system;
WITH calendar;
USE  calendar;
 
PACKAGE pkg01_tk_la03 IS
 
  TASK resource IS
 
         --+ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
      PRAGMA priority( zg_glob1.priority_1 );
    ENTRY problem_complete;
    ENTRY start_problem;
    ENTRY program_complete;
    ENTRY initialization_complete;
  END resource;
 
END pkg01_tk_la03;
 
PACKAGE BODY pkg01_tk_la03 IS
 
  PRAGMA suppress(access_check);
  PRAGMA suppress(discriminant_check);
  PRAGMA suppress(index_check);
  PRAGMA suppress(length_check);
  PRAGMA suppress(range_check);
  PRAGMA suppress(division_check);
  PRAGMA suppress(overflow_check);
  PRAGMA suppress(elaboration_check);
  PRAGMA suppress(storage_check);
 
  TASK BODY resource IS
  BEGIN
    ACCEPT initialization_complete;    -- sync with task main
 
    LOOP
      SELECT
        ACCEPT program_complete;
        EXIT;
      OR
        ACCEPT start_problem;
        LOOP
          SELECT
            ACCEPT problem_complete;
            EXIT;
          ELSE
            zg_glob1.bool := true;	-- potentially modify bool
            zg_glob4.proc0;
            DELAY 0.0;			-- DELAY 0.0 forces a schedule point.
	    -- A "DELAY 0.0;" in a low priority task on a system without 
	    -- preemption will be sufficient to schedule any runnable
            -- high priority task. Without this statement, a system not 
	    -- supporting preemptive priority scheduling will
	    -- "hang up" in a infinite loop.  Since the LRM has been 
	    -- interpreted to require support for preemptive priority 
	    -- scheduling, this DELAY statement should not be required, but
	    -- its presence should do no harm.
          END SELECT;
        END LOOP;
      END SELECT;
    END LOOP;
  END resource;
END pkg01_tk_la03;
 
WITH zg_glob1;
USE  zg_glob1;
WITH zg_glob2;
USE  zg_glob2;
WITH zg_glob3;
USE  zg_glob3;
WITH zg_glob4;
USE  zg_glob4;

WITH direct_io;
WITH calendar;
USE  calendar;
WITH pkg01_tk_la03;
USE  pkg01_tk_la03;
WITH system;

PROCEDURE tk_la_async_05 IS
 
  PRAGMA suppress(access_check);
  PRAGMA suppress(discriminant_check);
  PRAGMA suppress(index_check);
  PRAGMA suppress(length_check);
  PRAGMA suppress(range_check);
  PRAGMA suppress(division_check);
  PRAGMA suppress(overflow_check);
  PRAGMA suppress(elaboration_check);
  PRAGMA suppress(storage_check);
 
  TASK main IS
         --+ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
      PRAGMA priority( zg_glob1.priority_2 );
  END main;
 
  TASK BODY main IS
    never, sometimes, always  : boolean;
 
    number_of_records_in_file : CONSTANT := 1_000;
 
    TYPE a_100 IS ARRAY ( zg_glob1.integer16'(1) .. zg_glob1.integer16'(100) ) 
						OF character;
    TYPE rec_100 IS RECORD
      r100 : a_100;
    END RECORD;
 
    PACKAGE dir_io IS NEW direct_io (rec_100);
    USE  dir_io;
 
    data_file : dir_io.file_type;
 
    temp_100  : rec_100;
 
    string6   : string(1 .. 6);
 
  BEGIN			-- create file with 1000 100 byte records
    dir_IO.create (data_file, dir_IO.out_file, "iodata1");
    FOR i IN zg_glob1.integer16'(1) .. 
				zg_glob1.integer16'(number_of_records_in_file) 
    LOOP
      dir_IO.set_index ( data_file, dir_io.positive_count(i) );
                        -- put count in record
                        --   put( string(temp_100.r100(1..6) ), i );  
			--+tcl 20-oct-89 review
      zg_glob1.Integer16_IO.Put (string6, i);
      FOR j IN zg_glob1.integer16'(1) .. zg_glob1.integer16'(6) 
      LOOP
        temp_100.r100(j) := string6(integer(j));
      END LOOP;
      dir_IO.write (data_file, temp_100);
    END LOOP;
    dir_IO.close (data_file);
 
 
    --
    -- Test Name         :  tk_la_async_05
    -- Prime Purpose     :  Test of asynchronous I/O. It has two tasks: a high
    --                      priority task which does 1000 READs from direct file
    --                      and a low priority task which set a boolean flag if
    --                      it ever gets control. After completion of the high
    --                      priority task, the state of the flag will let the
    --                      system know whether the low priority task ever got
    --                      control. The test problem monitors if the low
    --                      priority task got control: never, sometimes, or
    --                      always. 
    -- LRM Version       :  A
    -- Optimization      :
    -- Related tests     :  async*
    -- Author            :  Thomas C. Leavitt
    -- Reviewer          :  Kermit E. Terrell
    -- Date              :  4 October 1989
    -- Source            :
    -- Dependencies      :
    -- Other Information :
    --
    resource.initialization_complete;                -- entry call to sync with
                                                     -- task resource
 
    never := true;
    sometimes := false;
    always := true;
 
    dir_IO.open (data_file, in_file, "iodata1");
 
    ------------------------------------------------------------------------
    zg_glob2.put_test_name("tk_la_async_05");

    PRAGMA include("startime");

    resource.start_problem;

    FOR i IN zg_glob1.integer16'(1) .. 
			zg_glob1.integer16'(number_of_records_in_file) 
    LOOP
      zg_glob1.bool := false;
      zg_glob4.proc0;
      dir_IO.set_index (data_file, dir_io.positive_count(i) );
      dir_IO.read (data_file, temp_100);
      zg_glob4.string10 := string (temp_100.r100(1 .. 10) );    -- make live
      zg_glob4.proc0;
      sometimes := sometimes OR ( i > 1 AND (
		(never AND zg_glob1.bool) OR (always AND NOT zg_glob1.bool) ) );
      never := never AND NOT zg_glob1.bool;
      always := always AND zg_glob1.bool;
    END LOOP;
    resource.problem_complete;

    PRAGMA include("stoptime0");

    zg_glob2.put_comment_optional
	("   high priority task does 1000 reads from direct file in");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("   ascending order. Problem checks whether the system allows");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("   a low priority task to execute during this loop.  While ");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("   waiting for I/O completion in the high priority task, it may be");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("   possible to permit a low priority task to use the CUP. ");
    zg_glob2.put_new_line_optional;

    PRAGMA include("stoptime2");
    ------------------------------------------------------------------------
    dir_Io.close(data_file);
 
    IF always THEN
      zg_glob2.put_ancillary(" >>> tk_la_async_05 i/o is always performed");
      zg_glob2.put_new_line;
    END IF;
 
    IF sometimes THEN
      zg_glob2.put_ancillary(" >>> tk_la_async_05 i/o is sometimes performed");
      zg_glob2.put_new_line;
      zg_glob2.put_comment
	("   This is most likely to occur when the high priority ");
      zg_glob2.put_new_line;
      zg_glob2.put_comment
	("   task gives up control to perform a physical I/O ");
      zg_glob2.put_new_line;
      zg_glob2.put_comment
	("   operation but retains control as long as it can ");
      zg_glob2.put_new_line;
      zg_glob2.put_comment
	("   satisfy the record request from the current buffer in memory. ");
      zg_glob2.put_new_line;
    END IF;
 
    IF never THEN
      zg_glob2.put_ancillary(" >>> tk_la_async_05 i/o is never performed");
      zg_glob2.put_new_line;
    END IF;
 
    IF boolean'pos(always) + boolean'pos(sometimes) + boolean'pos(never) /= 1
    THEN
      zg_glob2.put_ancillary(" >>> tk_la_async_05 error in exclusion ");
      zg_glob2.put_new_line;
    END IF;
  
    -------------------------
    resource.program_complete;
  END main;
 
BEGIN
  NULL;
END tk_la_async_05;
