-- +++ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
WITH zg_glob1;
USE  zg_glob1;
WITH zg_glob2;
USE  zg_glob2;
WITH zg_glob3;
USE  zg_glob3;
WITH zg_glob4;

WITH calendar;
USE  calendar;
WITH system;
 
PROCEDURE tk_lb01 IS
 
  PRAGMA suppress(access_check);
  PRAGMA suppress(discriminant_check);
  PRAGMA suppress(index_check);
  PRAGMA suppress(length_check);
  PRAGMA suppress(range_check);
  PRAGMA suppress(division_check);
  PRAGMA suppress(overflow_check);
  PRAGMA suppress(elaboration_check);
  PRAGMA suppress(storage_check);
 
  abort_error         : EXCEPTION;
  dependent_exception : EXCEPTION;
 
  PROCEDURE d_abort1 IS
 
    PROCEDURE test_proc IS
 
      TASK tk_lf_task_01 IS
         --+ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
        PRAGMA priority( zg_glob1.priority_2 );
      END tk_lf_task_01;
 
      TASK tk_lf_task_02 IS
         --+ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
         PRAGMA priority( zg_glob1.priority_1 );
        ENTRY sync;
      END tk_lf_task_02;
 
      TASK BODY tk_lf_task_01 IS
      BEGIN
        tk_lf_task_02.sync;
        ABORT tk_lf_task_02;
      END tk_lf_task_01;
 
      TASK BODY tk_lf_task_02 IS
      BEGIN
        ACCEPT sync;
        zg_glob4.proc0;
        DELAY 0.0;
        RAISE abort_error;    -- should not get here if delay is sync point for
                              -- abort
      END tk_lf_task_02;
 
    BEGIN
      NULL;
    END test_proc;
 
  BEGIN          -- d_abort1
 
  --
  -- Test Name         :  tk_lb_delay_abort_01
  -- Prime Purpose     :  Tests whether the target system terminates an abnormal
  --                      task at a DELAY 0.0 statement. If the system does not
  --                      print an execution time error code for this statement,
  --                      the system is properly terminating the aborted tasks
  --                      before the DELAY 0.0 statement finishes.
  -- LRM Version       :  A
  -- Optimization      :
  -- Related tests     :
  -- Author            :  Thomas C. Leavitt
  -- Reviewer          :  Kermit E. Terrell
  -- Date              :
  -- Source            :
  -- Dependencies      :  Requires single processor system ensuring that the
  --			  high priority task executes before the low priority 
  --                      task.
  --                      A system with multiply processors would validly raise
  --                      the error indication by performing both tasks in
  --                      parallel.
  -- Other Information :
  --
 
  BEGIN    -- exception block
      zg_glob2.put_test_name("tk_lb_delay_abort_01");

      PRAGMA include("startime");

      test_proc;

      PRAGMA include("stoptime0");

      zg_glob2.put_comment_optional
         ("   The problem creates two tasks.  After they ");
      zg_glob2.put_new_line_optional;
      zg_glob2.put_comment_optional
         ("   synchronize, the high priority task will abort the low");
      zg_glob2.put_new_line_optional;
      zg_glob2.put_comment_optional
         ("   priority task and the low priority task will perform a ");
      zg_glob2.put_new_line_optional;
      zg_glob2.put_comment_optional
         ("   DELAY 0.0; and then raise an exception if the statement");
      zg_glob2.put_new_line_optional;
      zg_glob2.put_comment_optional
         ("   after the DELAY 0.0 is executed. The high priority task ");
      zg_glob2.put_new_line_optional;
      zg_glob2.put_comment_optional
         ("   will have made the low priority task abnormal and it should");
      zg_glob2.put_new_line_optional;
      zg_glob2.put_comment_optional
         ("   be terminated on a valid uniprocessor Ada system. On a ");
      zg_glob2.put_new_line_optional;
      zg_glob2.put_comment_optional
         ("   multiprocessor, it could be valid to execute the statement ");
      zg_glob2.put_new_line_optional;
      zg_glob2.put_comment_optional
         ("   after the DELAY 0.0, depending on the relative timings of ");
      zg_glob2.put_new_line_optional;
      zg_glob2.put_comment_optional
         ("   the ABORT statement and the DELAY 0.O statement. ");
      zg_glob2.put_new_line_optional;

      PRAGMA include("stoptime2");

    EXCEPTION
      WHEN tasking_error | abort_error =>
        zg_glob2.put_comment_optional
	         ("   error: System is not terminating an abnormal task ");
        zg_glob2.put_new_line_optional;
        zg_glob2.put_comment_optional
	         ("   at a DELAY 0.0 statement. ");
        zg_glob2.put_new_line_optional;
        zg_glob2.put_comment_optional
	         ("   For a uniprocessor target, failure indicates an error ");
        zg_glob2.put_new_line_optional;
        zg_glob2.put_comment_optional
	         ("   not an implementation dependent problem. However, the ");
        zg_glob2.put_new_line_optional;
        zg_glob2.put_comment_optional
	         ("   condition tested for in this problem is not an error ");
        zg_glob2.put_new_line_optional;
        zg_glob2.put_comment_optional
	         ("   condition on a multiprocessor target. ");
        zg_glob2.put_new_line_optional;
        zg_glob2.put_error_code (zg_glob1.err_dependent_test);
      WHEN OTHERS =>
        zg_glob2.put_comment_optional("   error: OTHERS ");
        zg_glob2.put_new_line_optional;
        zg_glob2.put_error_code (zg_glob1.err_at_execution_time);
    END;         -- exception block
 
  END d_abort1;
 
  PROCEDURE d_abort2 IS
    state : zg_glob1.integer16 := 0;
 
    PROCEDURE test_proc IS
      TASK tk_lf_task_01 IS
         --+ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
        PRAGMA priority( zg_glob1.priority_1 );
      END tk_lf_task_01;
 
      TASK tk_lf_task_02 IS
         --+ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
        PRAGMA priority( zg_glob1.priority_2 );
        ENTRY sync;
      END tk_lf_task_02;
 
      TASK BODY tk_lf_task_01 IS
      BEGIN
        tk_lf_task_02.sync;
        ABORT tk_lf_task_02;
      END tk_lf_task_01;
 
      TASK BODY tk_lf_task_02 IS
      BEGIN
        ACCEPT sync;
        state := 1;
        DELAY 1.0;
        state := 2;
        DELAY 0.0;
        RAISE abort_error;    -- should not get here if delay is
      END tk_lf_task_02;              -- sync point for abort
 
    BEGIN
      NULL;
    END test_proc;
 
  BEGIN          -- d_abort2
 
  --
  -- Test Name         :  tk_lb_delay_abort_02
  -- Prime Purpose     :  Test whether a system immediately kill a task which
  --                      is aborted while in the middle of a DELAY or whether
  --                      it waits until the DELAY completes.
  -- LRM Version       :  A
  -- Optimization      :
  -- Related tests     :
  -- Author            :  Thomas C. Leavitt
  -- Reviewer          :  Kermit E. Terrell
  -- Date              :
  -- Source            :
  -- Dependencies      :  This test problem is implementation dependent.
  --                      Comparison between systems are defined only for
  --                      systems which terminate an aborted task which is
  --                      waiting in a delay statement "immediately" and does
  --                      not wait until an outstanding DELAY expires.
  -- Other Information :
  --
 
  BEGIN    -- exception block
      zg_glob2.put_test_name("tk_lb_delay_abort_02");

      PRAGMA include("startime");

      test_proc;
      IF state = 2 THEN
        RAISE dependent_exception;
      END IF;

      PRAGMA include("stoptime0");

      zg_glob2.put_comment_optional
         ("   This test problem aborts a task waiting in a ");
      zg_glob2.put_new_line_optional;
      zg_glob2.put_comment_optional
         ("   DELAY.  Does the system immediately terminate it or does it ");
      zg_glob2.put_new_line_optional;
      zg_glob2.put_comment_optional
         ("   wait until the DELAY completes? This is implementation ");
      zg_glob2.put_new_line_optional;
      zg_glob2.put_comment_optional
         ("   dependent - either approach is valid.  This test problem ");
      zg_glob2.put_new_line_optional;
      zg_glob2.put_comment_optional
         ("   only reports valid measurements for systems which implement");
      zg_glob2.put_new_line_optional;
      zg_glob2.put_comment_optional
         ("   an ""immediate"" termination of an aborted task which is ");
      zg_glob2.put_new_line_optional;
      zg_glob2.put_comment_optional
         ("   waiting on a DELAY. ");
      zg_glob2.put_new_line_optional;

      PRAGMA include("stoptime2");

    EXCEPTION
      WHEN tasking_error | abort_error =>
        zg_glob2.put_comment_optional
	         ("   error: System is not terminating an abnormal task ");
        zg_glob2.put_new_line_optional;
        zg_glob2.put_comment_optional("      at a DELAY 0.0 statement. ");
        zg_glob2.put_new_line_optional;
        zg_glob2.put_error_code (zg_glob1.err_at_execution_time);
      WHEN dependent_exception =>
        zg_glob2.put_comment_optional
	         ("   implementation dependency: System is not ");
        zg_glob2.put_new_line_optional;
        zg_glob2.put_comment_optional
	         ("   terminating an abnormal task until DELAY expires.");
        zg_glob2.put_new_line_optional;
        zg_glob2.put_comment_optional
	         ("   For comparisons between systems, the test problem");
        zg_glob2.put_new_line_optional;
        zg_glob2.put_comment_optional
	         ("   is intended to measure execution time when the ");
        zg_glob2.put_new_line_optional;
        zg_glob2.put_comment_optional
	         ("   delayed task is immediately terminated. Because it");
        zg_glob2.put_new_line_optional;
        zg_glob2.put_comment_optional
	         ("   is NOT an error for a system to implement abortion");
        zg_glob2.put_new_line_optional;
        zg_glob2.put_comment_optional
	         ("   by waiting for all delays to expire, this is an ");
        zg_glob2.put_new_line_optional;
        zg_glob2.put_comment_optional
	         ("   implementation dependent test problem and not a failure");
        zg_glob2.put_new_line_optional;
        zg_glob2.put_new_line_optional;
        zg_glob2.put_error_code (zg_glob1.err_dependent_test);
      WHEN OTHERS =>
        zg_glob2.put_comment_optional("   error: OTHERS ");
        zg_glob2.put_error_code (zg_glob1.err_at_execution_time);
    END;         -- exception block
 
    CASE state IS
      WHEN 1 =>
        zg_glob2.put_ancillary (" >>> tk_lb_delay_abort_02 system kills" &
	   " aborted task inside a DELAY immediately");
        zg_glob2.put_new_line;
      WHEN 2 =>
        zg_glob2.put_ancillary (" >>> tk_lb_delay_abort_02 system waits" &
           " until DELAY completes to kill an aborted task");
        zg_glob2.put_new_line;
      WHEN OTHERS =>
        zg_glob2.put_ancillary (" >>> tk_lb_delay_abort_02 strange behavior. ");
        zg_glob2.put_new_line;
    END CASE;
 
  END d_abort2;
 
BEGIN    -- of d_abort
 
  d_abort1;
  d_abort2;
 
END tk_lb01 ;
