-- +++ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
--+ KET 9/14/89 modified to correct Eilers #7
-- +++ tcl 2-8-89 change to make ancillary information self-identifying
 
PACKAGE pkg01_tk_lf04 IS
  SUBTYPE image IS string(1 .. 10);
  x1 : image := "abcdefghij";
  x2 : image := "0123456789";
  y  : image;
END pkg01_tk_lf04;
 
--  tk_lf_task_04 rendezvous with parameters
--  from "Task Management in Ada," by Roberts,Evans and Morgan in Software:
--  Practice and Experience, Volume 11 #10

WITH zg_glob1;
USE  zg_glob1;
WITH zg_glob2;
USE  zg_glob2;
WITH zg_glob3;
USE  zg_glob3;

WITH calendar;
USE  calendar;
WITH system;

WITH pkg01_tk_lf04;
USE  pkg01_tk_lf04;
 
PROCEDURE tk_lf_task_04 IS
 
  PRAGMA suppress(access_check);
  PRAGMA suppress(discriminant_check);
  PRAGMA suppress(index_check);
  PRAGMA suppress(length_check);
  PRAGMA suppress(range_check);
  PRAGMA suppress(division_check);
  PRAGMA suppress(overflow_check);
  PRAGMA suppress(elaboration_check);
  PRAGMA suppress(storage_check);
 
  TASK buffer IS
         --+ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
      PRAGMA priority( zg_glob1.priority_1 );
    ENTRY read  ( x : OUT pkg01_tk_lf04.image );
    ENTRY write ( x : IN  pkg01_tk_lf04.image );
  END buffer;
 
  TASK BODY buffer IS
 
    n      : CONSTANT         := 10;
    buffer : ARRAY ( zg_glob1.integer16'(1) .. zg_glob1.integer16'(n) ) OF 
							pkg01_tk_lf04.image;
    i, j   : zg_glob1.integer16 RANGE 1 .. n := 1;
    c      : zg_glob1.integer16 RANGE 0 .. n := 0;
 
  BEGIN
    LOOP
      SELECT
        WHEN c > 0 =>
          ACCEPT read(x : OUT pkg01_tk_lf04.image) DO
            x := buffer(j);
          END read;
          j := j MOD n + 1;
          c := c - 1;
      OR
        WHEN c < n =>
          ACCEPT write(x : IN pkg01_tk_lf04.image) DO
            buffer(i) := x;
          END write;
          i := i MOD n + 1;
          c := c + 1;
      END SELECT;
    END LOOP;
  END buffer;
 
  TASK main IS
         --+ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
      PRAGMA priority( zg_glob1.priority_2 );
  END main;
 
  TASK BODY main IS
  BEGIN
 
 
    --
    -- Test Name         :  tk_lf_task_04
    -- Prime Purpose     :  Language feature test of aspects of tasking.  
    --                      Passes a nonscalar parameter.  
    --                      Conditional select statement with WHEN clause.
    -- LRM Version       :  A
    -- Optimization      :
    -- Related Tests     :
    -- Author            :  Thomas C. Leavitt
    -- Reviewer          :  Kermit E. Terrell
    -- Date              :
    -- Source            :  "Task Management in Ada," by Roberts, Evans and
    -- Morgan, Software:  Practice and Experience, Volume 11,
    --                          number 10.
    -- Dependencies      :
    -- Other Information :
    --
 
    --+ KET 9/14/89 modified to correct Eilers #7
 
    ----------------------------------------------------------------------------
    zg_glob2.put_test_name("tk_lf_task_04");

    PRAGMA include("startime");

    buffer.write ( pkg01_tk_lf04.x1 );
    buffer.write ( pkg01_tk_lf04.x2 );
    buffer.read  ( pkg01_tk_lf04.y );
    buffer.read  ( pkg01_tk_lf04.y );

    PRAGMA include("stoptime0");

    zg_glob2.put_comment_optional("   bounded buffer");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional("   buffer.write ( x1) ;");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional("   buffer.write ( x2 );");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional("   buffer.read ( y );");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional("   buffer.read ( y );");

    PRAGMA include("stoptime2");
    ----------------------------------------------------------------------------
 
		----------------------------------------------------
		-- +++ tcl 2-8-89 make ancillary information self-identifying
    zg_glob2.put_ancillary(" >>> tk_lf_task_04 Time per rendezvous = ");
    zg_glob2.put_value ( (zg_glob3.min_time * 1000000.0 / 4.0), 8, 1, 0);
    zg_glob2.put_new_line;
		----------------------------------------------------
 
    ABORT buffer;    -- must abort buffer to clean up
  END main;
 
BEGIN    -- main program
  NULL;
END tk_lf_task_04;
