-- +++ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
-- +++ tcl 2-8-89 change to make ancillary information self-identifying
--  tk_lf_task_13
--  modeled after EXAMPLE3 in Task Management in Ada, E. Roberts, A. Evans,
--  C. Morgan, and E. Clarke in Software: Practice and Experience, Volume 11,
--  Number 10, Oct 1981.
 
--  simple_task can be Habermann/Nassi-ized
--  However, it has code within the body of the task which is not enclosed in
--  an accept statement. This requires a slightly more complex treatment that
--  forces the server task to maintain an independent thread of control.
 
WITH zg_glob1;
USE  zg_glob1;
WITH zg_glob2;
USE  zg_glob2;
WITH zg_glob3;
USE  zg_glob3;
WITH zg_glob4;
USE  zg_glob4;

WITH calendar;
USE  calendar;
WITH system;
 
PROCEDURE tk_lf_task_13 IS
 
  PRAGMA suppress(access_check);
  PRAGMA suppress(discriminant_check);
  PRAGMA suppress(index_check);
  PRAGMA suppress(length_check);
  PRAGMA suppress(range_check);
  PRAGMA suppress(division_check);
  PRAGMA suppress(overflow_check);
  PRAGMA suppress(elaboration_check);
  PRAGMA suppress(storage_check);
 
  TASK simple_task IS
         --+ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
      PRAGMA priority( zg_glob1.priority_2 );
    ENTRY entry1;
    ENTRY entry2;
    ENTRY entry3;
    ENTRY entry4;
    ENTRY entry5;
  END simple_task;
 
  TASK BODY simple_task IS
  BEGIN
    LOOP
      zg_glob4.ii := 0;
      zg_glob4.proci1(zg_glob4.ii);
      ACCEPT entry1 DO
        zg_glob4.ii := 1;
        zg_glob4.proc0;
      END entry1;
      zg_glob4.proci1(zg_glob4.ii);
      ACCEPT entry2 DO
        zg_glob4.ii := 2;
        zg_glob4.proc0;
      END entry2;
      zg_glob4.proci1(zg_glob4.ii);
      ACCEPT entry3 DO
        zg_glob4.ii := 3;
        zg_glob4.proc0;
      END entry3;
      zg_glob4.proci1(zg_glob4.ii);
      ACCEPT entry4 DO
        zg_glob4.ii := 4;
        zg_glob4.proc0;
      END entry4;
      zg_glob4.proci1(zg_glob4.ii);
      ACCEPT entry5 DO
        zg_glob4.ii := 5;
        zg_glob4.proc0;
      END entry5;
      zg_glob4.proci1(zg_glob4.ii);
    END LOOP;
  END simple_task;
 
  TASK main IS
         --+ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
      PRAGMA priority( zg_glob1.priority_1 );
  END main;
 
  TASK BODY main IS
  BEGIN
 
 
    --
    -- Test Name         :  tk_lf_task_13
    -- Prime Purpose     :  language feature test of aspects of tasking.
    --                      Entering tasks make a sequence of entry calls.
    --                      Accepting task contains a sequence of ACCEPTs with
    --                      DO containing a simple code sequence. Some code
    --                      outside of the rendezvous. Accepting task will
    --                      arrive at rendezvous first.
    -- LRM Version       :  A
    -- Optimization      :  Habermann/Nassi
    -- Related tests     :
    -- Author            :  Thomas C. Leavitt
    -- Reviewer          :  Kermit E. Terrell
    -- Date              :
    -- Source            :  "Task Management in Ada," by E. Roberts, A. Evans,
    --                         C. Morgan, and E. Clarke, Software:  Practice and
    --                         Experience, volume 11, number 10, October 1981.
    -- Dependencies      :
    -- Other Information :  Modeled after Example3 in the above citation.  It
    --                      has code within the body of the task which is not
    --                      enclosed in an accept statement.  This requires a
    --                      slightly more complex treatment that forces the
    --                      server task to maintain an independent thread of
    --                      control. 
 
    ----------------------------------------------------------------------------
    zg_glob2.put_test_name("tk_lf_task_13");

    PRAGMA include("startime");

    simple_task.entry1;
    simple_task.entry2;
    simple_task.entry3;
    simple_task.entry4;
    simple_task.entry5;

    PRAGMA include("stoptime0");

    zg_glob2.put_comment_optional
	("   simple_task.entry1;simple_task.entry2;simple_task.entry3;");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("   simple_task.entry4;simple_task.entry5;");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("   -- sequence of accepts in receiving task");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("   -- with some code not enclosed in an accept statement");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("   -- priority is such that accepting task will arrive at");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("   -- rendezvous first");
    zg_glob2.put_new_line_optional;

    PRAGMA include("stoptime2");
    ----------------------------------------------------------------------------
 
		----------------------------------------------------
		-- +++ tcl 2-8-89 make ancillary information self-identifying
    zg_glob2.put_ancillary(" >>> tk_lf_task_13 time per rendezvous = ");
    zg_glob2.put_value( (zg_glob3.min_time * 1000000.0 / 5.0), 8, 1, 0);
    zg_glob2.put_new_line;
                ----------------------------------------------------
 
    ABORT simple_task;    -- must stop "simple_task" to permit main program to
                          -- terminate
  END main;
 
BEGIN
  NULL;
END tk_lf_task_13;
