-- +++ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
WITH zg_glob1;
USE  zg_glob1;
WITH zg_glob2;
USE  zg_glob2;
WITH zg_glob3;
USE  zg_glob3;

WITH calendar;
USE  calendar;
WITH system;
 
PROCEDURE tk_lf18 IS
 
  PRAGMA suppress(access_check);
  PRAGMA suppress(discriminant_check);
  PRAGMA suppress(index_check);
  PRAGMA suppress(length_check);
  PRAGMA suppress(range_check);
  PRAGMA suppress(division_check);
  PRAGMA suppress(overflow_check);
  PRAGMA suppress(elaboration_check);
  PRAGMA suppress(storage_check);
 
  skip_print : exception;

  problem_start_time : time;
 
  PROCEDURE proca IS
    t4_has_run : boolean := false;    -- signal to stop
 
    TASK t4 IS
         --+ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
      PRAGMA priority( zg_glob1.priority_2 );
    END t4;
 
    TASK t1 IS
         --+ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
      PRAGMA priority( zg_glob1.priority_1 );
    END t1;
 
    TASK BODY t1 IS
    BEGIN
      LOOP                                             -- background loop that
                                                       -- runs continually
        EXIT WHEN t4_has_run;
 
        --  If the problem takes more than 10 minutes,
        --  the task scheduler is not preemptive.
        --  Abort; print an error message.
        IF clock > problem_start_time + 600.0 THEN
          zg_glob2.put_test_name("tk_lf_task_45b");
          zg_glob2.put_comment_optional("   DELAY 0.010; ");
          zg_glob2.put_new_line_optional;
          zg_glob2.put_comment_optional("   t3.call3;    -- simple entry call");
          zg_glob2.put_new_line_optional;
          zg_glob2.put_comment_optional
		("   timing an entry call by a higher priority task ");
          zg_glob2.put_new_line_optional;
          zg_glob2.put_comment_optional
		("   --   which becomes eligible to run while ");
          zg_glob2.put_new_line_optional;
          zg_glob2.put_comment_optional
		("   --   lower priority task is executing ");
          zg_glob2.put_new_line_optional;
          zg_glob2.put_comment_optional("   ERROR:  Nonpreemptive");
          zg_glob2.put_new_line_optional;
          zg_glob2.put_error_code (zg_glob1.err_at_execution_time);--+tcl review

          zg_glob2.put_test_name("tk_lf_task_45a");
          zg_glob2.put_error_code (zg_glob1.err_at_execution_time);--+tcl review

          ABORT t4;                               -- t3 should terminate itself
          EXIT;                                   -- this will terminate t1
        END IF;
      END LOOP;
    END t1;
 
    TASK t3 IS
         --+ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
      PRAGMA priority( zg_glob1.priority_2 );
      ENTRY call3;
    END t3;
 
    TASK BODY t3 IS
    BEGIN
      LOOP
        SELECT
          ACCEPT call3;
        OR
          TERMINATE;
        END SELECT;
      END LOOP;
    END t3;
 
    TASK BODY t4 IS
    BEGIN
 
    ----------------------------------------------------------------------------
    --
    -- Test Name         :  tk_lf_task_45a
    -- Prime Purpose     :  Higher priority task becoming eligible to run 
    --                      during running of lower priority task.
    -- LRM Version       :  Revision A
    -- Optimization      :
    -- Related tests     :  tk_lf_task_44a , tk_lf_task_44b
    -- Author            :  Kermit Terrell
    -- Reviewer          :  T. Leavitt, 12-oct-89
    -- Date              :  22 July 1987
    -- Source            :  
    -- Dependencies      :  On systems which do not do pre-emptive scheduling,
    --                      this task may terminate with an error message.
    -- Other Information :  This is a DELAY problem. The execution time will not
    --                      be proportional to a systems factor.
    ----------------------------------------------------------------------------
      zg_glob2.put_test_name("tk_lf_task_45a");

      begin
      PRAGMA include("startime");

      FOR i IN 1 .. 10 LOOP
        DELAY 0.010;
      END LOOP;

      PRAGMA include("stoptime0");

      zg_glob2.put_comment_optional("  DELAY 0.010; ");
      zg_glob2.put_new_line_optional;
      zg_glob2.put_comment_optional
	("   -- use this time to subtract out delay time from 45b");
      zg_glob2.put_new_line_optional;

      if zg_glob3.verification_success then
        standard_dev( timing_vector(1..mcount), timing_mean, timing_sigma );
        zg_glob2.put_measurement (
          result_kind       => zg_glob2.test_result,
          size              => abs( float6(system_storage_unit) *
                                    expansion_size),
          minimum_time      => delay_problem / 1_000_000.0 ,
          mean_time         => timing_mean,
          inner_count_lower => ncount_lower,
          inner_count_upper => ncount_upper,
          outer_count       => mcount,
          sigma             => timing_sigma,
          unreliable_flag   => false );
        zg_glob2.put_ancillary(
        " >>> tk_lf_task_45a Time for ten 10-millisecond delays is ");
        zg_glob2.put_value( timing_mean * 1_000_000.0, 6, 1, 0 );
        zg_glob2.put_comment(" microseconds");
        zg_glob2.put_new_line;
        raise skip_print;
      end if;
      PRAGMA include("stoptime2");
      exception when skip_print => null ;
      end;

    ----------------------------------------------------------------------------
  
      ----------------------------------------------------
      zg_glob2.put_ancillary(" >>> tk_lf_task_45a Time per rendezvous = ");
      zg_glob2.put_value( (zg_glob3.min_time * 1000000.0 / 20.0), 8, 1, 0);
      zg_glob2.put_new_line;
    ----------------------------------------------------------------------------
    --
    -- Test Name         :  tk_lf_task_45b
    -- Prime Purpose     :  Higher priority task becoming eligible to run 
    --                      during running of lower priority task.
    -- LRM Version       :  Revision A
    -- Optimization      :
    -- Related tests     :  tk_lf_task_44a , tk_lf_task_44b
    -- Author            :  Kermit Terrell
    -- Reviewer          :  T. Leavitt, 12-oct-89
    -- Date              :  22 July 1987
    -- Source            :  
    -- Dependencies      :  On systems which do not do pre-emptive scheduling,
    --                      this task may terminate with an error message.
    -- Other Information :  This is a DELAY problem. The execution time will not
    --                      be proportional to a systems factor.
    ----------------------------------------------------------------------------
      zg_glob2.put_test_name("tk_lf_task_45b");

      begin
      PRAGMA include("startime");

      FOR i IN 1 .. 10 LOOP
        DELAY 0.010;
        t3.call3;
      END LOOP;

      PRAGMA include("stoptime0");

      zg_glob2.put_comment_optional("   DELAY 0.010; ");
      zg_glob2.put_new_line_optional;
      zg_glob2.put_comment_optional("   t3.call3;    -- simple entry call");
      zg_glob2.put_new_line_optional;
      zg_glob2.put_comment_optional
	("   -- timing an entry call by a higher priority task ");
      zg_glob2.put_new_line_optional;
      zg_glob2.put_comment_optional
	("   --   which becomes eligible to run while ");
      zg_glob2.put_new_line_optional;
      zg_glob2.put_comment_optional
	("   --   lower priority task is executing ");
      zg_glob2.put_new_line_optional;

      if zg_glob3.verification_success then
        standard_dev( timing_vector(1..mcount), timing_mean, timing_sigma );
        zg_glob2.put_measurement (
          result_kind       => zg_glob2.test_result,
          size              => abs( float6(system_storage_unit)*expansion_size),
          minimum_time      => delay_problem / 1_000_000.0 ,
          mean_time         => timing_mean,
          inner_count_lower => ncount_lower,
          inner_count_upper => ncount_upper,
          outer_count       => mcount,
          sigma             => timing_sigma,
          unreliable_flag   => false );
        zg_glob2.put_ancillary(
       " >>> tk_lf_task_45b Time for ten 10-millisecond delays and a call is ");
        zg_glob2.put_value( timing_mean * 1_000_000.0, 6, 1, 0 );
        zg_glob2.put_comment(" microseconds");
        zg_glob2.put_new_line;
        raise skip_print;
      end if;
      PRAGMA include("stoptime2");
      exception when skip_print => null ;
      end;
    ----------------------------------------------------------------------------
  
      t4_has_run := true;                  -- will terminate t1
    END t4;
 
  BEGIN
    NULL;
  END proca;
 
BEGIN
  zg_glob2.put_comment("   On systems which do not do pre-emptive scheduling,");
  zg_glob2.put_new_line;
  zg_glob2.put_comment("   this task may terminate with an error message.");
  zg_glob2.put_new_line;
 
  problem_start_time := clock;
  proca;
END tk_lf18 ;
