-- +++ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
-- tk_lf_task_51 creation/termination of task as a record component
--+ BDL 2 JAN 1991 Modified to correct PCR#82, deallocation of record with task
WITH system;                 --+ BDL 2 JAN 1991 added for PCR#82    
WITH unchecked_deallocation; --+ BDL 2 JAN 1991 added for PCR#82    

WITH zg_glob1;
USE  zg_glob1;
WITH zg_glob2;
USE  zg_glob2;
WITH zg_glob3;
USE  zg_glob3;
WITH zg_glob4;
USE  zg_glob4;

WITH calendar;
USE  calendar;
 
PROCEDURE tk_lf_task_51 IS
 
  PRAGMA suppress(access_check);
  PRAGMA suppress(discriminant_check);
  PRAGMA suppress(index_check);
  PRAGMA suppress(length_check);
  PRAGMA suppress(range_check);
  PRAGMA suppress(division_check);
  PRAGMA suppress(overflow_check);
  PRAGMA suppress(elaboration_check);
  PRAGMA suppress(storage_check);
 
  TASK TYPE minimal_task IS 
         --+ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
      PRAGMA priority( zg_glob1.priority_2 );
                                           --+ BDL 2 JAN 1991 added for PCR#82 
                                           --+     Make created task high
                                           --+     priority so that it should
                                           --+     run to completion before
                                           --+     the creating task is 
                                           --+     reactivated.
  END minimal_task;
 
  TYPE record_with_task IS RECORD
    short_lived : minimal_task;
  END RECORD;
 
  TASK BODY minimal_task IS
  BEGIN
    --+ changed null body to the following: KET, 19 Oct 1989 
    IF zg_glob3.boolean_false		   -- always false
    THEN    
      zg_glob4.proc0;
    END IF;
  END minimal_task;
 
  TASK main IS
         --+ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
      PRAGMA priority( zg_glob1.priority_1 );
         --+ BDL 2 JAN 1991 added for PCR#82 
  END main;
 
  TASK BODY main IS
    TYPE task_ref IS ACCESS record_with_task;
    short_life : task_ref;

    --+ BDL 2 JAN 1991 added for PCR#82 
    procedure free_record_with_task is new unchecked_deallocation (   
                                                  record_with_task, task_ref);  
BEGIN
 
 
  --
  -- Test Name         :  tk_lf_task_51
                      --+ BDL 2 JAN 1991 Purpose expanded on to discuss possible
                      --+ memory non-reclamation
  -- Prime Purpose     :  To dynamically allocate a new record which contains a
  --                      task of higher priority than the allocating task. 
  --                      The created task evaluates an if statement condition. 
  --                      The problem explicitly deallocates the record 
  --                      containing the embedded task type to avoid exhausting
  --                      available memory.  However, a system might not free
  --                      all the space associated with task when it releases
  --                      the record (and the task completes). Systems have
  --                      been observed where the Task Control Block (TCB) 
  --                      was not deallocated, leading to possible exhaustion
  --                      of memory. This problem is not primarily intended as 
  --                      a correctness test, but it can provoke runtime 
  --                      failures.  It is intended to test task creation and 
  --                      termination when a task is a component of a record.  
  --                      The test CLAIM49 is an explicit test for possible 
  --                      non-reclamation of task control blocks. 
  -- LRM Version       :  Revision A
  -- Optimization      :
  -- Related tests     :  CLAIM49
  -- Author            :  Thomas C. Leavitt
  -- Reviewer          :  Kermit E. Terrell
  -- Date              :
  -- Source            :
                          --+ BDL 2 JAN 1991 PCR #82
  -- Dependencies      :  On systems which do not reclaim space associated 
  --                      with tasks when the task completes, whether this
  --                      test problem runs to completion or exhausts memory
  --                      is dependent on the amount of available memory.
  -- Other Information :
  --
  -----------------------------------------------------------------------
  -- Revisions       : For ACEC release 2.1 corrected PCR#82 by specifying 
  --                   priorities for tasks and adding code to deallocate 
  --                   the record with a task element. This should not 
  --                   invalidate comparisons with version 2.0 of this 
  --                   program.
  -- Author          : B. Decker-Lindsey
  -- Reviewer        : T. Leavitt 
  -- Date            : 2 JAN 1991

  ----------------------------------------------------------------------------
  zg_glob2.put_test_name("tk_lf_task_51");

  PRAGMA include("startime");

  FOR i IN 1 .. 10 LOOP
    BEGIN
      short_life := NEW record_with_task;
      free_record_with_task (short_life);  --+ BDL 2 JAN 1991 added for PCR#82 
    EXCEPTION                              --+ BDL 2 JAN 1991 added for PCR#82 
      WHEN others =>                       --  output error code on failure
        zg_glob2.put_comment
    	  ("   creation/termination of task as a record component");
        zg_glob2.put_new_line;
        zg_glob2.put_comment
	  ("   Exception raised.  Likely that space is not reclaimed");
        zg_glob2.put_new_line;
        zg_glob2.put_error_code (zg_glob1.err_at_execution_time);
        RAISE;
    END;
  END LOOP;

  PRAGMA include("stoptime0");

  zg_glob2.put_comment_optional
	("   creation/termination of task as a record component");
  zg_glob2.put_new_line_optional;

  PRAGMA include("stoptime2");
  ----------------------------------------------------------------------------
 
END main;
 
BEGIN
  NULL;
END tk_lf_task_51;
