-- +++ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
--+ KET 9/14/89 modified to correct Eilers #7
 
-- tk_lf_task_52 simple rendezvous on a record component

WITH zg_glob1;
USE  zg_glob1;
WITH zg_glob2;
USE  zg_glob2;
WITH zg_glob3;
USE  zg_glob3;

WITH calendar;
USE  calendar;
WITH system;
 
PROCEDURE tk_lf_task_52 IS
 
  PRAGMA suppress(access_check);
  PRAGMA suppress(discriminant_check);
  PRAGMA suppress(index_check);
  PRAGMA suppress(length_check);
  PRAGMA suppress(range_check);
  PRAGMA suppress(division_check);
  PRAGMA suppress(overflow_check);
  PRAGMA suppress(elaboration_check);
  PRAGMA suppress(storage_check);
 
  TASK TYPE minimal_task IS 
         --+ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
      PRAGMA priority( zg_glob1.priority_1 );
  ENTRY request;
  ENTRY release;
END minimal_task;
 
TYPE record_with_task IS RECORD
  task_component : minimal_task;
END RECORD;
 
TASK BODY minimal_task IS
BEGIN
  LOOP
    ACCEPT request;
    ACCEPT release;
  END LOOP;
END minimal_task;
 
TASK main IS
         --+ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
      PRAGMA priority( zg_glob1.priority_2 );
END main;
 
TASK BODY main IS
  TYPE task_ref IS ACCESS record_with_task;
 
  --+ KET 9/14/89 modified to correct Eilers #7
  dynamic_record : CONSTANT task_ref := NEW record_with_task;
 
BEGIN
 
 
  --
  -- Test Name         :  tk_lf_task_52
  -- Prime Purpose     :  To measure the time for a simple rendezvous with a
  --                      task created as a component of a dynamically 
  --                      allocated record.  The time of task 
  --                      creation/termination is excluded.
  -- LRM Version       :  Revision A
  -- Optimization      :
  -- Related tests     :
  -- Author            :  Thomas C. Leavitt
  -- Reviewer          :  Kermit E. Terrell
  -- Date              :
  -- Source            :
  -- Dependencies      :
  -- Other Information :
  --
 
  ----------------------------------------------------------------------------
  zg_glob2.put_test_name("tk_lf_task_52");

  PRAGMA include("startime");

  FOR i IN 1 .. 10 LOOP
    dynamic_record.task_component.request;
    dynamic_record.task_component.release;
  END LOOP;

  PRAGMA include("stoptime0");

  zg_glob2.put_comment_optional("   simple rendezvous on a record component");
  zg_glob2.put_new_line_optional;
  zg_glob2.put_comment_optional("   FOR i IN 1..10 ");
  zg_glob2.put_new_line_optional;
  zg_glob2.put_comment_optional("   LOOP");
  zg_glob2.put_new_line_optional;
  zg_glob2.put_comment_optional("      dynamic_record.task_component.request;");
  zg_glob2.put_new_line_optional;
  zg_glob2.put_comment_optional("      dynamic_record.task_component.release;");
  zg_glob2.put_new_line_optional;
  zg_glob2.put_comment_optional("   END loop;");
  zg_glob2.put_new_line_optional;
  zg_glob2.put_comment_optional("   -- priorities ensure task making entry");
  zg_glob2.put_new_line_optional;
  zg_glob2.put_comment_optional("   -- call will arrive at rendezvous first");
  zg_glob2.put_new_line_optional;
  zg_glob2.put_comment_optional("   -- compare with tk_lf_task_03");
  zg_glob2.put_new_line_optional;

  PRAGMA include("stoptime2");
  ----------------------------------------------------------------------------
 
			----------------------------------------------------
  zg_glob2.put_ancillary(" >>> tk_lf_task_52 time per rendezvous = ");
  zg_glob2.put_value( (zg_glob3.min_time * 1000000.0 / 20.0), 8, 1, 0);
  zg_glob2.put_new_line;
			----------------------------------------------------
 
  ABORT dynamic_record.task_component;    -- stop execution
END main;
 
BEGIN
  NULL;
END tk_lf_task_52;
