
    -- DELAY_TS  Determine task-switch time associated with DELAY expiration.
    --           Must be indirectly measured because is it not feasible to use
    --           the normal ACES timing loop to isolate the feature (when the
    --           DELAY is executing, the system may be idle).
    --           Verify that preemptive priority scheduling occurs


     --------------       IMPLEMENTATION DEPENDENT MODIFICATION MAY BE NEEDED
     --------------                 TO ADAPT VALUE OF "SMALL_DURATION"

     --------------       REQUIRES PREEMPTIVE PRIORITY SCHEDULING
     --------------           DELAY EXPIRATION MUST CAUSE SCHEDULING 


with zg_glob1;
use  zg_glob1;
with zg_glob2;
with zg_glob3;
with zg_glob4;
with system;
pragma elaborate(zg_glob1, zg_glob2, zg_glob3, zg_glob4 );
package tk_lfpkg_60 is

  zero_duration : duration := duration(zg_glob1.zero);

        -- <>                    
  small_duration: duration := duration'(0.010_000);      -- 10 milliseconds 
        -- MAY NEED TO ADAPTED TO THE SMALLEST NON-NULL DELAY VALUE        
        -- HOWEVER, PROBLEM DELAY_TS3 WILL EXECUTE FOR "NUMBER_OF_CALLS *
        -- TIME_TO_DELAY_SMALL_DURATION," SO MAKING SMALL_DURATION UNNECESSARILY
        -- LARGE WILL RESULT IN VERY LONG EXECUTION TIMES AND WILL INTRODUCE 
        -- ERRORS INTO THE COMPUTATION OF THE TASK_SWITCH TIME. IF 
        -- TIME_TO_DELAY_SMALL_DURATION CORRESPONDS TO ZERO DELAY AND
        -- DOES NOT PERMIT THE LOW PRIORITY TASK TO RUN AT ALL, THEN IT MUST BE
        -- MADE LARGER.

--  type large_counter is delta 1.0 range 0.0 .. 2.0**31-1.0;
--  subtype large_counter is float9 range 0.0 .. 2.0**31-1.0;  
    type large_counter is range 0 .. 2**31-1;
    subtype lc is large_counter;

  increment_count  : large_counter ;
  save_increment_count  : large_counter ;
  stop_count            : large_counter ;
  
  lc_zero :large_counter := large_counter(0);
  lc_one  :large_counter := large_counter(1);
  lc_thousand :large_counter := large_counter(1_000);
  lc_million :large_counter := large_counter(1_000_000);

  stop_received : boolean := false; 

  pragma SHARED ( INCREMENT_COUNT );

  NUMBER_OF_CALLS : constant zg_glob1.integer16 := 5_000 ;

  time_delay_ts1 , sigma_ts1 : zg_glob1.float6 := 0.0 ;
  number_ts1 : zg_glob1.integer16 := 0 ; 
  time_delay_ts2 , sigma_ts2 : zg_glob1.float6 := 0.0 ;
  number_ts2 : zg_glob1.integer16 := 0 ; 
  time_delay_ts2x            : zg_glob1.float6 := 0.0 ;
  time_delay_ts3 , sigma_ts3 : zg_glob1.float6 := 0.0 ;
  number_ts3 : zg_glob1.integer16 := 0 ; 
  task_switch_time           : zg_glob1.float6 := 0.0 ;

  task TASK1 is
    pragma priority( zg_glob1.priority_1 );
    entry START_PROBLEM;
    entry STOP_PROBLEM;
  end TASK1;

  task TASK2 is
    pragma priority( zg_glob1.priority_2 );
    entry START_PROBLEM;
    entry STOP_PROBLEM;
  end TASK2;

end tk_lfpkg_60 ;

--++++++++++++++++++++++++++++++++++++++++++++

with tk_lfpkg_60; 
use tk_lfpkg_60;
procedure tk_lfproc_60 is

  PRAGMA suppress(access_check);
  PRAGMA suppress(discriminant_check);
  PRAGMA suppress(index_check);
  PRAGMA suppress(length_check);
  PRAGMA suppress(range_check);
  PRAGMA suppress(division_check);
  PRAGMA suppress(overflow_check);
  PRAGMA suppress(elaboration_check);
  PRAGMA suppress(storage_check);

begin
  tk_lfpkg_60.save_increment_count := tk_lfpkg_60.increment_count ;
  tk_lfpkg_60.increment_count := tk_lfpkg_60.lc_zero;
  raise program_error; 
end tk_lfproc_60 ;

--++++++++++++++++++++++++++++++++++++++++++++

with zg_glob1;
use  zg_glob1;
with zg_glob2;
with zg_glob3;
with zg_glob4;
with calendar;
use  calendar;
with system;
with tk_lfpkg_60;
use tk_lfpkg_60;
with tk_lfproc_60 ;
procedure delay_ts_ts1 is

  PRAGMA suppress(access_check);
  PRAGMA suppress(discriminant_check);
  PRAGMA suppress(index_check);
  PRAGMA suppress(length_check);
  PRAGMA suppress(range_check);
  PRAGMA suppress(division_check);
  PRAGMA suppress(overflow_check);
  PRAGMA suppress(elaboration_check);
  PRAGMA suppress(storage_check);

  task dummy_reciever is 
    entry start_problem; 
    entry stop_problem ; 
  end dummy_reciever ;

  task body dummy_reciever is
  begin
    accept start_problem;      

---------------------------------------------------------------------------
  -- Test Name         :  TK_LF_TASK_60
  -- Prime Purpose     :  This is the first of three test problems to
  --                      determine the task-switch time. Problem TK_LF_TASK_60 
  --                      measures the time to increment a counter.
  -- LRM Version       :  Revision A
  -- Optimization      :
  -- Related tests     :  TK_LF_TASK_61 & TK_LF_TASK_62 
  -- Author            :  Thomas C. Leavitt
  -- Reviewer          :  Kermit E. Terrell
  -- Date              :  24 Sep 1991
  -- Source            :  
  -- Dependencies      :  Systems must support preemptive priority scheduling
  --                      and recognize the expiration of a DELAY as a 
  --                      scheduling point. Systems must accept a value of
  --                      SMALL_DURATION (by default 10 milliseconds) as large
  --                      enough to force an actual DELAY (not be quantized 
  --                      into a zero by the precision of predefined type 
  --                      DURATION).
  -- Other Information :
-------------------------------------------------------------------------------

    zg_glob2.put_test_name("tk_lf_task_60 ");
    pragma include("startime");
    tk_lfpkg_60.increment_count := tk_lfpkg_60.lc_zero ;
    tk_lfpkg_60.stop_count := tk_lfpkg_60.lc_thousand ;
    loop
      loop 
        if zg_glob3.boolean_false then
          tk_lfproc_60 ;                -- Modify increment_count
          zg_glob3.boolean_false := not zg_glob3.boolean_false;
          zg_glob4.die; 
        else
          tk_lfpkg_60.increment_count := tk_lfpkg_60.increment_count + 
                                         tk_lfpkg_60.lc_one ;
        end if;
        exit when tk_lfpkg_60.increment_count >= tk_lfpkg_60.stop_count; 
      end loop;

      tk_lfpkg_60.stop_count := tk_lfpkg_60.stop_count + 
                                tk_lfpkg_60.lc_thousand ; 
  
      select 
        accept stop_problem      -- this will never be called 
        do
          tk_lfpkg_60.stop_received := true; 
        end;
      else        
        tk_lfpkg_60.stop_received := false; 
      end select;
      exit when tk_lfpkg_60.stop_received or else 
               tk_lfpkg_60.increment_count >= tk_lfpkg_60.lc_million ;
    end loop;
    pragma include("stoptime0");
      zg_glob2.put_comment_optional(
         "   1_000_000 increments of large_counter ");
      zg_glob2.put_new_line_optional;
    pragma include("stoptime2");

    if zg_glob3.min_time /= 0.0 then
      tk_lfpkg_60.time_delay_ts1 := zg_glob3.min_time ; 
      tk_lfpkg_60.sigma_ts1      := zg_glob3.timing_sigma ;
      tk_lfpkg_60.number_ts1     := zg_glob3.mcount ;
    else       
          -- taken only if user compiles program without running INCLUDE 
      tk_lfpkg_60.time_delay_ts1 := 0.0 ;
      tk_lfpkg_60.sigma_ts1      := 0.0 ;
      tk_lfpkg_60.number_ts1     := 1 ;
    end if;
  end dummy_reciever ;

  task dummy_caller is end dummy_caller;
  task body dummy_caller is
  begin
    dummy_reciever.start_problem;      
    if zg_glob3.boolean_false then
      dummy_reciever.stop_problem; 
      raise program_error;
    end if; 
    zg_glob4.proc0;
  end dummy_caller;
begin
  null;
end delay_ts_ts1;

--++++++++++++++++++++++++++++

with zg_glob1;
use  zg_glob1;
with zg_glob2;
with zg_glob3;
with zg_glob4;
with calendar;
use  calendar;
with system;
with tk_lfpkg_60;
use tk_lfpkg_60;
with tk_lfproc_60 ;
procedure delay_ts_ts2 is 

  PRAGMA suppress(access_check);
  PRAGMA suppress(discriminant_check);
  PRAGMA suppress(index_check);
  PRAGMA suppress(length_check);
  PRAGMA suppress(range_check);
  PRAGMA suppress(division_check);
  PRAGMA suppress(overflow_check);
  PRAGMA suppress(elaboration_check);
  PRAGMA suppress(storage_check);

begin

---------------------------------------------------------------------------
  -- Test Name         :  TK_LF_TASK_61
  -- Prime Purpose     :  This is the second of three test problems to
  --                      determine the task-switch time. Problem TK_LF_TASK_61 
  --                      measures the time to perform a FOR loop 
  --                      NUMBER_OF_CALLS times whose body contains a DELAY
  --                      statement with a zero value. 
  -- LRM Version       :  Revision A
  -- Optimization      :
  -- Related tests     :  TK_LF_TASK_60 & TK_LF_TASK_62 
  -- Author            :  Thomas C. Leavitt
  -- Reviewer          :  Kermit E. Terrell
  -- Date              :  24 Sep 1991
  -- Source            :  
  -- Dependencies      :  Systems must support preemptive priority scheduling
  --                      and recognize the expiration of a DELAY as a 
  --                      scheduling point. Systems must accept a value of
  --                      SMALL_DURATION (by default 10 milliseconds) as large
  --                      enough to force an actual DELAY (not be quantized 
  --                      into a zero by the precision of predefined type 
  --                      DURATION).
  -- Other Information :
-------------------------------------------------------------------------------

    zg_glob2.put_test_name("tk_lf_task_61 ");
    pragma include("startime");
      tk_lfpkg_60.increment_count := tk_lfpkg_60.lc_zero ;
      for i in 1..TK_LFPKG_60.NUMBER_OF_CALLS 
      loop
        if zg_glob3.boolean_false then
          tk_lfproc_60 ;                -- Modify increment_count
          zg_glob3.boolean_false := not zg_glob3.boolean_false;
          zg_glob4.die; 
        end if;
        delay tk_lfpkg_60.zero_duration; 
      end loop;
    pragma include("stoptime0");
      zg_glob2.put_comment_optional(
         "   NUMBER_OF_CALLS iterations of DELAY TK_LFPKG_60.ZERO_DURATION");
      zg_glob2.put_new_line_optional;
      zg_glob2.put_error_code(delay_problem); 
    pragma include("stoptime2");

   if zg_glob3.min_time /= 0.0 then
     tk_lfpkg_60.time_delay_ts2 := zg_glob3.min_time; 
     tk_lfpkg_60.sigma_ts2      := zg_glob3.timing_sigma ;
     tk_lfpkg_60.number_ts2     := zg_glob3.mcount ;
   else
       -- taken only if user compiles program without running INCLUDE 
     tk_lfpkg_60.time_delay_ts2 := 0.0 ;
     tk_lfpkg_60.sigma_ts2      := 0.0 ;
     tk_lfpkg_60.number_ts2     := 1 ;
   end if;

end delay_ts_ts2;

--+++++++++++++++++++++++++++++++++

with zg_glob1;
use  zg_glob1;
with zg_glob2;
with zg_glob3;
with zg_glob4;
with calendar;
use  calendar;
with system;
with tk_lfpkg_60;
use tk_lfpkg_60;
with tk_lfproc_60 ;
procedure delay_ts_ts2x is

  PRAGMA suppress(access_check);
  PRAGMA suppress(discriminant_check);
  PRAGMA suppress(index_check);
  PRAGMA suppress(length_check);
  PRAGMA suppress(range_check);
  PRAGMA suppress(division_check);
  PRAGMA suppress(overflow_check);
  PRAGMA suppress(elaboration_check);
  PRAGMA suppress(storage_check);

begin
  pragma include("startime");
    if zg_glob3.boolean_false then
      tk_lfproc_60 ;                -- Modify increment_count
      zg_glob3.boolean_false := not zg_glob3.boolean_false;
      zg_glob4.die; 
    end if;
    delay tk_lfpkg_60.small_duration; 
  pragma include("stoptime0");
    zg_glob2.put_comment_optional(
       "   DELAY SMALL_DURATION; ");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional(
       "   Not a ""real"" test problem but used to compute a temporary");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional(
       "   value to determine if TK_LF_TASK_62 would take an excessive");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional(
       "   time to complete. ");
    zg_glob2.put_new_line_optional;
  pragma include("stoptime2");
  tk_lfpkg_60.time_delay_ts2x := zg_glob3.min_time; 
end delay_ts_ts2x;

-- ++++++++++++++++++++++++++++++++++

with zg_glob1;
use  zg_glob1;
with zg_glob2;
with zg_glob3;
with zg_glob4;
with calendar;
use  calendar;
with system;
with tk_lfpkg_60;
use tk_lfpkg_60;
with zm_ran; 
with tk_lfproc_60 ;
procedure delay_ts_ts3 is

  PRAGMA suppress(access_check);
  PRAGMA suppress(discriminant_check);
  PRAGMA suppress(index_check);
  PRAGMA suppress(length_check);
  PRAGMA suppress(range_check);
  PRAGMA suppress(division_check);
  PRAGMA suppress(overflow_check);
  PRAGMA suppress(elaboration_check);
  PRAGMA suppress(storage_check);

  t0, t1, t2, t3, t4, t5, t6, t7, t8, t9, t10, t11, t12 : 
  tk_lfpkg_60.large_counter; 
begin
  tk_lfpkg_60.task1.start_problem;

---------------------------------------------------------------------------
  -- Test Name         :  TK_LF_TASK_62
  -- Prime Purpose     :  This is the third of three test problems to
  --                      determine the task-switch time. Problem TK_LF_TASK_62 
  --                      This problem measures the time to do the following:
  --                         1. Reset a counter; 
  --                         2. Execute a FOR loop NUMBER_OF_CALLS times where 
  --                            the loop body contains a DELAY which specifies 
  --                            a non-zero value in a high priority task, while 
  --                            a low priority task is incrementing the counter 
  --                            which was reset at the start of this test 
  --                            problem. 
  --                         3. Save the counter value in SAVE_INCREMENT_COUNT. 
  --                      This test problem will estimate its time to complete 
  --                      by performing a preliminary test involving a DELAY 
  --                      statement for a small duration.  If the estimate is 
  --                      excessive, the user will be notified and the test 
  --                      not run. A user would choose to to modify and rerun 
  --                      the test problem, changing either the time limit 
  --                      compared against or reducing the value of 
  --                      NUMBER_OF_CALLS to specify a faster running test 
  --                      problem. 
  --                      
  --                      After collecting the time measurements for the above 
  --                      test problems, an estimate of the task-switch time 
  --                      can be calculated as:
  --                            ( measured_time_for_problem_with_nonzero_delay -
  --                              measured_time_for_problem_with_zero_delay -
  --                              measured_time_to_increment_counter * 
  --                                 SAVE_INCREMENT_COUNT
  --                            ) / (NUMBER_OF_CALLS*2)
  --                      
  --                      The test problem output task switch time and an 
  --                      appropriate confidence level. 
  -- LRM Version       :  Revision A
  -- Optimization      :
  -- Related tests     :  TK_LF_TASK_60 & TK_LF_TASK_62 
  -- Author            :  Thomas C. Leavitt
  -- Reviewer          :  Kermit E. Terrell
  -- Date              :  24 Sep 1991
  -- Source            :  
  -- Dependencies      :  Systems must support preemptive priority scheduling
  --                      and recognize the expiration of a DELAY as a 
  --                      scheduling point. Systems must accept a value of
  --                      SMALL_DURATION (by default 10 milliseconds) as large
  --                      enough to force an actual DELAY (not be quantized 
  --                      into a zero by the precision of predefined type 
  --                      DURATION).
  -- Other Information :
-------------------------------------------------------------------------------

  tk_lfpkg_60.save_increment_count := tk_lfpkg_60.lc_zero ;
  zg_glob2.put_test_name("tk_lf_task_62 ");
  pragma include("startime");
    if not zg_glob3.boolean_false then 
       tk_lfpkg_60.increment_count := tk_lfpkg_60.lc_zero ; 
    end if;
    for i in 1..TK_LFPKG_60.NUMBER_OF_CALLS 
    loop
      if zg_glob3.boolean_false then
        tk_lfproc_60 ;                -- Modify increment_count
        zg_glob3.boolean_false := not zg_glob3.boolean_false;
           -- Present complex expressions (in the sense that they require 
           -- temporary storage) to discourage a compiler from assignment the 
           -- value of INCREMENT_COUNT to a register. If a compiler ignores the
           -- PRAGMA SHARED and optimizes INCREMENT_COUNT into a register
           -- then it will not work as intended and the system will NEVER
           -- notice that INCREMENT_COUNT is modified by another task. 
        t1  := tk_lfpkg_60.large_counter( zm_ran.random + 1.0 );
        t2  := tk_lfpkg_60.large_counter( zm_ran.random + 1.0 );
        t3  := tk_lfpkg_60.large_counter( zm_ran.random + 1.0 );
        t4  := tk_lfpkg_60.large_counter( zm_ran.random + 1.0 );
        t5  := tk_lfpkg_60.large_counter( zm_ran.random + 1.0 );
        t6  := tk_lfpkg_60.large_counter( zm_ran.random + 1.0 );
        t7  := tk_lfpkg_60.large_counter( zm_ran.random + 1.0 );
        t8  := tk_lfpkg_60.large_counter( zm_ran.random + 1.0 );
        t9  := tk_lfpkg_60.large_counter( zm_ran.random + 1.0 );
        t10 := tk_lfpkg_60.large_counter( zm_ran.random + 1.0 );
        t11 := tk_lfpkg_60.large_counter( zm_ran.random + 1.0 );
        t12 := tk_lfpkg_60.large_counter( zm_ran.random + 1.0 );
        t0  := 
            (((tk_lfpkg_60.lc((t1 + t9) / (t1 + t10)) + 
               tk_lfpkg_60.lc((t1 + t11) / (t1 + t12)))
            / (tk_lfpkg_60.lc((t2 + t9) / (t2 + t10)) + 
               tk_lfpkg_60.lc((t2 + t11) / (t2 + t12))))
           + ((tk_lfpkg_60.lc((t3 + t9) / (t3 + t10)) + 
               tk_lfpkg_60.lc((t3 + t11) / (t4 + t12)))
            / (tk_lfpkg_60.lc((t4 + t9) / (t4 + t10)) + 
               tk_lfpkg_60.lc((t4 + t11) / (t4 + t12)))))
          / (((tk_lfpkg_60.lc((t5 + t9) / (t5 + t10)) + 
               tk_lfpkg_60.lc((t5 + t11) / (t5 + t12)))
            / (tk_lfpkg_60.lc((t6 + t9) / (t6 + t10)) + 
               tk_lfpkg_60.lc((t6 + t11) / (t6 + t12))))
           + ((tk_lfpkg_60.lc((t7 + t9) / (t7 + t10)) + 
               tk_lfpkg_60.lc((t7 + t11) / (t7 + t12)))
            / (tk_lfpkg_60.lc((t8 + t9) / (t8 + t10)) + 
               tk_lfpkg_60.lc((t8 + t11) / (t8 + t12)))));
        zg_glob4.proc1( zg_glob1.float6(t0) );
        zg_glob4.die; 
      end if;
      delay tk_lfpkg_60.small_duration; 
    end loop;
    tk_lfpkg_60.save_increment_count := tk_lfpkg_60.increment_count ;
  pragma include("stoptime0");

  tk_lfpkg_60.task1.stop_problem;

  zg_glob2.put_comment_optional(
     "   Loop of DELAY SMALL_DURATION with low priority task");
  zg_glob2.put_new_line_optional;
  zg_glob2.put_comment_optional(
     "   which increments counter. Time for this problem will be ");
  zg_glob2.put_new_line_optional;
  zg_glob2.put_comment_optional(
     "   roughly NUMBER_OF_CALLS * TIME_TO_DELAY_SMALL_DURATION ");
  zg_glob2.put_new_line_optional;
  zg_glob2.put_error_code(delay_problem); 

  if zg_glob3.min_time /= 0.0 then
    tk_lfpkg_60.time_delay_ts3 := zg_glob3.min_time; 
    tk_lfpkg_60.sigma_ts3      := zg_glob3.timing_sigma ;
    tk_lfpkg_60.number_ts3     := zg_glob3.mcount ;
  else
       -- taken only if use compiles program without running INCLUDE 
    tk_lfpkg_60.time_delay_ts3 := 0.0 ;
    tk_lfpkg_60.sigma_ts3      := 0.0 ;
    tk_lfpkg_60.number_ts3     := 1 ;
  end if;

  if tk_lfpkg_60.save_increment_count <= tk_lfpkg_60.lc_one then
    zg_glob2.put_comment(
       "   anomaly detected, either SMALL_DURATION too small");
    zg_glob2.put_new_line;
    zg_glob2.put_comment(
       "   or priorities not honored. If former then it must be ");
    zg_glob2.put_new_line;
    zg_glob2.put_comment(
       "   made larger to get a valid estimate for task-switch ");
    zg_glob2.put_new_line;
    zg_glob2.put_comment(
       "   time. If priorities not honored, cannot get estimate");
    zg_glob2.put_new_line;
    zg_glob2.put_comment(
       "   for task-switch time using the technique of this program.");
    zg_glob2.put_new_line;
    zg_glob2.put_error_code(err_dependent_test);
  end if;
  pragma include("stoptime2");

  exception 
  when others => 
    tk_lfpkg_60.task1.stop_problem;
end delay_ts_ts3;

--++++++++++++++++++++++++++++++++++++++++

with delay_ts_ts1, delay_ts_ts2, delay_ts_ts2x, delay_ts_ts3;
with tk_lfproc_60 ;
package body TK_LFPKG_60 is

  PRAGMA suppress(access_check);
  PRAGMA suppress(discriminant_check);
  PRAGMA suppress(index_check);
  PRAGMA suppress(length_check);
  PRAGMA suppress(range_check);
  PRAGMA suppress(division_check);
  PRAGMA suppress(overflow_check);
  PRAGMA suppress(elaboration_check);
  PRAGMA suppress(storage_check);

  task body TASK1 is                      -- low priority task
  begin
   accept start_problem;
   tk_lfpkg_60.increment_count := tk_lfpkg_60.lc_zero ; 
   tk_lfpkg_60.stop_count := tk_lfpkg_60.lc_thousand ; 
   loop
    loop
      if zg_glob3.boolean_false then
        tk_lfproc_60 ;                    -- Modify increment_count
        zg_glob3.boolean_false := not zg_glob3.boolean_false;
        zg_glob4.die; 
      else
        tk_lfpkg_60.increment_count := tk_lfpkg_60.increment_count +
                                       tk_lfpkg_60.lc_one ;
      end if;
      exit when tk_lfpkg_60.increment_count >= tk_lfpkg_60.stop_count; 
    end loop;

    tk_lfpkg_60.stop_count := tk_lfpkg_60.increment_count + 
                              tk_lfpkg_60.lc_thousand ; 

    select 
      accept stop_problem
      do
        tk_lfpkg_60.stop_received := true; 
      end;
    else
      tk_lfpkg_60.stop_received := false; 
    end select;
    exit when tk_lfpkg_60.stop_received;    
   end loop;    
  exception 
  when others => 
    abort task2;
    zg_glob2.put_comment("  unexpected exception raised in TASK1 ");
    zg_glob2.put_new_line;
    raise program_error;
  end TASK1;

  task body TASK2 is         
  begin

    accept start_problem;

    zg_glob3.min_iteration_count := 16;
    zg_glob3.max_iteration_count := 16;
    delay_ts_ts1;

    zg_glob3.min_iteration_count := 16;
    zg_glob3.max_iteration_count := 16;
    delay_ts_ts2;

    if zg_glob1.float6(tk_lfpkg_60.small_duration) = 0.0 then
        zg_glob2.put_comment(
           "   TK_LF_TASK_62 SMALL_DURATION converts to DURATION'(0.0) ");
        zg_glob2.put_new_line;
        zg_glob2.put_comment(
           "   Task-switch time can be calculated without other");
        zg_glob2.put_new_line;
        zg_glob2.put_comment(
           "   modification if value of SMALL_DURATION is ");
        zg_glob2.put_new_line;
        zg_glob2.put_comment(
           "   increased to size large enough to ensure that the");
        zg_glob2.put_new_line;
        zg_glob2.put_comment(
           "   task executing it is placed on the delay queue.");
        zg_glob2.put_error_code(err_at_compilation_time);
        zg_glob4.die;
    end if;

    delay_ts_ts2x;

  zg_glob3.min_iteration_count := 4;
  zg_glob3.max_iteration_count := 4;

  if zg_glob1.float6(zg_glob3.min_iteration_count) * 
     zg_glob1.float6(tk_lfpkg_60.number_of_calls) * 
                     tk_lfpkg_60.time_delay_ts2x > 
     zg_glob3.excessive_time 
  then
    zg_glob2.put_new_line;
    zg_glob2.put_comment(
       " Estimate that problem DELAY_TS3 would take an excessive amount");
    zg_glob2.put_new_line;
    zg_glob2.put_comment(
       " time to complete. To obtain estimate of task-switch time, ");
    zg_glob2.put_new_line;
    zg_glob2.put_comment(
       " for which DELAY_TS3 is needed, user may reduce value of ");
    zg_glob2.put_new_line;
    zg_glob2.put_comment(
       " NUMBER_OF_CALLS or of SMALL_DURATION"); 
    zg_glob2.put_new_line;
    zg_glob2.put_comment(" estimate is ");
    zg_glob2.put_value( zg_glob1.float6(zg_glob3.min_iteration_count) * 
                        zg_glob1.float6(tk_lfpkg_60.number_of_calls) * 
                        tk_lfpkg_60.time_delay_ts2x, 5, 1, 0 );
    zg_glob2.put_comment     (" seconds.");
    zg_glob2.put_new_line;
    zg_glob2.put_comment("TK_LF_TASK_62 ");
    zg_glob2.put_error_code (err_no_data);
    zg_glob2.put_ancillary(
    " >>> TK_LF_TASK_62 Task-switch time not computed, user adaptation needed");
    zg_glob2.put_new_line;
    abort task1; 
    zg_glob4.die;
  end if;

  zg_glob2.put_comment(" Expect test problem DELAY_TS3 to execute for");
  zg_glob2.put_value(  zg_glob1.float6(zg_glob3.min_iteration_count) * 
                       zg_glob1.float6(tk_lfpkg_60.number_of_calls) * 
                       tk_lfpkg_60.time_delay_ts2x, 5, 1, 0 );
  zg_glob2.put_comment(" seconds. Be patient.");
  zg_glob2.put_new_line;

  delay_ts_ts3;

  zg_glob2.put_comment(" small_duration =  ");
  zg_glob2.put_value( zg_glob1.float6(tk_lfpkg_60.small_duration) , 1,6,0);
  zg_glob2.put_new_line;
   
  zg_glob2.put_comment(" NUMBER_OF_CALLS = ");
  zg_glob2.put_value( zg_glob1.integer16(tk_lfpkg_60.number_of_calls) );
  zg_glob2.put_new_line;

  zg_glob2.put_comment(" save_increment_count = ");
  zg_glob2.put_value( zg_glob1.float6(tk_lfpkg_60.save_increment_count), 
                      1, 6, 2);
  zg_glob2.put_new_line;

  zg_glob2.put_comment(" Estimate for DELAY task-switch time is ");
  zg_glob2.put_new_line;
  zg_glob2.put_comment(
     "   ( delay_ts3 -    -- time for version with both tasks   ");
  zg_glob2.put_new_line;
  zg_glob2.put_comment(
     "     delay_ts2 -    -- time for high priority task with zero delay ");
  zg_glob2.put_new_line;
  zg_glob2.put_comment(
     "    (delay_ts1/1_000_000.0) * save_increment_count-- low priority task");
  zg_glob2.put_new_line;
  zg_glob2.put_comment(
     "   ) / (tk_lfpkg_60.number_of_calls*2) ");
  zg_glob2.put_new_line;
  zg_glob2.put_comment(
     "        The execution time not allocated to TASK1 or TASK2 ");
  zg_glob2.put_new_line;
  zg_glob2.put_comment(
     "        is assumed to be spent system overhead which should be");
  zg_glob2.put_new_line;
  zg_glob2.put_comment(
     "        mostly task-switch time plus some extra to place task");
  zg_glob2.put_new_line;
  zg_glob2.put_comment(
     "        on the DELAY queue and to respond to delay expiration. ");
  zg_glob2.put_new_line;
                                                  
  DECLARE           -- declare for confidence interval computation
    temp3 : float6 := time_delay_ts3 ;
    temp2 : float6 := time_delay_ts2 ;
    temp1 : float6 := ( time_delay_ts1 / zg_glob1.float6 (lc_million) ) *
                        zg_glob1.float6 (save_increment_count) ;
    temp1_sigma : float6 := ( sigma_ts1 / zg_glob1.float6 (lc_million) ) *
                        zg_glob1.float6 (save_increment_count) ;

    scale_factor : float6 := zg_glob1.float6 (number_of_calls) * 2.0 ;

       TYPE tTableType IS ARRAY ( 1 .. 34 ) OF ZG_glob1.float6 ;
       tTable : CONSTANT tTableType := (   -- student's t distribution
                          6.3138, 2.9200, 2.3534, 2.1318, 2.0150, 
                          1.9432, 1.8946, 1.8595, 1.8331, 1.8125, 
                          1.7959, 1.7823, 1.7709, 1.7613, 1.7530, 
                          1.7459, 1.7396, 1.7341, 1.7291, 1.725,
                          1.721,  1.717,  1.714,  1.711,  1.708,
                          1.706,  1.703,  1.701,  1.699,  1.697,
                          1.684,  1.671,  1.658,  1.645   ) ;

       -- This is a local function (not externally visible) to compute SQRT.
       -- It uses Heron's formula.  To use the version of SQRT in math would
       -- create a dependency on MATH, when would force users to port
       -- MATH before they could run this test
                ----
       FUNCTION Sqrt ( x : IN ZG_glob1.float6 ) RETURN ZG_glob1.float6 IS
                ----
          y : ZG_glob1.float6 ;
       BEGIN
          IF x <= 0.0  THEN  RETURN 0.0 ;  END IF ;

          y := 0.0001;
          LOOP
             y := ( y + x / y ) / 2.0;
             EXIT WHEN 
                abs ( ( y * y - x ) / x ) < 0.0001; -- sufficient accuracy
          END LOOP;

          RETURN y;
       END Sqrt ;

                 ----------------
       PROCEDURE pairedStatistics ( 
                 ----------------        
                x1, s1 : zg_glob1.float6 ; n1 : zg_glob1.integer16 ;
                x2, s2 : zg_glob1.float6 ; n2 : zg_glob1.integer16  ) 
       IS
          diff ,
          cInterval : zg_glob1.float6 ;         

                   -------
          FUNCTION tLookUp ( df : ZG_glob1.float6 )  RETURN  zg_glob1.float6 IS
                   -------
             dfIndex : Natural := Natural ( df ) ;
          BEGIN
             IF dfIndex = 0
             THEN
                dfIndex := 1 ;
             ELSIF dfIndex <= 30    -- table look up adjustment for > 30
             THEN
                NULL ;
             ELSIF dfIndex <= 40
             THEN
                dfIndex := 31 ;
             ELSIF dfIndex <= 60
             THEN
                dfIndex := 32 ;
             ELSIF dfIndex <= 120
             THEN
                dfIndex := 33 ;
             ELSE
                dfIndex := 34 ;
             END IF ;

             RETURN tTable ( dfIndex ) ;
          END tLookUp ;

       -- T test : difference between two means, population variances not equal 
       -- reference : pages 139-141 in Biostatistics: A Foundation for Analysis
       --             in the Health Sciences by Wayne W. Daniel, John Wiley and
       --             Sons, New York, 1978.
                   ------------
          FUNCTION ConfInterval RETURN  zg_glob1.float6 IS
                   ------------
             temp1,
             temp2,
             df : ZG_glob1.float6 := 0.0 ;
          BEGIN
             IF s1 = 0.0 OR ELSE s2 = 0.0
             THEN
                RETURN 0.0 ;
             END IF ;
             temp1 := s1 * s1 / ZG_glob1.float6 ( n1 ) ;
             temp2 := s2 * s2 / ZG_glob1.float6 ( n2 ) ;
             df := ( temp1 + temp2 ) * ( temp1 + temp2 ) /
                   ( ( temp1 * temp1 / ZG_glob1.float6 ( n1 ) ) + 
                     ( temp2 * temp2 / ZG_glob1.float6 ( n2 ) ) ) ;
             RETURN tLookUp ( df ) * Sqrt ( temp1 + temp2 ) ;
          END ConfInterval ;

       BEGIN    -- pairedStatistics 
          diff := x1 - x2 ;                                      
          cInterval := confInterval ;

          zg_glob2.put_time_measurement ( diff - cInterval ) ;
          zg_glob2.put_comment(" .. ");
          zg_glob2.put_time_measurement ( diff + cInterval ) ;
          zg_glob2.put_comment(" microseconds.");
          zg_glob2.put_new_line;
       END pairedStatistics ;

  BEGIN             -- declare for confidence interval computation
    task_switch_time := 
      ( ( temp3 - temp1 )  - temp2 )   -- this order of subtraction will 
                                       -- minimize loss of numeric significance 
      / ( zg_glob1.float6 ( number_of_calls ) * 2.0 ) ;
                                                                    
  zg_glob2.put_ancillary(
     " >>> TK_LF_TASK_62 task-switch overhead is calculated as "); 

  if task_switch_time <= 0.0 then                    
    zg_glob2.put_time_measurement ( task_switch_time );
    zg_glob2.put_new_line;
    zg_glob2.put_ancillary(
       " >>> TK_LF_TASK_62 Impossible to compute task_switch_time. Rerun. ");
    zg_glob2.put_new_line;
  elsif task_switch_time > tk_lfpkg_60.time_delay_ts2 / 2.0 then
    zg_glob2.put_ancillary(
       " >>> TK_LF_TASK_62 Inconsistent task-switch time ");
    zg_glob2.put_time_measurement ( task_switch_time );
    zg_glob2.put_new_line;
    zg_glob2.put_comment( 
       " >>> NOT meaningful time because the system could not perform ");
    zg_glob2.put_new_line;
    zg_glob2.put_comment(                               
       " >>> problem DELAY_TS2 within the measured time for it if a task");
    zg_glob2.put_new_line;
    zg_glob2.put_comment(
       " >>> switch actually took as long as estimated from formula. ");
    zg_glob2.put_new_line;
    zg_glob2.put_comment(
       " >>> Suggest test program be rerun.");
    zg_glob2.put_new_line;
  else
    zg_glob2.put_time_measurement ( task_switch_time );
    zg_glob2.put_comment(" microseconds.");
    zg_glob2.put_new_line;
  end if;

    IF temp2 / ( temp3 - temp1 ) >= 0.050
    THEN                              
      zg_glob2.put_ancillary(                                
" >>> TK_LF_TASK_62 DELAY_TS2 is not negligible? Its percentage of total is ");
       zg_glob2.put_value( 
         100.0 * temp2 / ( temp3 - temp1 ), 4, 2, 0 );
      zg_glob2.put_comment("% ");
       zg_glob2.put_new_line;
      zg_glob2.put_ancillary(                                
            " >>> TK_LF_TASK_62 Confidence interval was not computed!");
       zg_glob2.put_new_line;
    ELSE 
      zg_glob2.put_new_line;
                
      zg_glob2.put_ancillary(                                
         " >>> TK_LF_TASK_62 95% confidence interval is ");
                                                                     
      -- change scale to match task_switch_time
      temp3       := temp3 / scale_factor ;
      sigma_ts3   := sigma_ts3 / scale_factor ;
      temp1       := temp1 / scale_factor ;           
      temp1_sigma := temp1_sigma / scale_factor ;

      pairedStatistics ( temp3, sigma_ts3, number_ts3,
                         temp1, temp1_sigma, number_ts1 ) ;
      zg_glob2.put_new_line;
    END IF ;                

      if task_switch_time > 0.0 then
        zg_glob2.put_comment_optional(
               "  TK_LF_TASK_62 The task-switching time was");
        zg_glob2.put_value( (task_switch_time*2.0 * 
                zg_glob1.float6(tk_lfpkg_60.number_of_calls) * 100.0) /
                tk_lfpkg_60.time_delay_ts3, 4, 2, 0);
        zg_glob2.put_comment("% of the total.");
        zg_glob2.put_new_line;
        zg_glob2.put_comment_optional (
         "  The calculation is sensitive to errors in measurements. It ");
        zg_glob2.put_new_line;
        zg_glob2.put_comment_optional (
         "  subtracts quantities which may be close - for a system with");
        zg_glob2.put_new_line;
        zg_glob2.put_comment_optional (
         "  a 500 microsecond ""switch-time"" and a 10 millisecond delay ");
        zg_glob2.put_new_line;
        zg_glob2.put_comment_optional (
         "  the difference calculated will be 10% of the total time. The ");
        zg_glob2.put_new_line;
        zg_glob2.put_comment_optional (
         "  error bounds on this derived measure will be larger than for ");
        zg_glob2.put_new_line;
        zg_glob2.put_comment_optional (
         "  test problems measured directly, however they should be ");
        zg_glob2.put_new_line;
        zg_glob2.put_comment_optional (                              
         "  sufficiently accurate, at least on embedded targets, to determine");
        zg_glob2.put_new_line;
        zg_glob2.put_comment_optional (
         "  whether the task-switch time is ""excessively"" large. Under a ");
        zg_glob2.put_new_line;
        zg_glob2.put_comment_optional (                                     
         "  general purpose operating system, a task switch may require a");
        zg_glob2.put_new_line;
        zg_glob2.put_comment_optional (                                  
         "  few thousand machine instructions; a bare machine target should");
        zg_glob2.put_new_line;
        zg_glob2.put_comment_optional (
         "  be MUCH faster.");
        zg_glob2.put_new_line;
      else
        zg_glob2.put_comment (
         "  TK_LF_TASK_62 No meaningful task-switch time was computed. ");
        zg_glob2.put_new_line;
      end if;
  END ;

  accept stop_problem;

  exception 
  when others => 
    abort task1;
    zg_glob2.put_comment("  unexpected exception raise in TASK2 ");
    zg_glob2.put_new_line;
    raise program_error;
  end TASK2;

end tk_lfpkg_60;

--++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

with zg_glob1;
use  zg_glob1;
with zg_glob2;
with zg_glob3;
use  zg_glob3;
with zg_glob4;
WITH calendar;
USE  calendar;
with tk_lfpkg_60; 
          
PROCEDURE tk_lf28 IS

  PRAGMA suppress(access_check);
  PRAGMA suppress(discriminant_check);
  PRAGMA suppress(index_check);
  PRAGMA suppress(length_check);
  PRAGMA suppress(range_check);
  PRAGMA suppress(division_check);
  PRAGMA suppress(overflow_check);
  PRAGMA suppress(elaboration_check);
  PRAGMA suppress(storage_check);

BEGIN    
  tk_lfpkg_60.task2.start_problem;
  tk_lfpkg_60.task2.stop_problem;
EXCEPTION
  WHEN OTHERS =>
    ABORT tk_lfpkg_60.task1, tk_lfpkg_60.task2 ;
    RAISE ;
END tk_lf28 ;
