with zg_glob1;
use  zg_glob1;
with zg_glob4;

package tk_lfpkg_70 is     

  type message is 
    record
      field1 : string(1..80);
    end record; 

  type routing_priority is ( low, medium, high ) ;
  type place is range 1..3 ;

  type evaluation_array_type is array ( zg_glob1.integer16'(1) .. 3 ) 
                           of routing_priority ;
  evaluation_order_array : evaluation_array_type := ( LOW, LOW, LOW );
  evaluation_order_array_index : zg_glob1.integer16 ;
  first_evaluation_order_array : evaluation_array_type ;
  first_evaluation_order_array_index : zg_glob1.integer16 ;
  consistent : boolean := false ;
  first_time_flag : boolean := true ;

  procedure route( m : in message; to : in place ) ;

  task destination_1  is
    entry send( m : in message );
  end destination_1  ;

  task destination_2  is
    entry send( m : in message );
  end destination_2  ;

  task destination_3  is
    entry send( m : in message );
  end destination_3  ;

  task transmit is
    entry routed_priority( routing_priority ) (m : in message; to : in place );
  end transmit;
  
end tk_lfpkg_70 ;

--------------------------------------------------------------------------------

package body tk_lfpkg_70 is

  PRAGMA suppress(access_check);
  PRAGMA suppress(discriminant_check);
  PRAGMA suppress(index_check);
  PRAGMA suppress(length_check);
  PRAGMA suppress(range_check);
  PRAGMA suppress(division_check);
  PRAGMA suppress(overflow_check);
  PRAGMA suppress(elaboration_check);
  PRAGMA suppress(storage_check);

  function guard_function_high return boolean is
  begin
    evaluation_order_array ( evaluation_order_array_index ) := high ;
    evaluation_order_array_index := evaluation_order_array_index + 1;
    return true;
  end guard_function_high ;

  function guard_function_medium return boolean is
  begin
    evaluation_order_array ( evaluation_order_array_index ) := medium ;
    evaluation_order_array_index := evaluation_order_array_index + 1;
    return true;
  end guard_function_medium ;

  function guard_function_low return boolean is
  begin
    evaluation_order_array ( evaluation_order_array_index ) := low ;
    evaluation_order_array_index := evaluation_order_array_index + 1;
    return true;
  end guard_function_low ;

  procedure route ( m : in message ; to : in place ) is begin
    case to is
      --+ when 1 => destination_3.send(m); --+ tcl 6 April 1992 PCR #100
                                           --  replaced line with the following
                                           --  to use the entries DESTINATION_1
                                           --  and DESTINATION_2
      when 1 => destination_1.send(m);     --+ tcl 6 April 1992 PCR #100
      --+ when 2 => destination_3.send(m); --+ tcl 6 April 1992 PCR #100
      when 2 => destination_2.send(m);     --+ tcl 6 April 1992 PCR #100
      when 3 => destination_3.send(m);
    end case;
  end route;

  task body transmit is
  begin
    loop
      evaluation_order_array_index := evaluation_order_array'first ;
      select
        when guard_function_high => 
          accept routed_priority(high) (m : in message; to : in place ) do
            route(m,to);
          end;
      or
        when guard_function_medium and 
             routed_priority(high)'count = 0 => 
          accept routed_priority(medium) (m : in message ; to : in place ) do
            route(m,to);
          end routed_priority;
      or 
        when guard_function_low and 
             routed_priority(high)'count = 0 and 
             routed_priority(medium)'count = 0 => 
          accept routed_priority(low) (m : in message; to : in place ) do
            route(m,to);
          end routed_priority;
      end select;
      if first_time_flag then
        first_time_flag := false ;
        first_evaluation_order_array_index := evaluation_order_array_index ;
        first_evaluation_order_array := evaluation_order_array ;
        consistent := true ;
      else
        consistent := consistent and 
                      ( first_evaluation_order_array = 
                        evaluation_order_array ) and
                      ( first_evaluation_order_array_index = 
                        evaluation_order_array_index ) ;
      end if;
    end loop;
  end transmit;

  task body destination_1 is
  begin
    loop
      accept send( m : in message ) 
      do
        zg_glob4.string10 := m.field1(1..10);        
        zg_glob4.proc0;
      end ;
    end loop;
  end destination_1  ;

  task body destination_2 is
  begin
    loop
      accept send( m : in message ) 
      do
        zg_glob4.string10 := m.field1(1..10);        
        zg_glob4.proc0;
      end ;
    end loop;
  end destination_2 ;

  task body destination_3 is
  begin
    loop
      accept send( m : in message ) 
      do
        zg_glob4.string10 := m.field1(1..10);        
        zg_glob4.proc0;
      end ;
    end loop;
  end destination_3 ;

end tk_lfpkg_70 ;

--------------------------------------------------------------------------------

with zg_glob1;
use  zg_glob1;
with zg_glob2;
with zg_glob3;
with zg_glob4;
with calendar;
use  calendar;
with tk_lfpkg_70 ; 
use  tk_lfpkg_70 ; 
 
PROCEDURE tk_lf_task_70 IS

  PRAGMA suppress(access_check);
  PRAGMA suppress(discriminant_check);
  PRAGMA suppress(index_check);
  PRAGMA suppress(length_check);
  PRAGMA suppress(range_check);
  PRAGMA suppress(division_check);
  PRAGMA suppress(overflow_check);
  PRAGMA suppress(elaboration_check);
  PRAGMA suppress(storage_check);

  my_message : tk_lfpkg_70.message := ( field1 => (others => 'm' ) );
  your_message : tk_lfpkg_70.message := ( field1 => (others => 'y' ) );

  begin

--------------------------------------------------------------------------------
  -- Test Name         :  TK_LF_TASK_70 
  -- Prime Purpose     :  This problem determines the order of evaluation of 
  --                      guard expressions in a selective wait with a family 
  --                      of task entries. 
  -- Optimization      :
  -- Related tests     :  TK_LS_TASK_67
  -- Author            :  Thomas C. Leavitt
  -- Reviewer          :  B. Decker-Lindsey  - 2 March 1992
  -- Date              :  December 1991 
  -- Source            :  Adapted from Software Engineering with Ada, by
  --                      G. Booch, 1983
  -- Dependencies      :
  -- Other Information :
  -- Revisions         :  PCR#100
  -- Author            :  Thomas C. Leavitt
  -- Reviewer          :  Julie Leastman - 14 April 1992
  -- Date              :  6 April 1992
  -- Change            :  Modified entry calls in PROCEDURE ROUTE to call 
  --                   :  on distinct entries. 
--------------------------------------------------------------------------------

  zg_glob2.put_test_name("tk_lf_task_70");
  PRAGMA include("startime");
    tk_lfpkg_70.transmit.routed_priority(tk_lfpkg_70.medium)
       (my_message,   to => 1);
    tk_lfpkg_70.transmit.routed_priority(tk_lfpkg_70.low)   
       (your_message, to => 3);
    tk_lfpkg_70.transmit.routed_priority(tk_lfpkg_70.high)  
       (your_message, to => 2);
  PRAGMA include("stoptime0");
  zg_glob2.put_comment_optional(
    "   Determine the order of evaluation of guard expression in a selective");
  zg_glob2.put_new_line_optional;

  zg_glob2.put_comment_optional(
    "   wait statement using a family of task entries as recommended in ");
  zg_glob2.put_new_line_optional;
  zg_glob2.put_comment_optional(
    "   ""Software Engineering with Ada,"" by G. Booch. It checks whether");
  zg_glob2.put_new_line_optional;
  zg_glob2.put_comment_optional(
    "   the system evaluates the guard expressions in lexical, reverse ");
  zg_glob2.put_new_line_optional;
  zg_glob2.put_comment_optional(
    "   lexical, or some other consistent order - for more information about");
  zg_glob2.put_new_line_optional;
  zg_glob2.put_comment_optional(
    "   the order of guard evaluations, refer to problem TK_LS_TASK_67. ");
  zg_glob2.put_new_line_optional;
  PRAGMA include("stoptime2");

  zg_glob2.put_ancillary(
     " >>> tk_lf_task_70 Order of evaluation of guards in entry family is");
  if tk_lfpkg_70.consistent then
     if    tk_lfpkg_70.first_evaluation_order_array = 
           tk_lfpkg_70.evaluation_array_type'( tk_lfpkg_70.HIGH, 
                                               tk_lfpkg_70.MEDIUM, 
                                               tk_lfpkg_70.LOW ) 
     then    
       zg_glob2.put_comment(" in lexical order.");
     elsif tk_lfpkg_70.first_evaluation_order_array = 
           tk_lfpkg_70.evaluation_array_type'( tk_lfpkg_70.LOW, 
                                               tk_lfpkg_70.MEDIUM, 
                                               tk_lfpkg_70.HIGH ) 
     then    
       zg_glob2.put_comment(" in reverse lexical order.");
     else
       zg_glob2.put_comment(" in neither lexical nor ");
       zg_glob2.put_new_line;
       zg_glob2.put_ancillary(
          " >>> tk_lf_task_70 reverse lexical order. ");
       zg_glob2.put_new_line;
       zg_glob2.put_ancillary(
          " >>> tk_lf_task_70 first_evaluation_order_array (1) is " & 
          tk_lfpkg_70.routing_priority'image(
          tk_lfpkg_70.first_evaluation_order_array (1) )  );
       zg_glob2.put_new_line;
       zg_glob2.put_ancillary(
          " >>> tk_lf_task_70 first_evaluation_order_array (2) is " & 
          tk_lfpkg_70.routing_priority'image(
          tk_lfpkg_70.first_evaluation_order_array (2) )  );
       zg_glob2.put_new_line;
       zg_glob2.put_ancillary(
          " >>> tk_lf_task_70 first_evaluation_order_array (3) is " & 
                              tk_lfpkg_70.routing_priority'image(
                              tk_lfpkg_70.first_evaluation_order_array (3) )  );
     end if;
  else
     zg_glob2.put_comment(" not consistent.");
  end if;   
  zg_glob2.put_new_line;

            -- need to abort active tasks to permit program termination
  abort tk_lfpkg_70.transmit, 
        tk_lfpkg_70.destination_1, 
        tk_lfpkg_70.destination_2, 
        tk_lfpkg_70.destination_3 ;
  
END tk_lf_task_70 ;
