with zg_glob1;
use  zg_glob1;
with zg_glob2;
with zg_glob3;
with zg_glob4;
with calendar;
use  calendar;

procedure tk_lf_task_71 is

pragma suppress(access_check);
pragma suppress(discriminant_check);
pragma suppress(index_check);
pragma suppress(length_check);
pragma suppress(range_check);
pragma suppress(division_check);
pragma suppress(overflow_check);
pragma suppress(elaboration_check);
pragma suppress(storage_check);

     pragma OPTIMIZE (TIME);

    task BUFFERED_CHANNEL is
         entry PUT (X : in zg_glob1.integer16 );
         entry GET (X : out zg_glob1.integer16 );
    end BUFFERED_CHANNEL;

    task body BUFFERED_CHANNEL is
         N : constant zg_glob1.integer16 := 100;      -- Size of buffer
         BUFFER : array (1 .. N) of zg_glob1.integer16 ;

                          -- For adding and removing things from buffer
         IN_PTR, OUT_PTR : zg_glob1.integer16 := 1;   

                          -- Number of items in buffer
         COUNT : zg_glob1.integer16 range 0 .. N := 0;  
    begin

         loop
              select
                   when COUNT < N =>
                   accept PUT (X : in zg_glob1.integer16 ) do
                        BUFFER (IN_PTR) := X;
                        IN_PTR := IN_PTR mod N + 1;
                        COUNT := COUNT + 1;
                   end;
                or
                   when count > 0 =>
                   accept GET (X : out zg_glob1.integer16 ) do
                        X := BUFFER (OUT_PTR);
                        OUT_PTR := OUT_PTR mod N + 1;
                        COUNT := COUNT - 1;
                   end;
                end select;
         end loop;

 end BUFFERED_CHANNEL;

 -- The following task has two entries, START and STOP.
    -- Both entries are merely synchronization points.
    -- Between these two entries, the task writes to the buffered
    -- channel 1000 times.
    task SERVER is
         entry START;
         entry STOP;
    end SERVER;

    task body SERVER is
    begin
         loop
             accept START;

             -- Write to buffered channel
             for I in 1 .. 1000 loop
                  BUFFERED_CHANNEL.PUT ( zg_glob1.integer16 (1001 - I));
             end loop;

             accept STOP;
         end loop;
    end SERVER;

    -- The following task has two entries, START and STOP.
    -- Both entries are merely synchronization points.
    -- Between these two entries, the task reads from the buffered
    -- channel 1000 times.
    task CONSUMER is
         entry START;
         entry STOP;
    end CONSUMER;

    task body CONSUMER is
         X : zg_glob1.integer16 ;   -- For holding returned variable from READ.
    begin
         loop
             accept START;

             -- Read from buffered channel
             for I in 1 .. 1000 loop
                  BUFFERED_CHANNEL.GET (X);
                  if X /= 1001 - zg_glob1.integer16(I) then
                    raise program_error;
                  end if;
             end loop;

             accept STOP;
         end loop;
    end CONSUMER;

begin

  -- Test Name         :  tk_lf_task_71 
  -- Prime Purpose     :  First of 4 tasks which detect presence of "passive
  --                      task" optimization. This test contains a simple
  --                      task which should be easy to optimize if the system
  --                      is performing the optimization into monitor tasks
  --                      at all.
  -- Optimization      :  
  -- Related tests     :  tk_lf_task_72, tk_lf_task_73, tk_lf_task_74
  -- Author            :  Thomas C. Leavitt
  -- Reviewer          :  B. Decker-Lindsey  -  31 March 1992
  -- Date              :  10 March 1992
  -- Source            :  AES Test TO20, first case
  -- Dependencies      :  
  -- Other Information :  May adapt by using implementation dependent PRAGMA
  --                      to request treatment of the task as "passive" or as
  --                      a monitor task. 

   zg_glob2.put_test_name("tk_lf_task_71");

   pragma include("startime");
                 -- Call START synchronization points to set the SERVER
                 -- and CONSUMER tasks on with their reading and writing
                 -- then call STOPs to synchronize the end of this.
     SERVER.START;
     CONSUMER.START;
     SERVER.STOP;
     CONSUMER.STOP;
   pragma include ("stoptime0");
   zg_glob2.put_comment_optional(
      "   First of 4 tasks which detect presence of ""passive task"" ");
   zg_glob2.put_new_line_optional;
   zg_glob2.put_comment_optional(
     "   optimization. This test contains a simple task which should be ");
   zg_glob2.put_new_line_optional;
   zg_glob2.put_comment_optional(
     "   easy to optimize if the system is performing the optimization into ");
   zg_glob2.put_new_line_optional;
   zg_glob2.put_comment_optional(
     "   monitor tasks at all. The task contains 2004 rendezvous.");
   pragma include ("stoptime2");

   abort BUFFERED_CHANNEL, SERVER, CONSUMER;      -- permit normal termination
exception 
when others => 
   abort BUFFERED_CHANNEL, SERVER, CONSUMER;      -- permit normal termination
end tk_lf_task_71 ;
