with zg_glob1;
use  zg_glob1;
with zg_glob2;
with zg_glob3;
with zg_glob4;
with calendar;
use  calendar;

procedure tk_lf_task_72 is

pragma suppress(access_check);
pragma suppress(discriminant_check);
pragma suppress(index_check);
pragma suppress(length_check);
pragma suppress(range_check);
pragma suppress(division_check);
pragma suppress(overflow_check);
pragma suppress(elaboration_check);
pragma suppress(storage_check);

pragma OPTIMIZE (TIME);

         N : constant zg_glob1.integer16 := 100;      -- Size of buffer
         BUFFER_READ, BUFFER_WRITE : array (1 .. N) of zg_glob1.integer16 
                         := (others => 0 );

                          -- For adding and removing things from buffer
         IN_PTR_WRITE, IN_PTR_READ, 
         OUT_PTR_WRITE, OUT_PTR_READ : zg_glob1.integer16 := 1;   

                          -- Number of items in buffer
         COUNT_WRITE, COUNT_READ : zg_glob1.integer16 range 0 .. N := 0;
 
   procedure BUFFERED_CHANNEL_PUT (X : in zg_glob1.integer16 ) is
   begin
      BUFFER_WRITE (IN_PTR_WRITE) := X;
      IN_PTR_WRITE := IN_PTR_WRITE mod N + 1;
      COUNT_WRITE  := COUNT_WRITE  mod N + 1; 
   end BUFFERED_CHANNEL_PUT ;

   procedure BUFFERED_CHANNEL_GET (X : out zg_glob1.integer16 ) is
   begin
      X := BUFFER_READ (OUT_PTR_READ);
      OUT_PTR_READ := OUT_PTR_READ mod N + 1;
      COUNT_READ   := COUNT_READ   mod N + 1; 
   end BUFFERED_CHANNEL_GET ;

    -- The following task has two entries, START and STOP.
    -- Both entries are merely synchronization points.
    -- Between these two entries, the task writes to the buffered
    -- channel 1000 times.
    task SERVER is
         entry START;
         entry STOP;
    end SERVER;

    task body SERVER is
    begin
         loop
             accept START;

             -- Write to buffered channel
             for I in 1 .. 1000 loop
                  BUFFERED_CHANNEL_PUT ( zg_glob1.integer16 (1001 - I));
             end loop;

             accept STOP;
         end loop;
    end SERVER;


    -- The following task has two entries, START and STOP.
    -- Both entries are merely synchronization points.
    -- Between these two entries, the task reads from the buffered
    -- channel 1000 times.
    task CONSUMER is
         entry START;
         entry STOP;
    end CONSUMER;

    task body CONSUMER is
         X : zg_glob1.integer16 ;   -- For holding returned variable from READ.
    begin
         loop
             accept START;

             -- Read from buffered channel
             for I in 1 .. 1000 loop
                 BUFFERED_CHANNEL_GET (X);
                 if ZG_GLOB3.BOOLEAN_FALSE and then
                       (X /= zg_glob1.integer16(I)) then
                   raise program_error;
                 end if;
             end loop;

             accept STOP;
         end loop;
    end CONSUMER;

begin

  -- Test Name         :  tk_lf_task_72 
  -- Prime Purpose     :  Second of 4 tasks which detect presence of "passive
  --                      task" optimization.  This test problem effectively 
  --                      hand optimizes the entry call into a subprogram call 
  --                      and its time will represent the best which might be 
  --                      hoped for by a "monitor-task" optimization of problem
  --                      tk_lf_task_71.  As coded, this problem is an overly
  --                      optimistic translation of the task performed by the
  --                      other three problems in this set in that it does not
  --                      provide for any mutual exclusion which is necessary
  --                      in the other problems, and because the other problems
  --                      in this set will require additional task switches
  --                      between the SERVER and the CONSUMER tasks (at least
  --                      9 task switches with an optimal schedule and perhaps
  --                      900), and because it avoid checking for buffer 
  --                      overflow (the GET and PUT procedure uses separate 
  --                      variables to avoid exclusion problems - the actual
  --                      operation of this test problem as codes does NOT 
  --                      really represent a correct implementation of the 
  --                      exclusion problem).  The performance of this problem 
  --                      represents a lower bound on the best performance 
  --                      which should be achievable by exploiting passive 
  --                      task optimization and the consequent replacement of 
  --                      rendezvous with simple subprogram calls. 
  -- Optimization      :  
  -- Related tests     :  tk_lf_task_71, tk_lf_task_73, tk_lf_task_74
  -- Author            :  Thomas C. Leavitt
  -- Reviewer          :  B. Decker-Lindsey  -  31 March 1992
  -- Date              :  10 March 1992
  -- Source            :  AES Test TO20, variation of first case 
  -- Dependencies      :  
  -- Other Information :  

   zg_glob2.put_test_name("tk_lf_task_72 ");

   pragma include("startime");
                 -- Call START synchronization points to set the SERVER
                 -- and CONSUMER tasks on with their reading and writing
                 -- then call STOPs to synchronize the end of this.
     SERVER.START;
     CONSUMER.START;
     SERVER.STOP;
     CONSUMER.STOP;
     pragma include ("stoptime0");
   zg_glob2.put_comment_optional(
   "   Second of 4 tasks which detect presence of ""passive task"" ");
   zg_glob2.put_new_line_optional;
   zg_glob2.put_comment_optional(
   "   optimization. This test is ""hand optimized"" to use subprogram calls");
   zg_glob2.put_new_line_optional;
   zg_glob2.put_comment_optional(
   "   rather than rendezvous for the lowest-level process.  It is an overly");
   zg_glob2.put_new_line_optional;
   zg_glob2.put_comment_optional(
   "   optimistic translation of the task performed in the other problems in");
   zg_glob2.put_new_line_optional;
   zg_glob2.put_comment_optional(
   "   this set because it does not provide any mutual exclusion, and because");
   zg_glob2.put_new_line_optional;
   zg_glob2.put_comment_optional(
   "   the other problems in this set will require additional task switches");
   zg_glob2.put_new_line_optional;
   zg_glob2.put_comment_optional(
   "   between the SERVER and the CONSUMER tasks (at least task switches with");
   zg_glob2.put_new_line_optional;
   zg_glob2.put_comment_optional(
   "   an optimal schedule and perhaps 900), and because it avoid checking ");
   zg_glob2.put_new_line_optional;
   zg_glob2.put_comment_optional(
   "   for buffer overflow (the GET and PUT procedure uses separate variables");
   zg_glob2.put_new_line_optional;
   zg_glob2.put_comment_optional(
   "   to avoid exclusion problems - the actual operations of this test ");
   zg_glob2.put_new_line_optional;
   zg_glob2.put_comment_optional(
   "   problem as codes does NOT really represent a correct implementation ");
   zg_glob2.put_new_line_optional;
   zg_glob2.put_comment_optional(
   "   of the exclusion problem).  The performance of this problem as coded ");
   zg_glob2.put_new_line_optional;
   zg_glob2.put_comment_optional(
   "   does however, provide a lower bound to judge the effectiveness of ");
   zg_glob2.put_new_line_optional;
   zg_glob2.put_comment_optional(
   "   the ""monitor-task"" optimization. ");
   pragma include ("stoptime2");

   abort SERVER, CONSUMER;      -- permit normal termination
   exception 
   when others => 
      abort SERVER, CONSUMER;      -- permit normal termination
end tk_lf_task_72 ;
