with zg_glob1;
use  zg_glob1;
with zg_glob2;
with zg_glob3;
with zg_glob4;
with calendar;
use  calendar;

procedure tk_lf_task_79 is

pragma suppress(access_check);
pragma suppress(discriminant_check);
pragma suppress(index_check);
pragma suppress(length_check);
pragma suppress(range_check);
pragma suppress(division_check);
pragma suppress(overflow_check);
pragma suppress(elaboration_check);
pragma suppress(storage_check);

pragma OPTIMIZE (TIME);

v_read, v_write  : zg_glob1.integer16 := 0;

count_of_calls_read, count_of_calls_write : zg_glob1.integer16 := 0;

procedure variable_write ( x : in zg_glob1.integer16 ) is
begin
  v_write := x ; 
  count_of_calls_write := count_of_calls_write + 1;
end variable_write;

procedure variable_read ( x : out zg_glob1.integer16 ) is
begin
  x := v_read ;
  count_of_calls_read := count_of_calls_read + 1;
end variable_read ; 

    -- The following task has two entries, START and STOP.
    -- Both entries are merely synchronization points.
    -- Between these two entries, the task writes to the 
    -- variable 1000 times.
    task SERVER is
         entry START;
         entry STOP;
    end SERVER;

    task body SERVER is
    begin
         loop
             accept START;
             -- Write to variable
             for I in zg_glob1.integer16'(1) .. zg_glob1.integer16'(1000) loop
                  VARIABLE_WRITE ( 1001 - I );
             end loop;
             accept STOP;
         end loop;
    end SERVER;

    -- The following task has two entries, START and STOP.
    -- Both entries are merely synchronization points.
    -- Between these two entries, the task reads from the 
    -- variable 1000 times.
    task CONSUMER is
         entry START;
         entry STOP;
    end CONSUMER;

    task body CONSUMER is
         X : zg_glob1.integer16 ;   -- For holding returned variable from READ.
    begin
         loop
             accept START;

             -- Read from variable
             for I in 1 .. 1000 loop
                  VARIABLE_READ(X);
                  if ZG_GLOB3.BOOLEAN_FALSE then ZG_GLOB4.PROCI1(X); end if; 
             end loop;

             accept STOP;
         end loop;
    end CONSUMER;

begin

  -- Test Name         :  tk_lf_task_79 
  -- Prime Purpose     :  Second of 4 tests which detect presence of "passive
  --                      task" optimization for a task which protects a 
  --                      variable - there is no queue permitting buffered 
  --                      accesses. Two different tasks will be trying to 
  --                      write to and read from the variable a fixed number
  --                      of times. The total time is measured.  This test 
  --                      contains a "hand-optimized" version translating the 
  --                      protecting task into simple procedure calls. It 
  --                      represents an optimistic (lower-bound) implementation
  --                      of the task performed in the 4 tests in this set 
  --                      because if an optimizing compiler was performing
  --                      a "monitor-task" optimization of problem TK_LF_TASK_78
  --                      it would have to provide for a semaphore for mutual
  --                      exclusion.  
  -- Optimization      :  
  -- Related tests     :  tk_lf_task_78, tk_lf_task_80, tk_lf_task_81
  -- Author            :  Thomas C. Leavitt
  -- Reviewer          :  B. Decker-Lindsey  -  1 April 1992
  -- Date              :  18 March 1992
  -- Source            :  AES Test TO19, first case, hand optimized version
  -- Dependencies      :  
  -- Other Information :  

   zg_glob2.put_test_name("tk_lf_task_79");

   zg_glob1.bool := true;

   pragma include("startime");
                 -- Call START synchronization points to set the SERVER
                 -- and CONSUMER tasks on with their reading and writing
                 -- then call STOPs to synchronize the end of this.
     count_of_calls_read := 0;
     count_of_calls_write := 0;
     SERVER.START;
     CONSUMER.START;
     SERVER.STOP;
     CONSUMER.STOP;
     zg_glob1.bool := zg_glob1.bool and 
                      count_of_calls_read  = 1_000 and 
                      count_of_calls_write = 1_000 ;
   pragma include ("stoptime0");
   zg_glob2.put_comment_optional(
      "   Second of 4 tests which detect presence of ""passive task"" ");
   zg_glob2.put_new_line_optional;
   zg_glob2.put_comment_optional(
     "   optimization. This version is ""hand optimized"" to use a simple ");
   zg_glob2.put_new_line_optional;
   zg_glob2.put_comment_optional(
     "   procedure call rather than a rendezvous.  This problem will be ");
   zg_glob2.put_new_line_optional;
   zg_glob2.put_comment_optional(
     "   faster then the others in the set even on systems which implement ");
   zg_glob2.put_new_line_optional;
   zg_glob2.put_comment_optional(
     "   the ""monitor-task"" optimization because it does not set up a");
   zg_glob2.put_new_line_optional;
   zg_glob2.put_comment_optional(
     "   critical section.  It provides a lower bound on the performance of ");
   zg_glob2.put_new_line_optional;
   zg_glob2.put_comment_optional(
     "   this optimization, which will be close to the performance of problem");
   zg_glob2.put_new_line_optional;
   zg_glob2.put_comment_optional(
     "   TK_LF_TASK_78 if the ""monitor-task"" optimization is performed. ");

   pragma include ("stoptime2");

   abort SERVER, CONSUMER;      -- permit normal termination
exception 
when others => 
   abort SERVER, CONSUMER;      -- permit normal termination
end tk_lf_task_79 ;
