with zg_glob1;
use  zg_glob1;
with zg_glob2;
with zg_glob3;
with zg_glob4;
with calendar;
use  calendar;

procedure tk_lf_task_81 is

pragma suppress(access_check);
pragma suppress(discriminant_check);
pragma suppress(index_check);
pragma suppress(length_check);
pragma suppress(range_check);
pragma suppress(division_check);
pragma suppress(overflow_check);
pragma suppress(elaboration_check);
pragma suppress(storage_check);

pragma OPTIMIZE (TIME);

failed : boolean := false ;

    task PROTECTED_VARIABLE is
         entry READ  (X : out zg_glob1.integer16 );
         entry WRITE (X : in  zg_glob1.integer16 );
    end PROTECTED_VARIABLE ;

    task body PROTECTED_VARIABLE is
         V : zg_glob1.integer16 := -1 ;
    begin
         loop
              select
                   accept READ (X : out zg_glob1.integer16 ) do
                        X := V;
                   end;
                   failed := not failed ;
              or
                   accept WRITE (X : in zg_glob1.integer16 ) do
                        V := X;
                   end;
                   failed := not failed ;
              end select;
         end loop;

         accept WRITE (X : in zg_glob1.integer16 ) do
              V := X;
         end;
         failed := not failed ;

    end PROTECTED_VARIABLE ; 

    -- The following task has two entries, START and STOP.
    -- Both entries are merely synchronization points.
    -- Between these two entries, the task writes to the protected 
    -- variable 1000 times.
    task SERVER is
         entry START;
         entry STOP;
    end SERVER;

    task body SERVER is
    begin
         loop
             accept START;
             -- Write to protected variable
             for I in 1 .. 1000 loop
                  PROTECTED_VARIABLE.WRITE ( zg_glob1.integer16 (1001 - I));
             end loop;
             accept STOP;
         end loop;
    end SERVER;

    -- The following task has two entries, START and STOP.
    -- Both entries are merely synchronization points.
    -- Between these two entries, the task reads from the 
    -- protected variable 1000 times.
    task CONSUMER is
         entry START;
         entry STOP;
    end CONSUMER;

    task body CONSUMER is
         X : zg_glob1.integer16 ;   -- For holding returned variable from READ.
    begin
         loop
             accept START;

             -- Read from protected variable
             for I in 1 .. 1000 loop
                  PROTECTED_VARIABLE.READ(X);
                  if ZG_GLOB3.BOOLEAN_FALSE then ZG_GLOB4.PROCI1(X); end if;
             end loop;

             accept STOP;
         end loop;
    end CONSUMER;

begin

  -- Test Name         :  tk_lf_task_81 
  -- Prime Purpose     :  Fourth of 4 tests which detect presence of "passive
  --                      task" optimization for a task which protects a 
  --                      variable - there is no queue permitting buffered 
  --                      accesses. Two different tasks will be trying to 
  --                      write to and read from the variable a fixed number
  --                      of times. The total time is measured.  This test 
  --                      contains a task which has extra statements after the 
  --                      ACCEPT body, making it unlikely that the problem will
  --                      be amenable to the passive task optimization. 
  -- Optimization      :  
  -- Related tests     :  tk_lf_task_78, tk_lf_task_79, tk_lf_task_80
  -- Author            :  Thomas C. Leavitt
  -- Reviewer          :  B. Decker-Lindsey  -  1 April 1992
  -- Date              :  18 March 1992
  -- Source            :  AES Test TO19, third case
  -- Dependencies      :  
  -- Other Information :  May adapt by using implementation dependent PRAGMA
  --                      to request treatment of the task as "passive" or as
  --                      a monitor task. 

   zg_glob2.put_test_name("tk_lf_task_81");

   pragma include("startime");
                 -- Call START synchronization points to set the SERVER
                 -- and CONSUMER tasks on with their reading and writing
                 -- then call STOPs to synchronize the end of this.
     SERVER.START;
     CONSUMER.START;
     SERVER.STOP;
     CONSUMER.STOP;
   pragma include ("stoptime0");
   zg_glob2.put_comment_optional(
   "   Fourth of 4 tests which detect presence of ""passive task"" ");
   zg_glob2.put_new_line_optional;
   zg_glob2.put_comment_optional(
   "   optimization. This test contains a task functioning to protect");
   zg_glob2.put_new_line_optional;
   zg_glob2.put_comment_optional(
   "   a variable.  The task contains code after the ACCEPT body which will ");
   zg_glob2.put_new_line_optional;
   zg_glob2.put_comment_optional(
   "   often inhibit the application of the ""passive task"" optimization. ");
   zg_glob2.put_new_line_optional;
   zg_glob2.put_comment_optional(
   "   The task contains 2004 rendezvous.");
   pragma include ("stoptime2");

   abort PROTECTED_VARIABLE, SERVER, CONSUMER;      -- permit normal termination
exception 
when others => 
   abort PROTECTED_VARIABLE, SERVER, CONSUMER;      -- permit normal termination
end tk_lf_task_81 ;
