WITH zg_glob1;
USE  zg_glob1;
WITH zg_glob2;
WITH zg_glob3;
with zg_glob4;

WITH calendar;
USE  calendar;
 
PROCEDURE tk_lf_task_82 IS
 
  PRAGMA suppress(access_check);
  PRAGMA suppress(discriminant_check);
  PRAGMA suppress(index_check);
  PRAGMA suppress(length_check);
  PRAGMA suppress(range_check);
  PRAGMA suppress(division_check);
  PRAGMA suppress(overflow_check);
  PRAGMA suppress(elaboration_check);
  PRAGMA suppress(storage_check);
 
  TASK task_1 IS
    PRAGMA priority( zg_glob1.priority_1 );
    ENTRY phase_1 ;
  END task_1 ;
 
  TASK task_2 IS
    PRAGMA priority( zg_glob1.priority_2 );
    ENTRY phase_2 ;
  END task_2 ;
 
  TASK BODY task_2 IS
  BEGIN
    LOOP
      task_1.phase_1;  -- make entry call
      ACCEPT phase_2 
      do 
        null;
      end;
                  -- In this problem, the following IF statement with a
                  -- tasking option will occur within the rendezvous.  It 
                  -- occurs here to make the related programs more comparable
                  -- (they will execute the same statements, but in one version
                  -- they will occur within a rendezvous, and in others they 
                  -- occur outside the rendezvous.) 
      if zg_glob3.boolean_false then delay 0.1; zg_glob4.die; end if;
    END LOOP;
  END task_2 ;
 
  TASK BODY task_1 IS
  BEGIN
 
 
    --
    -- Test Name         :  tk_lf_task_82
    -- Prime Purpose     :  language feature test of aspects of tasking.  
    --                      This is the first of three versions of a problem
    --                      with varying degrees of complexity of code within
    --                      a rendezvous in order to test for optimization. 
    --                      This version has an explicit NULL sequence of 
    --                      statements in the rendezvous. That is "DO NULL; 
    --                      END;".  The problem performs 20 rendezvous, in 
    --                      half of these the task making the entry call 
    --                      arrives at the rendezvous first, and in the other 
    --                      half the task making the ACCEPT arrives first. 
    -- Optimization      :
    -- Related tests     :  tk_lf_task_83, tk_lf_task_84
    -- Author            :  Thomas C. Leavitt
    -- Reviewer          :  Julie Leastman - 8/21/92
    -- Date              :  16 June 1992
    -- Source            :  AES Test TK02, with a simpler rendezvous
    -- Dependencies      :
    -- Other Information :
    --
  
    ----------------------------------------------------------------------------
    zg_glob2.put_test_name("tk_lf_task_82");

    PRAGMA include("startime");

    FOR i IN 1 .. 10 LOOP
      ACCEPT phase_1 
      do 
        null;
      end;
                  -- In this problem, the following IF statement with a
                  -- tasking option will occur within the rendezvous.  It 
                  -- occurs here to make the related programs more comparable
                  -- (they will execute the same statements, but in one version
                  -- they will occur within a rendezvous, and in others they 
                  -- occur outside the rendezvous.) 
      if zg_glob3.boolean_false then delay 0.1; zg_glob4.die; end if;
      task_2.phase_2 ;
    END LOOP;

    PRAGMA include("stoptime0");

    zg_glob2.put_comment_optional(
       "   Rendezvous with explicit ""DO NULL; END;"" within the rendezvous.");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional(
       "   For comparison with other test problems to determine whether ");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional(
       "   a compiler is optimizing rendezvous code. ");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional("        FOR i IN 1..10");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional("        LOOP");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional("           ACCEPT phase_1 ");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional("           DO ");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional("            NULL; ");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional("           END;");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional(
       "                -- The following line occurs here for comparability ");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional(
       "                -- with other versions of the problem. ");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional(
 "           if zg_glob3.boolean_false then delay 0.1; zg_glob4.die; end if;");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional("           task_2.phase_2 ;");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional("        END LOOP;");
    zg_glob2.put_new_line_optional;

    PRAGMA include("stoptime2");
    ----------------------------------------------------------------------------
 
    zg_glob2.put_ancillary(" >>> tk_lf_task_82 time per rendezvous = ");
    zg_glob2.put_value( (zg_glob3.min_time * 1000000.0 / 20.0), 8, 1, 0);
    zg_glob2.put_new_line;
 
    ABORT task_2 ;    -- stop execution
  END task_1 ;
 
BEGIN
  NULL;
END tk_lf_task_82;

--------------------------------------------------------------------------------
