with zg_glob1;
use  zg_glob1;
with zg_glob2;
with zg_glob3;
with zg_glob4;
with calendar;
use  calendar;
--with system;                    --+    A "WITH SYSTEM;" may be necessary on
                                  --+    some compilers to use PRAGMA PRIORITY
                                  --+    even though Ada Interpretation
                                  --+    AI-00197 clearly states that it should
                                  --+    NOT be required.

pragma elaborate(zg_glob1, zg_glob2, zg_glob3, zg_glob4);

package TK_lfpkg_88 is

  task task_1 is
    PRAGMA priority( zg_glob1.priority_1 );
    entry START_PROGRAM;
    entry STOP_PROGRAM;
    ENTRY request;
    ENTRY release;
  end task_1 ;

  TASK task_2 is
    PRAGMA priority( zg_glob1.priority_2 );
    ENTRY request;
    ENTRY release;
  end task_2 ;

  TASK task_3 is
    PRAGMA priority( zg_glob1.priority_1 );
  end task_3 ;

  TASK task_4 is
    PRAGMA priority( zg_glob1.priority_1 );
  end task_4 ;

end TK_lfpkg_88 ;

--------------------------------------------------------------------------------

package body tk_lfpkg_88 is

  PRAGMA suppress(access_check);
  PRAGMA suppress(discriminant_check);
  PRAGMA suppress(index_check);
  PRAGMA suppress(length_check);
  PRAGMA suppress(range_check);
  PRAGMA suppress(division_check);
  PRAGMA suppress(overflow_check);
  PRAGMA suppress(elaboration_check);
  PRAGMA suppress(storage_check);

  TASK BODY task_3 IS
  BEGIN
    LOOP
      task_1.release;
    END LOOP;
  exception
  when tasking_error => null;
  END task_3; 

  TASK BODY task_4 IS
  BEGIN
    LOOP
      task_2.release;
    END LOOP;
  exception
  when tasking_error => null;
  END task_4; 

  TASK BODY task_2 IS
  BEGIN
    LOOP
      ACCEPT request;
      ACCEPT release; 
      task_1.request;
    END LOOP;
  END task_2 ; 

  TASK BODY task_1 is
  BEGIN

  -- Test Name         :  TK_LF_TASK_88
  -- Prime Purpose     :  This is the second of two problems designed to
  --                      determine whether there is a performance overhead
  --                      associated with performing an ACCEPT in a rendezvous
  --                      (versus completing the rendezvous and performing an
  --                      ACCEPT.) This version does not nest ACCEPT statements.
  --                      In half the cases, the task performing the entry call
  --                      will arrive at the rendezvous first and in the other
  --                      half, the task performing the ACCEPT will arrive 1st.
  -- Optimization      :
  -- Related tests     :  TK_LF_TASK_87
  -- Author            :  Thomas C. Leavitt
  -- Reviewer          :  Julie Leastman 8/25/92
  -- Date              :  8 JUNE 1992 
  -- Source            :  AES test TL11 
  -- Dependencies      :
  -- Other Information :

--------------------------------------------------------------------------------

  ACCEPT start_program;

  zg_glob2.put_test_name("tk_lf_task_88 ");
  pragma include("startime");
      task_2.request;
      ACCEPT request;
      ACCEPT release; 
  pragma include("stoptime0");
    zg_glob2.put_comment_optional(
       "   This is the second of two test problems designed to determine  ");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional(
       "   whether there is a performance overhead associated with  ");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional(
       "   performing an ACCEPT in a rendezvous versus completing the  ");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional(
       "   rendezvous and performing an ACCEPT. This version does NOT nest ");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional(
       "   ACCEPT statements. The problem performs four rendezvous. ");
    zg_glob2.put_new_line_optional;
  pragma include("stoptime2");

  ACCEPT stop_program;
  abort task_2, task_3, task_4; 

end task_1 ;

end tk_lfpkg_88 ;

--------------------------------------------------------------------------------

with tk_lfpkg_88 ;
procedure tk_lf_task_88 is

  PRAGMA suppress(access_check);
  PRAGMA suppress(discriminant_check);
  PRAGMA suppress(index_check);
  PRAGMA suppress(length_check);
  PRAGMA suppress(range_check);
  PRAGMA suppress(division_check);
  PRAGMA suppress(overflow_check);
  PRAGMA suppress(elaboration_check);
  PRAGMA suppress(storage_check);

begin
  tk_lfpkg_88.task_1.start_program; 
  tk_lfpkg_88.task_1.stop_program;
end tk_lf_task_88 ;
