-- +++ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
-- +++ tcl 2-8-89 change to make ancillary information self-identifying
-- tk_ls_task_12
--  modeled after EXAMPLE2 in Task Management in Ada, E. Roberts, A. Evans,
--  C. Morgan, and E. Clarke in Software: Practice and Experience, Volume 11,
--  Number 10, Oct 1981.
 
--  simple_task can be Habermann/Nassi-ized with slightly more difficulty
--  than tk_lf_task_11
 
WITH zg_glob1;
USE  zg_glob1;
WITH zg_glob2;
USE  zg_glob2;
WITH zg_glob3;
USE  zg_glob3;
WITH zg_glob4;
USE  zg_glob4;

WITH calendar;
USE  calendar;
WITH system;
 
PROCEDURE tk_ls_task_12 IS
 
  PRAGMA suppress(access_check);
  PRAGMA suppress(discriminant_check);
  PRAGMA suppress(index_check);
  PRAGMA suppress(length_check);
  PRAGMA suppress(range_check);
  PRAGMA suppress(division_check);
  PRAGMA suppress(overflow_check);
  PRAGMA suppress(elaboration_check);
  PRAGMA suppress(storage_check);
 
  TASK simple_task IS
         --+ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
      PRAGMA priority( zg_glob1.priority_2 );
    ENTRY entry1;
    ENTRY entry2;
    ENTRY entry3;
    ENTRY entry4;
    ENTRY entry5;
  END simple_task;
 
  TASK BODY simple_task IS
  BEGIN
    LOOP
      SELECT
        ACCEPT entry1 DO
          zg_glob4.ii := 1;
          zg_glob4.proc0;
        END entry1;
      OR
        ACCEPT entry2 DO
          zg_glob4.ii := 2;
          zg_glob4.proc0;
        END entry2;
      OR
        ACCEPT entry3 DO
          zg_glob4.ii := 3;
          zg_glob4.proc0;
        END entry3;
      OR
        ACCEPT entry4 DO
          zg_glob4.ii := 4;
          zg_glob4.proc0;
        END entry4;
      OR
        ACCEPT entry5 DO
          zg_glob4.ii := 5;
          zg_glob4.proc0;
        END entry5;
      END SELECT;
    END LOOP;
  END simple_task;
 
  TASK main IS
         --+ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
      PRAGMA priority( zg_glob1.priority_1 );
  END main;
 
  TASK BODY main IS
  BEGIN
 
 
    --
    -- Test Name         :  tk_ls_task_12
    -- Prime Purpose     :  Language feature test of aspects of tasking.
    --                      Entering tasks make a sequence of entry calls.
    --                      Accepting task contains a SELECT with a list of
    --                      alternatives with a DO containing a simple code
    --                      sequence.  Accepting task will arrive at rendezvous
    --                      first.
    -- LRM Version       :  Revision A
    -- Optimization      :  Habermann/Nassi
    -- Related tests     :
    -- Author            :  Thomas C. Leavitt
    -- Reviewer          :  Kermit E. Terrell
    -- Date              :
    -- Source            :  "Task Management in Ada," by E. Roberts, A. Evans,
    --                          C. Morgan, and E. Clarke, Software:  Practice
    --                          and Experience, volume 11, number 10, October
    --                          1981.
    -- Dependencies      :
    -- Other Information :  Modeled after Example2 in the above citation.
    --
 
    ----------------------------------------------------------------------------
    zg_glob2.put_test_name("tk_ls_task_12");

    PRAGMA include("startime");

    simple_task.entry1;
    simple_task.entry2;
    simple_task.entry3;
    simple_task.entry4;
    simple_task.entry5;

    PRAGMA include("stoptime0");

    zg_glob2.put_comment_optional
	("   simple_task.entry1;  simple_task.entry2;  simple_task.entry3;");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("   simple_task.entry4;  simple_task.entry5;");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("   -- select with alternatives in the receiving task. Priority is");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("   -- such that accepting task will arrive at rendezvous first");
    zg_glob2.put_new_line_optional;

    PRAGMA include("stoptime2");
    ----------------------------------------------------------------------------
 
		----------------------------------------------------
		-- +++ tcl 2-8-89 make ancillary information self-identifying
    zg_glob2.put_ancillary(" >>> tk_ls_task_12 time per rendezvous = ");
    zg_glob2.put_value ( (zg_glob3.min_time * 1_000_000.0 / 5.0), 8, 1, 0);
    zg_glob2.put_new_line;
                ----------------------------------------------------
 
    ABORT simple_task;    -- allow normal termination
  END main;
 
BEGIN
  NULL;
END tk_ls_task_12;
