-- +++ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
-- tk_ls_task_15
--  This is an example of conditional selects with no open alternatives,
--  and no entry calls have been made
 
WITH zg_glob1;
USE  zg_glob1;
WITH zg_glob2;
USE  zg_glob2;
WITH zg_glob3;
USE  zg_glob3;
WITH zg_glob4;

WITH calendar;
USE  calendar;
WITH system;
 
PROCEDURE tk_ls_task_15 IS
 
  PRAGMA suppress(access_check);
  PRAGMA suppress(discriminant_check);
  PRAGMA suppress(index_check);
  PRAGMA suppress(length_check);
  PRAGMA suppress(range_check);
  PRAGMA suppress(division_check);
  PRAGMA suppress(overflow_check);
  PRAGMA suppress(elaboration_check);
  PRAGMA suppress(storage_check);
 
  TASK main IS
         --+ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
      PRAGMA priority( zg_glob1.priority_1 );
  END main;
 
  TASK simple_task IS
         --+ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
      PRAGMA priority( zg_glob1.priority_2 );
    ENTRY entry1;
    ENTRY entry2;
    ENTRY entry3;
    ENTRY entry4;
    ENTRY entry5;
  END simple_task;
 
  TASK BODY simple_task IS
  BEGIN
 
 
    --
    -- Test Name         :  tk_ls_task_15
    -- Prime Purpose     :  Language feature test of aspects of tasking.
    -- Entering tasks make a sequence of entry calls.  Accepting task
    --                      contains a conditional select statement with WHEN
    --                      clauses, none of which is satisfied.  Accepting task
    --                      will arrive at rendezvous first.
    -- LRM Version       :  A
    -- Optimization      :
    -- Related tests     :
    -- Author            :  Thomas C. Leavitt
    -- Reviewer          :  Kermit E. Terrell
    -- Date              :
    -- Source            :
    -- Dependencies      :
    -- Other Information :
    --
 
    zg_glob4.ii := 0;    -- this setting will make all the alternatives closed
 
    ----------------------------------------------------------------------------
    zg_glob2.put_test_name("tk_ls_task_15");

    PRAGMA include("startime");

    SELECT
      WHEN zg_glob4.ii = 1 =>
        ACCEPT entry1 DO
          zg_glob4.ii := 2;
          zg_glob4.proc0;
        END entry1;
    OR
      WHEN zg_glob4.ii = 2 =>
        ACCEPT entry2 DO
          zg_glob4.ii := 3;
          zg_glob4.proc0;
        END entry2;
    OR
      WHEN zg_glob4.ii = 3 =>
        ACCEPT entry3 DO
          zg_glob4.ii := 4;
          zg_glob4.proc0;
        END entry3;
    OR
      WHEN zg_glob4.ii = 4 =>
        ACCEPT entry4 DO
          zg_glob4.ii := 5;
          zg_glob4.proc0;
        END entry4;
    OR
      WHEN zg_glob4.ii = 5 =>
        ACCEPT entry5 DO
          zg_glob4.ii := 1;
          zg_glob4.proc0;
        END entry5;
      ELSE
        zg_glob4.proc0;
    END SELECT;
 
    PRAGMA include("stoptime0");

    zg_glob2.put_comment_optional
	("   conditional select with no open alternatives");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("       -- no tasks waiting on entries");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("   SELECT");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("       WHEN ii = 1 =>");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("         ACCEPT entry1 DO ii := 2;  proc0;  END entry1;");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("     OR");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("       WHEN ii = 2 =>");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("         ACCEPT entry2 DO ii := 3;  proc0;  END entry2;");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("     OR");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("       WHEN ii = 3 =>");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("         ACCEPT entry3 DO ii := 4;  proc0;  END entry3;");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("     OR");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("       WHEN ii = 4 =>");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("         ACCEPT entry4 DO ii := 5;  proc0;  END entry4;");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("     OR");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("       WHEN ii = 5 =>");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("         ACCEPT entry5 DO ii := 1;  proc0;  END entry5;");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("     ELSE");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("       proc0; -- selected entry");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("   END SELECT;");
    zg_glob2.put_new_line_optional;

    PRAGMA include("stoptime2");
    ----------------------------------------------------------------------------
 
    ABORT main;
  END simple_task;
 
  TASK BODY main IS      -- make sure that there appears to be calls on
                         -- entries in simple_task, or optimizing compiler
  BEGIN                  -- might detect that they are unreachable
    LOOP
      IF zg_glob4.yy = zg_glob4.zz THEN    -- never true
        simple_task.entry1;
        simple_task.entry2;
        simple_task.entry3;
        simple_task.entry4;
        simple_task.entry5;
      ELSE
        DELAY 10.0;
      END IF;
    END LOOP;
  END main;
 
BEGIN
  NULL;    -- will terminate when task enclosed in it terminates
END tk_ls_task_15;
