-- +++ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
--+   TJT  06/24/88  Changed to reflect SPR # 24
-- tk_ls_task_19
--  This is an example of conditional selects with one alternative open
--  and entry calls waiting on all alternatives
 
WITH zg_glob1;
USE  zg_glob1;
WITH zg_glob2;
USE  zg_glob2;
WITH zg_glob3;
USE  zg_glob3;
WITH zg_glob4;

WITH calendar;
USE  calendar;
WITH system;
 
PROCEDURE tk_ls_task_19 IS
 
  PRAGMA suppress(access_check);
  PRAGMA suppress(discriminant_check);
  PRAGMA suppress(index_check);
  PRAGMA suppress(length_check);
  PRAGMA suppress(range_check);
  PRAGMA suppress(division_check);
  PRAGMA suppress(overflow_check);
  PRAGMA suppress(elaboration_check);
  PRAGMA suppress(storage_check);
 
  TASK simple_task IS
         --+ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
      PRAGMA priority( zg_glob1.priority_1 );
    ENTRY entry1;
    ENTRY entry2;
    ENTRY entry3;
    ENTRY entry4;
    ENTRY entry5;
  END simple_task;
 
  TASK t1 IS
         --+ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
      PRAGMA priority( zg_glob1.priority_2 );
  END t1;
 
  TASK BODY t1 IS
  BEGIN
    LOOP
      simple_task.entry1;
      zg_glob4.ii := 2;
      zg_glob4.proc0;
    END LOOP;
  END t1;
 
  TASK t2 IS
         --+ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
      PRAGMA priority( zg_glob1.priority_2 );
  END t2;
 
  TASK BODY t2 IS
  BEGIN
    LOOP
      simple_task.entry2;
      zg_glob4.ii := 3;
      zg_glob4.proc0;
    END LOOP;
  END t2;
 
  TASK t3 IS
         --+ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
      PRAGMA priority( zg_glob1.priority_2 );
  END t3;
 
  TASK BODY t3 IS
  BEGIN
    LOOP
      simple_task.entry3;
      zg_glob4.ii := 4;
      zg_glob4.proc0;
    END LOOP;
  END t3;
 
  TASK t4 IS
         --+ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
      PRAGMA priority( zg_glob1.priority_2 );
  END t4;
 
  TASK BODY t4 IS
  BEGIN
    LOOP
      simple_task.entry4;
      zg_glob4.ii := 5;
      zg_glob4.proc0;
    END LOOP;
  END t4;
 
  TASK t5 IS
         --+ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
      PRAGMA priority( zg_glob1.priority_2 );
  END t5;
 
  TASK BODY t5 IS
  BEGIN
    LOOP
      simple_task.entry5;
      zg_glob4.ii := 6;
      zg_glob4.proc0;
    END LOOP;
  END t5;
 
  TASK BODY simple_task IS
  BEGIN
 
 
    --
    -- Test Name         :  tk_ls_task_19
    -- Prime Purpose     :  Language feature test of aspects of tasking.
    --                      Accepting task contains a conditional SELECT 
    --                      statement with WHEN clauses, only one of which is
    --                      satisfied. Should be tasks waiting on all entries.  
    --			    Accepting task will arrive at rendezvous first.  
    --                      Test measures evaluation of alternative guards and 
    --                      picking one alternative. 
    -- LRM Version       :  A
    -- Revision          :  24 June, 1988   SPR 24   Tamera J. Terrell
    --                      Changed error message to include diagnostic
    --                      information.
    -- Optimization      :
    -- Related tests     :
    -- Author            :  Thomas C. Leavitt
    -- Reviewer          :  Kermit E. Terrell
    -- Date              :
    -- Source            :
    -- Dependencies      :
    -- Other Information :
    --
 
    ----------------------------------------------------------------------------
    zg_glob2.put_test_name("tk_ls_task_19");

    PRAGMA include("startime");

    zg_glob4.ii := 1;
    LOOP
      SELECT
        WHEN zg_glob4.ii = 1 =>
          ACCEPT entry1 DO
            zg_glob4.proci1 (zg_glob4.ii);
          END entry1;         -- code in entry (i) sets zg_glob4.ii to i+1
      OR
        WHEN zg_glob4.ii = 2 =>
          ACCEPT entry2 DO
            zg_glob4.proci1 (zg_glob4.ii);
          END entry2;
      OR
        WHEN zg_glob4.ii = 3 =>
          ACCEPT entry3 DO
            zg_glob4.proci1 (zg_glob4.ii);
          END entry3;
      OR
        WHEN zg_glob4.ii = 4 =>
          ACCEPT entry4 DO
            zg_glob4.proci1 (zg_glob4.ii);
          END entry4;
      OR
        WHEN zg_glob4.ii = 5 =>
          ACCEPT entry5 DO
            zg_glob4.proci1 (zg_glob4.ii);
          END entry5;
        ELSE
          IF zg_glob4.ii /= 6 THEN
                                                --+     
            zg_glob2.put_comment_optional("ERROR: -  entry not waiting ");    
            zg_glob2.put_new_line_optional;
            --put_line("error in tk_ls_task_19 - entry not waiting"); 
                                                --+ TJT 06/24/88 
            zg_glob2.put_error_code (zg_glob1.err_at_execution_time);   
						--+ TJT 06/24/88
            zg_glob4.die;
          END IF;
      END SELECT;
      EXIT WHEN zg_glob4.ii NOT IN 1 .. 5;      -- accept 5 entries and stop
    END LOOP;
 
    PRAGMA include("stoptime0");

    zg_glob2.put_comment_optional
	("   conditional select with one open alternative");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("   -- tasks waiting at all entries");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("   -- setup to cycle through each alternative");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("   -- task priorities on t(i) and simple_task insure that");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("   -- task t(i) should be waiting on entries in the accepts");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("   ii := 1;");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("   LOOP");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("    SELECT");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("      WHEN ii = 1 =>");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("        ACCEPT entry1 DO proci1(ii);  END entry1;");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("              -- code in entry (i) sets ii to i+1");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("    OR");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("      WHEN ii = 2 =>");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("        ACCEPT entry2 DO proci1(ii);  END entry2;");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("    OR");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("      WHEN ii = 3 =>");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("        ACCEPT entry3 DO proci1(ii);  END entry3;");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("    OR");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("      WHEN ii = 4 =>");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("        ACCEPT entry4 DO proci1(ii);  END entry4;");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("    OR");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("      WHEN ii = 5 =>");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("        ACCEPT entry5 DO proci1(ii);  END entry5;");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("    ELSE");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("      IF ii /=6 THEN ");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("        put_line(""error in tk_ls_task_19 - entry not waiting"");");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("        die; ");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("      END IF;");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("    END SELECT;");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("    EXIT WHEN ii NOT IN 1..5;  -- accept 5 entries and stop");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("  END LOOP;");
    zg_glob2.put_new_line_optional;

    PRAGMA include("stoptime2");
    ----------------------------------------------------------------------------
 
    ABORT t1, t2, t3, t4, t5;
  END simple_task;
 
BEGIN
  NULL;    -- will terminate when task enclosed in it terminates
END tk_ls_task_19;
