-- +++ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
-- +++ tcl 2-8-89 change to make ancillary information self-identifying
--+ tcl 12-07-88 modified to reflect SPR#25
 
-- tk_ls_task_20
-- compare with tk_ls_task_14, same with relative priority of tasks simple_task,
-- and main reversed
 
--  modeled after EXAMPLE4 in Task Management in Ada, E. Roberts, A. Evans,
--  C. Morgan, and E. Clarke in Software: Practice and Experience, Volume 11,
--  Number 10, Oct 1981.
 
--  simple_task can be Habermann/Nassi-ized with slightly more difficulty
--  This is a variant of EXAMPLE2 (tk_ls_task_12) with WHEN clauses on the accepts
 
WITH zg_glob1;
USE  zg_glob1;
WITH zg_glob2;
USE  zg_glob2;
WITH zg_glob3;
USE  zg_glob3;
WITH zg_glob4;

WITH calendar;
USE  calendar;
WITH system;
 
PROCEDURE tk_ls_task_20 IS
 
  PRAGMA suppress(access_check);
  PRAGMA suppress(discriminant_check);
  PRAGMA suppress(index_check);
  PRAGMA suppress(length_check);
  PRAGMA suppress(range_check);
  PRAGMA suppress(division_check);
  PRAGMA suppress(overflow_check);
  PRAGMA suppress(elaboration_check);
  PRAGMA suppress(storage_check);
 
  i : zg_glob1.integer16 := 1;    -- variable used to cycle between entries
 
  TASK simple_task IS
         --+ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
      PRAGMA priority( zg_glob1.priority_2 );
                   -- with this lower than timing loop, the
                   -- code should be waiting for entries
    ENTRY entry1;
    ENTRY entry2;
    ENTRY entry3;
    ENTRY entry4;
    ENTRY entry5;
  END simple_task;
 
  TASK BODY simple_task IS    -- if this task starts first, it should
  BEGIN                       -- wait on entry1
    LOOP
      SELECT
        WHEN i = 1 =>
          ACCEPT entry1 DO
            i := 2;
            zg_glob4.proc0;
          END entry1;
      OR
        WHEN i = 2 =>
          ACCEPT entry2 DO
            i := 3;
            zg_glob4.proc0;
          END entry2;
      OR
        WHEN i = 3 =>
          ACCEPT entry3 DO
            i := 4;
            zg_glob4.proc0;
          END entry3;
      OR
        WHEN i = 4 =>
          ACCEPT entry4 DO
            i := 5;
            zg_glob4.proc0;
          END entry4;
      OR
        WHEN i = 5 =>
          ACCEPT entry5 DO
            i := 1;
            zg_glob4.proc0;
          END entry5;
      END SELECT;
    END LOOP;
  END simple_task;
 
  TASK main IS
         --+ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
      PRAGMA priority( zg_glob1.priority_1 );
  END main;
 
  TASK BODY main IS
  BEGIN
 
 
    --
    -- Test Name         :  tk_ls_task_20
    -- Prime Purpose     :  Language feature test of aspects of tasking. 
    --                      Entering tasks makes sequence of entry calls.  
    --                      Accepting task
    --                      contains a SELECT statement with WHEN clauses, only
    --                      one of which is satisfied.  Entering task will
    --                      arrive at rendezvous first.
    -- LRM Version       :  A
    -- Optimization      :  Habermann/Nassi-ized ( can be done to simple task
    --                      with slightly more difficulty )
    -- Related tests     :  tk_ls_task_14 , tk_ls_task_12
    --                      Compare with tk_ls_task_14, same with relative 
    --                      priority of tasks simple_task, and main reversed.  
    --                      This is a variant of tk_ls_task_12 with WHEN 
    --                      clauses on the ACCEPTs.
    -- Author            :  Thomas C. Leavitt
    -- Reviewer          :  Kermit E. Terrell
    -- Date              :
    -- Source            :  "Task Management in Ada," by E. Roberts, A. Evans,
    --                      C. Morgan, and E. Clarke, Software:  Practice and
    --                      Experience, volume 11, number 10, October 1981.
    -- Dependencies      :
    -- Other Information :  Modeled after Example4 in the above citation.
    --
  
    ----------------------------------------------------------------------------
    zg_glob2.put_test_name("tk_ls_task_20");

    PRAGMA include("startime");

    simple_task.entry1;
    simple_task.entry2;
    simple_task.entry3;
    simple_task.entry4;
    simple_task.entry5;

    PRAGMA include("stoptime0");

    zg_glob2.put_comment_optional
	("   simple_task.entry1; simple_task.entry2; simple_task.entry3;");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("   simple_task.entry4; simple_task.entry5;");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("   -- conditional selects in the receiving task");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("   -- same as tk_ls_task_14 with priority of the accepting and ");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("   -- entering tasks reversed.");
    zg_glob2.put_new_line_optional;
 
    --+ tcl 12-07-88 modified to reflect SPR#25
    --+              changed working of message
    zg_glob2.put_comment_optional
	("    -- assignment of priorities are such that the task ");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("    -- performing ENTRY call will arrive at rendezvous before ");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional("    -- the task performing the ACCEPT. ");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional("    -- cf. tk_ls_task_14 ");
    zg_glob2.put_new_line_optional;

    PRAGMA include("stoptime2");
    ----------------------------------------------------------------------------
 
		----------------------------------------------------
		-- +++ tcl 2-8-89 make ancillary information self-identifying
    zg_glob2.put_ancillary(" >>> tk_ls_task_20 time per rendezvous = ");
    zg_glob2.put_value ( (zg_glob3.min_time * 1_000_000.0 / 5.0), 8, 1, 0);
    zg_glob2.put_new_line;
                ----------------------------------------------------
 
    ABORT simple_task;    -- permit normal termination
  END main;
 
BEGIN
  NULL;
END tk_ls_task_20;
