-- +++ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
-- +++ tcl 2-8-89 change to make ancillary information self-identifying
-- tk_ls_task_21
 
--  This is an example of conditional selects with one alternative open
--  and entry calls waiting on all alternatives
 
WITH zg_glob1;
USE  zg_glob1;
WITH zg_glob2;
USE  zg_glob2;
WITH zg_glob3;
USE  zg_glob3;
WITH zg_glob4;

WITH calendar;
USE  calendar;
WITH system;
 
PROCEDURE tk_ls_task_21 IS
 
  PRAGMA suppress(access_check);
  PRAGMA suppress(discriminant_check);
  PRAGMA suppress(index_check);
  PRAGMA suppress(length_check);
  PRAGMA suppress(range_check);
  PRAGMA suppress(division_check);
  PRAGMA suppress(overflow_check);
  PRAGMA suppress(elaboration_check);
  PRAGMA suppress(storage_check);
 
  i : zg_glob1.integer16 := 1;
 
  TASK simple_task IS
         --+ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
      PRAGMA priority( zg_glob1.priority_1 );
    ENTRY entry1;
    ENTRY entry2;
    ENTRY entry3;
    ENTRY entry4;
    ENTRY entry5;
  END simple_task;
 
  TASK t1 IS
         --+ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
      PRAGMA priority( zg_glob1.priority_2 );
  END t1;
 
  TASK BODY t1 IS
  BEGIN
 
    --
    -- Test Name         :  tk_ls_task_21
    -- Prime Purpose     :  Language feature test of aspects of tasking. 
    --                      Entering tasks make a sequence of entry calls.   
    --                      Accepting task
    --                      contains conditional SELECT with guards, only one of
    --                      which is satisfied.  Entering task will arrive at
    --                      Entering task will arrive at rendezvous first.
    -- LRM Version       :  A
    -- Optimization      :
    -- Related tests     :  tk_ls_task_22
    -- Author            :  Thomas C. Leavitt
    -- Reviewer          :  Kermit E. Terrell
    -- Date              :
    -- Source            :
    -- Dependencies      :
    -- Other Information :
    --
  
    ----------------------------------------------------------------------------
    zg_glob2.put_test_name("tk_ls_task_21");

    PRAGMA include("startime");

    FOR i IN 1 .. 10 LOOP
      simple_task.entry1;
      simple_task.entry2;
      simple_task.entry3;
      simple_task.entry4;
      simple_task.entry5;
    END LOOP;

    PRAGMA include("stoptime0");

    zg_glob2.put_comment_optional("   conditional select, one open alternative, one task");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional("   -- waiting on entry, task doing entry will be first at");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional("   -- rendezvous.  Compare with tk_ls_task_22 ");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional("   FOR i IN 1..10");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional("   LOOP");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("     simple_task.entry1;  simple_task.entry2;  simple_task.entry3;");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("     simple_task.entry4;  simple_task.entry5;");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional("   END LOOP;");

    PRAGMA include("stoptime2");
    ----------------------------------------------------------------------------
 
		----------------------------------------------------
		-- +++ tcl 2-8-89 make ancillary information self-identifying
    zg_glob2.put_ancillary(" >>> tk_ls_task_21 time per rendezvous = ");
    zg_glob2.put_value( (zg_glob3.min_time * 1_000_000.0 / 50.0), 8, 1, 0);
    zg_glob2.put_new_line;
		----------------------------------------------------
 
    ABORT simple_task;
  END t1;
 
  TASK BODY simple_task IS
 
  BEGIN
    LOOP
      SELECT
        WHEN i = 1 =>
          ACCEPT entry1 DO
            i := 2;
            zg_glob4.proci1(i);
          END entry1;
      OR
        WHEN i = 2 =>
          ACCEPT entry2 DO
            i := 3;
            zg_glob4.proci1(i);
          END entry2;
      OR
        WHEN i = 3 =>
          ACCEPT entry3 DO
            i := 4;
            zg_glob4.proci1(i);
          END entry3;
      OR
        WHEN i = 4 =>
          ACCEPT entry4 DO
            i := 5;
            zg_glob4.proci1(i);
          END entry4;
      OR
        WHEN i = 5 =>
          ACCEPT entry5 DO
            i := 1;
            zg_glob4.proci1(i);
          END entry5;
      END SELECT;
    END LOOP;
  END simple_task;
 
BEGIN
  NULL;    -- will terminate when task enclosed in it terminates
END tk_ls_task_21;
