-- +++ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
-- +++ tcl 2-8-89 change to make ancillary information self-identifying
-- tk_ls_task_31

WITH zg_glob3;
WITH zg_glob4;

WITH zg_glob1;
WITH calendar;
USE  calendar;
 
PACKAGE pkg01_tk_ls16 IS
 
  TASK resource IS
         --+ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
      PRAGMA priority( zg_glob1.priority_2 );
    ENTRY entry1;
    ENTRY initialization_complete;
  END resource;
 
END pkg01_tk_ls16;
 
PACKAGE BODY pkg01_tk_ls16 IS
 
  PRAGMA suppress(access_check);
  PRAGMA suppress(discriminant_check);
  PRAGMA suppress(index_check);
  PRAGMA suppress(length_check);
  PRAGMA suppress(range_check);
  PRAGMA suppress(division_check);
  PRAGMA suppress(overflow_check);
  PRAGMA suppress(elaboration_check);
  PRAGMA suppress(storage_check);
 
  TASK BODY resource IS
  BEGIN
    ACCEPT initialization_complete;    -- sync with task main
    LOOP
      SELECT
        ACCEPT entry1 DO
          zg_glob4.ii := 1;
          zg_glob4.proc0;
        END entry1;
      OR
        DELAY 10.0;            -- should be more than sufficient time
        zg_glob4.ii := 2;      -- If this alternative is taken, error condition
      END SELECT;
      IF seconds(clock) - zg_glob3.begin_time > 900.0 THEN
        zg_glob4.ii := 3;
        EXIT;                  -- exit loop and terminate task after 15 minutes
                               -- One system did not recognize ABORT.
      END IF;
    END LOOP;
  END resource;
END pkg01_tk_ls16;

WITH zg_glob1;
USE  zg_glob1;
WITH zg_glob2;
USE  zg_glob2;
WITH zg_glob3;
USE  zg_glob3;
WITH zg_glob4;

WITH calendar;
USE  calendar;
WITH pkg01_tk_ls16;
USE  pkg01_tk_ls16;
WITH system;
 
PROCEDURE tk_ls_task_31 IS
 
  PRAGMA suppress(access_check);
  PRAGMA suppress(discriminant_check);
  PRAGMA suppress(index_check);
  PRAGMA suppress(length_check);
  PRAGMA suppress(range_check);
  PRAGMA suppress(division_check);
  PRAGMA suppress(overflow_check);
  PRAGMA suppress(elaboration_check);
  PRAGMA suppress(storage_check);
 
  TASK main IS
         --+ tcl 6 Aug 91 modified to correct PCR #90, literal priorities
      PRAGMA priority( zg_glob1.priority_1 );
  END main;
 
 
  TASK BODY main IS
  BEGIN
    zg_glob3.begin_time := seconds(clock);
 
    resource.initialization_complete;    -- entry call to sync with task
                                         -- resource
    --
    -- Test Name         :  tk_ls_task_31
    -- Prime Purpose     :  Language feature test of aspects of tasking. 
    --                      Selective wait in accepting task.  Will take the 
    --                      delay alternative, and then the entry call will be 
    --                      made.  With canonical implementation, the system 
    --                      will have to cancel the DELAY notification it sets 
    --                      up to awaken the task if an entry was not made 
    --                      within the specified delay.
    -- LRM Version       :  A
    -- Optimization      :
    -- Related tests     :  tk_ls_task_30
    -- Author            :  Thomas C. Leavitt
    -- Reviewer          :  Kermit E. Terrell
    -- Date              :
    -- Source            :
    -- Dependencies      :
    -- Other Information :
    --
  
    ----------------------------------------------------------------------------
    zg_glob2.put_test_name("tk_ls_task_31");

    PRAGMA include("startime");

    FOR i IN 1 .. 10 LOOP                -- loop to minimize impact of
                                         -- resource.entry waiting on ACCEPT at
                                         -- beginning of timing loop
      zg_glob4.ii := 0;
      resource.entry1;
      IF zg_glob4.ii /= 1 THEN
        RAISE program_error;
      END IF;
    END LOOP;

    PRAGMA include("stoptime0");

    zg_glob2.put_comment_optional
	("   call on SELECT with delay alternative. Priorities are ");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("   -- set so that the accepting task is NOT ready to rendezvous ");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("   -- so the DELAY alternative is initially taken. The task ");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("   -- performing entry calls then becomes eligible to run and");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("   -- executes entry call. With canonical implementation, the");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("   -- system will have to cancel the DELAY notification it sets ");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("   -- up to awaken the task if an entry call was not made within ");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("   -- the specified time.");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional
	("   -- compare with tk_ls_task_30");
    zg_glob2.put_new_line_optional;

    PRAGMA include("stoptime2");
    ----------------------------------------------------------------------------
 
		----------------------------------------------------
		-- +++ tcl 2-8-89 make ancillary information self-identifying
    zg_glob2.put_ancillary(" >>> tk_ls_task_31 time per rendezvous = ");
    zg_glob2.put_value( (zg_glob3.min_time * 1_000_000.0) / 10.0, 8, 1, 0);
    zg_glob2.put_new_line;
                ------------------------------------------------------
 
    ABORT resource;                      -- stop execution
 
  EXCEPTION
    WHEN OTHERS =>
      IF zg_glob4.ii = 2 THEN
        zg_glob2.put_comment
		("ERROR completed delay alternative in SELECT ");
        zg_glob2.put_new_line;
        zg_glob2.put_error_code (zg_glob1.err_at_execution_time);
 
      ELSIF zg_glob4.ii = 3 THEN
        zg_glob2.put_comment
		("ERROR RESOURCE not terminate after 15 minutes ");
        zg_glob2.put_new_line;
        zg_glob2.put_error_code (zg_glob1.err_at_execution_time);
 
      ELSE
        zg_glob2.put_comment("ERROR ");
        zg_glob2.put_new_line;
        zg_glob2.put_error_code (zg_glob1.err_at_execution_time);
 
      END IF;
  END main;
 
BEGIN
  NULL;
END tk_ls_task_31;
