with zg_glob1;
use  zg_glob1;
with zg_glob2;
with zg_glob3;
with zg_glob4;
with calendar;
use  calendar;
--with system;                    --+ tcl 7 April 1992, PCR #100 part 3
                                  --+    A "WITH SYSTEM;" may be necessary on
                                  --+    some compilers to use PRAGMA PRIORITY
                                  --+    even though Ada Interpretation
                                  --+    AI-00197 clearly states that it should
                                  --+    NOT be required.

pragma elaborate(zg_glob1, zg_glob2, zg_glob3, zg_glob4);

package TK_LSPKG_62 is

  number_of_calls : constant integer16 := 30;

  task driving_task is
    entry START_PROGRAM;
    entry STOP_PROGRAM;
    entry problem_complete_1;
    entry problem_complete_2;
    entry problem_complete_3;
    entry problem_complete_4;
    entry problem_complete_5;
  end driving_task ;

  task receiving_task_1 is
    entry start_problem;
    entry stop_program;
    entry wait_1;
  end receiving_task_1 ;

  task receiving_task_2 is
    entry start_problem;
    entry stop_program;
    entry wait_2;
  end receiving_task_2 ;

  task receiving_task_3 is
    entry start_problem;
    entry stop_program;
    entry wait_3;
  end receiving_task_3 ;

  task receiving_task_4 is
    entry start_problem;
    entry stop_program;
    entry wait_4;
  end receiving_task_4 ;

  task receiving_task_5 is
    entry start_problem;
    entry stop_program;
    entry wait_5;
  end receiving_task_5 ;

 flag_1, flag_2, flag_3, flag_4, flag_5 : boolean;
end TK_LSPKG_62 ;

-------------------------------------------------------------------------------

package body tk_lspkg_62 is

  PRAGMA suppress(access_check);
  PRAGMA suppress(discriminant_check);
  PRAGMA suppress(index_check);
  PRAGMA suppress(length_check);
  PRAGMA suppress(range_check);
  PRAGMA suppress(division_check);
  PRAGMA suppress(overflow_check);
  PRAGMA suppress(elaboration_check);
  PRAGMA suppress(storage_check);

  task task_calling_1 is
    entry start_program;
    entry stop_program;
    entry start_iteration; 
  end task_calling_1;

  task task_calling_2 is
    entry start_program;
    entry stop_program;
    entry start_iteration; 
  end task_calling_2;

  task task_calling_3 is
    entry start_program;
    entry stop_program;
    entry start_iteration; 
  end task_calling_3;

  task task_calling_4 is
    entry start_program;
    entry stop_program;
    entry start_iteration; 
  end task_calling_4;

  task task_calling_5 is
    entry start_program;
    entry stop_program;
    entry start_iteration; 
  end task_calling_5;

  task body receiving_task_1 is
    stop_program_received : boolean := false;
  begin 
  loop
    select
      accept start_problem ;
        accept wait_1
        do
          flag_1 := true; 
        end;
    or 
      accept stop_program; 
      stop_program_received := true; 
    end select;
    exit when stop_program_received ;
  end loop;
  end receiving_task_1 ;

  task body receiving_task_2 is
    stop_program_received : boolean := false;
  begin 
  loop
    select
      accept start_problem ;
      accept wait_2
        do
          flag_2 := true; 
        end;
    or 
      accept stop_program; 
      stop_program_received := true; 
    end select;
    exit when stop_program_received ;
  end loop;
  end receiving_task_2 ;

  task body receiving_task_3 is
    stop_program_received : boolean := false;
  begin 
  loop
    select
      accept start_problem ;
      accept wait_3
        do
          flag_3 := true; 
        end;
    or 
      accept stop_program; 
      stop_program_received := true; 
    end select;
    exit when stop_program_received ;
  end loop;
  end receiving_task_3 ;

  task body receiving_task_4 is
    stop_program_received : boolean := false;
  begin 
  loop
    select
      accept start_problem ;
      accept wait_4
        do
          flag_4 := true; 
        end;
    or 
      accept stop_program; 
      stop_program_received := true; 
    end select;
    exit when stop_program_received ;
  end loop;
  end receiving_task_4 ;

  task body receiving_task_5 is
    stop_program_received : boolean := false;
  begin 
  loop
    select
      accept start_problem ;
      accept wait_5
        do
          flag_5 := true; 
        end;
    or 
      accept stop_program; 
      stop_program_received := true; 
    end select;
    exit when stop_program_received ;
  end loop;
  end receiving_task_5 ;

task body driving_task is                      
  accepted_flag_1, accepted_flag_2, accepted_flag_3, 
  accepted_flag_4, accepted_flag_5 : boolean;
begin

    accept start_program;

    task_calling_1.start_program;
    task_calling_2.start_program;
    task_calling_3.start_program;
    task_calling_4.start_program;
    task_calling_5.start_program;


--------------------------------------------------------------------------------
  -- Test Name         :  TK_LS_TASK_62
  -- Prime Purpose     :  This problem is one of a set which explores how 
  --                      rendezvous time varies with the number of accept 
  --                      alternatives in a SELECT statement, 
  --                      and how the time varies when there
  --                      are separate tasks, each containing a loop with one
  --                      ACCEPT statement (which may be a coding style variant
  --                      of a multiple alternative SELECT statement). This
  --                      problem has a five tasks with one ACCEPT statement 
  --                      in a loop in each. It performs the same number of
  --                      rendezvous as the other statements in the set.
  -- Optimization      :
  -- Related tests     :  TK_LS_TASK_61, TK_LS_TASK_63, and TK_LS_TASK_64
  -- Author            :  Thomas C. Leavitt
  -- Reviewer          :  B. Decker-Lindsey  - 2 March 1992
  -- Date              :  December 1991 
  -- Source            :  CECOM recommendation from "Final Report - Evaluation
  --                      of the ACEC Benchmark Suite for Real-Time
  --                      Applications" July 1990
  -- Dependencies      :
  -- Other Information :
  -- Revisions         :  PCR #100
  -- Author            :  Thomas C. Leavitt
  -- Reviewer          :  Julie Leastman - 14 April 1992
  -- Date              :  7 April 1992
  -- Changes           :  Commented out reference to WITH SYSTEM with comments 
  --                      stating that some systems may require it to use a 
  --                      PRAGMA PRIORITY directive.
--------------------------------------------------------------------------------

    zg_glob2.put_test_name("tk_ls_task_62 ");
    pragma include("startime");

    for i in 1 .. number_of_calls
    loop
      receiving_task_1.start_problem;
      receiving_task_2.start_problem;
      receiving_task_3.start_problem;
      receiving_task_4.start_problem;
      receiving_task_5.start_problem;

      accepted_flag_1 := false;
      accepted_flag_2 := false;
      accepted_flag_3 := false;
      accepted_flag_4 := false;
      accepted_flag_5 := false;

      task_calling_1.start_iteration;
      task_calling_2.start_iteration;
      task_calling_3.start_iteration;
      task_calling_4.start_iteration;
      task_calling_5.start_iteration;

      for i in 1..5
      loop
        select
          when not accepted_flag_1 => accept problem_complete_1
             do accepted_flag_1 := true; end;
        or
          when not accepted_flag_2 => accept problem_complete_2
             do accepted_flag_2 := true; end;
        or
          when not accepted_flag_3 => accept problem_complete_3
             do accepted_flag_3 := true; end;
        or
          when not accepted_flag_4 => accept problem_complete_4
             do accepted_flag_4 := true; end;
        or
          when not accepted_flag_5 => accept problem_complete_5
             do accepted_flag_5 := true; end;
        end select;
      end loop;

      if not ( accepted_flag_1 and
               accepted_flag_2 and
               accepted_flag_3 and
               accepted_flag_4 and
               accepted_flag_5 ) then
        zg_glob4.die;
      end if;
    end loop;
  pragma include("stoptime0");
    zg_glob2.put_comment_optional(
       "   Five receiving tasks with one ACCEPT each");
    zg_glob2.put_new_line_optional;
  pragma include("stoptime2");

  task_calling_1.stop_program;
  task_calling_2.stop_program;
  task_calling_3.stop_program;
  task_calling_4.stop_program;
  task_calling_5.stop_program;

  receiving_task_1.stop_program;
  receiving_task_2.stop_program;
  receiving_task_3.stop_program;
  receiving_task_4.stop_program;
  receiving_task_5.stop_program;

  accept stop_program;

end driving_task ;


task body task_calling_1 is
  stop_program_received : boolean := false ;
begin
  accept start_program;

  loop
    select
      accept start_iteration;
      receiving_task_1.wait_1; 
      driving_task.problem_complete_1;
    or
      accept stop_program; 
      stop_program_received := true;
    end select;
    exit when stop_program_received ;
  end loop;
end task_calling_1;

task body task_calling_2 is
  stop_program_received : boolean := false ;
begin
  accept start_program;
  loop
    select
      accept start_iteration;
      receiving_task_2.wait_2;
      driving_task.problem_complete_2;
    or
      accept stop_program; 
      stop_program_received := true;
    end select;
    exit when stop_program_received ;
  end loop;
end task_calling_2;

task body task_calling_3 is
  stop_program_received : boolean := false; 
begin
  accept start_program;
  loop
    select
      accept start_iteration;
      receiving_task_3.wait_3;
      driving_task.problem_complete_3;
    or
      accept stop_program; 
      stop_program_received := true;
    end select;
    exit when stop_program_received ;
  end loop;
end task_calling_3;


task body task_calling_4 is
  stop_program_received : boolean := false;
begin
  accept start_program;
  loop
    select
      accept start_iteration;
      receiving_task_4.wait_4;
      driving_task.problem_complete_4;
    or
      accept stop_program; 
      stop_program_received := true;
    end select;
    exit when stop_program_received ;
  end loop;
end task_calling_4;


task body task_calling_5 is
  stop_program_received : boolean := false;
begin
  accept start_program;
  loop
    select
      accept start_iteration;
      receiving_task_5.wait_5;
      driving_task.problem_complete_5;
    or
      accept stop_program; 
      stop_program_received := true;
    end select;
    exit when stop_program_received ;
  end loop;
end task_calling_5;

end tk_lspkg_62;

--------------------------------------------------------------------------------

with tk_lspkg_62;
procedure tk_ls_task_62 is

  PRAGMA suppress(access_check);
  PRAGMA suppress(discriminant_check);
  PRAGMA suppress(index_check);
  PRAGMA suppress(length_check);
  PRAGMA suppress(range_check);
  PRAGMA suppress(division_check);
  PRAGMA suppress(overflow_check);
  PRAGMA suppress(elaboration_check);
  PRAGMA suppress(storage_check);

begin
  tk_lspkg_62.driving_task.start_program;
  tk_lspkg_62.driving_task.stop_program;
end tk_ls_task_62 ;
