with zg_glob1;
use  zg_glob1;
with zg_glob2;
with zg_glob3;
with zg_glob4;
with calendar;
use  calendar;
--with system;                    --+ tcl 7 April 1992, PCR #100 part 3
                                  --+    A "WITH SYSTEM;" may be necessary on
                                  --+    some compilers to use PRAGMA PRIORITY
                                  --+    even though Ada Interpretation
                                  --+    AI-00197 clearly states that it should
                                  --+    NOT be required.
pragma elaborate(zg_glob1, zg_glob2, zg_glob3, zg_glob4 );

PACKAGE tk_lspkg_65 IS
  type arrival_array_type is array
       ( zg_glob1.integer16'(1)..zg_glob1.integer16'(5) ) of zg_glob1.integer16;
  first_arrival_order , arrival_order : arrival_array_type ;
  first_time_flag : boolean;
  consistent : boolean; 
  dummy : zg_glob1.integer16 ;
  task accepting_task is
    pragma priority( zg_glob1.priority_1 );
    entry start_program;
    entry stop_program; 
    entry wait_1;
    entry wait_2;
    entry wait_3;
    entry wait_4;
    entry wait_5;
  end accepting_task;

  task task_1 is 
    pragma priority( zg_glob1.priority_4 );
    entry sync;
    entry stop_program;
  end task_1 ;

  task task_2 is 
    pragma priority( zg_glob1.priority_3 );
    entry sync;
    entry stop_program;
  end task_2 ;

  task task_3 is 
    pragma priority( zg_glob1.priority_6 );
    entry sync;
    entry stop_program;
  end task_3 ;

  task task_4 is 
    pragma priority( zg_glob1.priority_2 );
    entry sync;
    entry stop_program;
  end task_4 ;

  task task_5 is 
    pragma priority( zg_glob1.priority_5 );
    entry sync;
    entry stop_program;
  end task_5 ;

end tk_lspkg_65 ;

--------------------------------------------------------------------------------

package body tk_lspkg_65 is 
  PRAGMA suppress(access_check);
  PRAGMA suppress(discriminant_check);
  PRAGMA suppress(index_check);
  PRAGMA suppress(length_check);
  PRAGMA suppress(range_check);
  PRAGMA suppress(division_check);
  PRAGMA suppress(overflow_check);
  PRAGMA suppress(elaboration_check);
  PRAGMA suppress(storage_check);

task body accepting_task is
begin
-------------------------------------------------------------------------------
  -- Test Name         :  TK_LS_TASK_65
  -- Prime Purpose     :  This is the first of a set of 2 test problems designed
  --                      to determine the algorithm used when choosing among
  --                      branches of a selective wait statement. The test 
  --                      checks whether the system appears to be using a 
  --                      consistent selection order and checks for several 
  --                      anticipated algorithms. It will not report that any 
  --                      particular algorithm is being used unless the expected
  --                      order is repeated on every iteration of the test 
  --                      problem. The algorithms anticipated are:
  --                        1. Lexical order of alternatives.
  --                        2. "Roving" lexical order of alternatives, which
  --                           gives priority to the first alternative the
  --                           first time the SELECT statement is executed; to
  --                           the second alternative the second time the SELECT
  --                           is executed; etc.
  --                        3. Reverse lexical order.
  --                        4. Order of arrival - First-In-First-Out.
  --                        5. Reverse order of arrival - Last-In-First-Out.
  --                        6. In order of priority of the tasks making the
  --                           entry calls.
  --                        7. Order of simplicity of guard expressions.
  --                      The programs are constructed such that each of the
  --                      anticipated selection algorithms corresponds to a
  --                      different sequence of tasks being accepted.
  --                      
  --                      Test TK_LS_TASK_65 has each of the calling tasks make
  --                      their entry calls, then the receiving task loops 
  --                      through the SELECT statement accepting each entry 
  --                      call.
  -- Optimization      :
  -- Related Tests     :  TK_LS_TASK_66
  -- Author            :  T. Leavitt, 
  -- Reviewer          :  B. Decker-Lindsey  - 2 March 1992
  -- Date              :  December 1991
  -- Source            :  Test for this feature was recommended in CECOM 
  --                      report "Evaluation of the ACEC Benchmark Suite 
  --                      for Real-Time Applications." CIN:C02 092LY 0003 00, 
  --                      23 July 1990
  -- Dependencies      :  Requires support for 6 levels of priority
  -- Other Information :  
  -- Revisions         :  PCR #100
  -- Author            :  Thomas C. Leavitt
  -- Reviewer          :  Julie Leastman - 14 April 1992
  -- Date              :  7 April 1992
  -- Changes           :  Commented out reference to WITH SYSTEM with comments 
  --                      stating that some systems may require it to use a 
  --                      PRAGMA PRIORITY directive. Modified descriptions of
  --                      output in several places to clarify discussion.
----------------------------------------------------------------------------

  accept start_program;
  zg_glob1.bool := true; 

  first_time_flag := false;
  consistent := true;

  zg_glob2.put_test_name("tk_ls_task_65 ");
  pragma include("startime");
    
    dummy := 0;
    select     -- conditional entry call
      task_2.sync; -- Tasks should be queued in order 2, 3, 1 ...
      if dummy /= 1 then
        zg_glob2.put_comment(
           " TASK_2 select error ACCEPTING_TASK run prior to TASK_2");
        zg_glob2.put_new_line;
      end if;
    else
      zg_glob2.put_comment(" TASK_2 select error TASK_2 not ready ");
      zg_glob2.put_new_line;
    end select;

    select     -- conditional entry call
      task_3.sync; -- Tasks should be queued in order 2, 3, 1 ...
      if dummy /= 1 then
        zg_glob2.put_comment(
           " TASK_3 select error ACCEPTING_TASK run prior to TASK_3");
        zg_glob2.put_new_line;
      end if;
    else
      zg_glob2.put_comment(" TASK_3 select error TASK_3 not ready ");
      zg_glob2.put_new_line;
    end select;

    dummy := 0;
    select     -- conditional entry call
      task_1.sync; -- Tasks should be queued in order 2, 3, 1 ...
      if dummy /= 1 then
        zg_glob2.put_comment(
           " TASK_1 select error ACCEPTING_TASK run prior to TASK_1");
        zg_glob2.put_new_line;
      end if;
    else
      zg_glob2.put_comment(" TASK_1 select error TASK_1 not ready ");
      zg_glob2.put_new_line;
    end select;

    dummy := 0;
    select     -- conditional entry call
      task_5.sync; -- Tasks should be queued in order 2 3, 1 ...
      if dummy /= 1 then
        zg_glob2.put_comment(
           " TASK_5 select error ACCEPTING_TASK run prior to TASK_5");
        zg_glob2.put_new_line;
      end if;
    else
      zg_glob2.put_comment(" TASK_5 select error TASK_5 not ready ");
      zg_glob2.put_new_line;
    end select;

    dummy := 0;
    select     -- conditional entry call
      task_4.sync; -- Tasks should be queued in order 2, 3, 1 ...
      if dummy /= 1 then
        zg_glob2.put_comment(
           " TASK_4 select error ACCEPTING_TASK run prior to TASK_4");
        zg_glob2.put_new_line;
      end if;
    else
      zg_glob2.put_comment(" TASK_4 select error TASK_4 not ready ");
      zg_glob2.put_new_line;
    end select;

    for i in arrival_order'range -- accept each task
    loop
      select
      when zg_glob1.max( zg_glob4.e1(zg_glob4.ei), zg_glob4.e1(zg_glob4.ej)) >= 
           zg_glob1.min ( zg_glob4.e2(zg_glob4.ei,zg_glob4.ej), 
                          zg_glob4.e2(zg_glob4.ej,zg_glob4.ei) ) =>
      accept wait_1 do arrival_order(i) := 1; end;
      or
      -- no condition
      accept wait_2 do arrival_order(i) := 2; end;
      or
      when zg_glob4.e1(zg_glob4.ei) = zg_glob4.e2(zg_glob4.ei,zg_glob4.ej) =>
      accept wait_3 do arrival_order(i) := 3; end;
      or
      when zg_glob4.ei = zg_glob4.ej =>
      accept wait_4 do arrival_order(i) := 4; end;
      or
      when zg_glob1.bool =>
      accept wait_5 do arrival_order(i) := 5; end;
      end select;
    end loop;
    if first_time_flag = false then
       first_time_flag := true;
       first_arrival_order := arrival_order;
    end if;

    if false then          -- debugging code
      if first_arrival_order /= arrival_order then
        zg_glob2.put_comment(" arrival_order is ( ");
        zg_glob2.put_value( arrival_order(1),1 );
        for i in 2..arrival_order'last
        loop
          zg_glob2.put_comment( ", ");
          zg_glob2.put_value( arrival_order(i),1 );
        end loop;
        zg_glob2.put_comment(" ) ");
        zg_glob2.put_new_line;
      end if;
    end if;
 
    consistent := consistent and first_arrival_order = arrival_order;       
  pragma include("stoptime0");
   zg_glob2.put_comment_optional(
               --+ tcl 7 April 1992, PCR #100 part 4
               --      correct grammar, replace "use to" with "used to"
      "   Determine algorithm used to determine the order ");
   zg_glob2.put_new_line_optional;
   zg_glob2.put_comment_optional(
      "   of task selection at a selective wait statement.");
   zg_glob2.put_new_line_optional;
   zg_glob2.put_comment_optional(
      "   The problem tests for the following selection orders:");
   zg_glob2.put_new_line_optional;
   zg_glob2.put_comment_optional(
      "      Lexical order or reverse lexical order ");
   zg_glob2.put_new_line_optional;
   zg_glob2.put_comment_optional(
      "      ""Roving"" lexical order ");
   zg_glob2.put_new_line_optional;
   zg_glob2.put_comment_optional(
      "      Order of arrival - FIFO ");
   zg_glob2.put_new_line_optional;
   zg_glob2.put_comment_optional(
      "      Reverse order of arrival - LIFO ");
   zg_glob2.put_new_line_optional;
   zg_glob2.put_comment_optional(
      "      Priority order ");
   zg_glob2.put_new_line_optional;
   zg_glob2.put_comment_optional(
      "      Order of complexity of guard expressions ");
   zg_glob2.put_new_line_optional;
                --+ tcl 7 April 1992, PCR #100 part 4
                --        Moved following description from ancillary data 
                --        discussion of inconsistent order.
   zg_glob2.put_comment_optional(
      "      Randomized order - to provide ""fair"" scheduling ");
   zg_glob2.put_new_line_optional;
   zg_glob2.put_comment_optional(
      "          of equal priority tasks");
   zg_glob2.put_new_line_optional;

  PRAGMA include("stoptime2");

  zg_glob2.put_comment(
     "   A user should not assume that an empirically determined scheduling ");
  zg_glob2.put_new_line;
  zg_glob2.put_comment(
     "   order is a guarantee that a system will always use that order ");
  zg_glob2.put_new_line;
  zg_glob2.put_comment(
                --+ tcl 7 April 1992, PCR #100 part 4
                --        correct grammar, replace "to use" with "may use"
     "   because a system may use different task selection algorithms  ");
  zg_glob2.put_new_line;
  zg_glob2.put_comment(
     "   for different SELECT statements.  ");
  zg_glob2.put_new_line;
  if consistent then
    if    arrival_order = ( 1, 2, 3, 4, 5 ) then
      zg_glob2.put_ancillary(
         " >>> TK_LS_TASK_65 Tasks are selected in lexical order.");
      zg_glob2.put_new_line;
    elsif arrival_order(1) = ( arrival_order(5) mod 5 ) + 1 and
          arrival_order(2) = ( arrival_order(1) mod 5 ) + 1 and
          arrival_order(3) = ( arrival_order(2) mod 5 ) + 1 and
          arrival_order(4) = ( arrival_order(3) mod 5 ) + 1 and
          arrival_order(5) = ( arrival_order(4) mod 5 ) + 1 then
      zg_glob2.put_ancillary(
         " >>> TK_LS_TASK_65 Tasks are selected in ""roving"" lexical order.");
      zg_glob2.put_new_line;
    elsif arrival_order = ( 5, 4, 3, 2, 1 ) then
      zg_glob2.put_ancillary(
                           --+ tcl 7 April 1992, PCR #100 part 4
                           --      correct grammar, replace "selected tasks in"
                           --      with "selected in"
      " >>> TK_LS_TASK_65 Tasks are selected in reverse lexical order.");
      zg_glob2.put_new_line;
    elsif arrival_order = ( 2, 3, 1, 5, 4 ) then
      zg_glob2.put_ancillary(
      " >>> TK_LS_TASK_65 Tasks are selected in FIFO or priority order.");
      zg_glob2.put_new_line;
      zg_glob2.put_ancillary(
      " >>> TK_LS_TASK_65 Examine results of TK_LS_TASK_66 to distinguish. ");
      zg_glob2.put_new_line;
    elsif arrival_order = ( 4, 5, 1, 3, 2 ) then
      zg_glob2.put_ancillary(
         " >>> TK_LS_TASK_65 Tasks are selected in LIFO order. ");
      zg_glob2.put_new_line;
    elsif arrival_order = ( 2, 5, 4, 3, 1 ) then
      zg_glob2.put_ancillary(
         " >>> TK_LS_TASK_65 Tasks are selected in guard complexity order. ");
      zg_glob2.put_new_line;
    else  
      zg_glob2.put_ancillary(
       " >>> TK_LS_TASK_65 Tasks are selected in unanticipated order.");
      zg_glob2.put_new_line;
                            --+ tcl 7 April 1992, PCR #100 part 4
                            --      Moved description of orders tested to the
                            --      problem description.
      zg_glob2.put_ancillary(
      " >>> TK_LS_TASK_65 The order observed was: arrival_order( ");
      zg_glob2.put_new_line;
      zg_glob2.put_ancillary(
      " >>> TK_LS_TASK_65 ");
      zg_glob2.put_value(  arrival_order(1), 1 );
      for i in 2 .. arrival_order'last
      loop
        zg_glob2.put_comment( ", ");
        zg_glob2.put_value( arrival_order(i),1 );
      end loop;
      zg_glob2.put_comment(" ) ");
      zg_glob2.put_new_line;
     end if;
  else
    zg_glob2.put_ancillary(
       " >>> TK_LS_TASK_65 Tasks were selected in inconsistent order.");
    zg_glob2.put_new_line;
      zg_glob2.put_ancillary(
      " >>> TK_LS_TASK_65 Last observed order of selection is ( ");
    zg_glob2.put_value(  arrival_order(1),1 );
    for i in 2 .. arrival_order'last
    loop
      zg_glob2.put_comment( ", ");
      zg_glob2.put_value( arrival_order(i),1 );
    end loop;
    zg_glob2.put_comment(" ) ");
    zg_glob2.put_new_line;
           --+ tcl 7 April 1992, PCR #100 part 4
           --        Moved discussion of randomized order to problem description
           --        Added following comment reflecting the possible meaning of 
           --        the occurrence of an inconsistent order.
    zg_glob2.put_ancillary(
       " >>> TK_LS_TASK_65 May reflect intentional randomizing order.");
    zg_glob2.put_new_line;
  end if;

  task_1.stop_program;
  task_2.stop_program;
  task_3.stop_program;
  task_4.stop_program;
  task_5.stop_program;

  accept stop_program;

end accepting_task;
 
task body task_5 is
begin
  loop
    select
      accept stop_program; 
      exit;                     -- exit loop if shut down task cleanly
    or
      accept sync;
      dummy:=1;
      select 
        accepting_task.wait_5;
        zg_glob2.put_comment(" ACCEPTING_TASK error immediate rendezvous ");
        zg_glob2.put_new_line;
      else
        accepting_task.wait_5;
      end select;
    end select;
    zg_glob4.proc0;
    delay duration(zg_glob1.zero);
  end loop;
end task_5 ;

task body task_4 is
begin
  loop
    select
      accept stop_program; 
      exit;                     -- exit loop if shut down task cleanly
    or
      accept sync;
      dummy:=1;
      select 
        accepting_task.wait_4;
        zg_glob2.put_comment(" ACCEPTING_TASK error immediate rendezvous ");
        zg_glob2.put_new_line;
      else
        accepting_task.wait_4;
      end select;
    end select;
    zg_glob4.proc0;
    delay duration(zg_glob1.zero);
  end loop;
end task_4 ;

task body task_3 is
begin
  loop
    select
      accept stop_program; 
      exit;                     -- exit loop if shut down task cleanly
    or
      accept sync;
      dummy:=1;
      select 
        accepting_task.wait_3;
        zg_glob2.put_comment(" ACCEPTING_TASK error immediate rendezvous ");
        zg_glob2.put_new_line;
      else
        accepting_task.wait_3;
      end select;
    end select;
    zg_glob4.proc0;
    delay duration(zg_glob1.zero);
  end loop;
end task_3 ;

task body task_2 is
begin
  loop
    select
      accept stop_program; 
      exit;                     -- exit loop if shut down task cleanly
    or
      accept sync;
      dummy:=1;
      select 
        accepting_task.wait_2;
        zg_glob2.put_comment(" ACCEPTING_TASK error immediate rendezvous ");
        zg_glob2.put_new_line;
      else
        accepting_task.wait_2;
      end select;
    end select;
    zg_glob4.proc0;
    delay duration(zg_glob1.zero);
  end loop;
end task_2 ;

task body task_1 is
begin
  loop
    select
      accept stop_program; 
      exit;                     -- exit loop if shut down task cleanly
    or
      accept sync;
      dummy:=1;
      select 
        accepting_task.wait_1;
        zg_glob2.put_comment(" ACCEPTING_TASK error immediate rendezvous ");
        zg_glob2.put_new_line;
      else
        accepting_task.wait_1;
      end select;
    end select;
    zg_glob4.proc0;
    delay duration(zg_glob1.zero);
  end loop;
end task_1 ;

end tk_lspkg_65 ;

--------------------------------------------------------------------------------

--with zg_glob1;                  --+ tcl 7 April 1992, PCR #100 part 3
--use  zg_glob1;                  --+ tcl 7 April 1992, PCR #100 part 3
--with zg_glob2;                  --+ tcl 7 April 1992, PCR #100 part 3
--with zg_glob3;                  --+ tcl 7 April 1992, PCR #100 part 3
--with zg_glob4;                  --+ tcl 7 April 1992, PCR #100 part 3
--WITH calendar;                  --+ tcl 7 April 1992, PCR #100 part 3
--USE  calendar;                  --+ tcl 7 April 1992, PCR #100 part 3
with tk_lspkg_65 ;
 
PROCEDURE tk_ls_task_65 IS

  PRAGMA suppress(access_check);
  PRAGMA suppress(discriminant_check);
  PRAGMA suppress(index_check);
  PRAGMA suppress(length_check);
  PRAGMA suppress(range_check);
  PRAGMA suppress(division_check);
  PRAGMA suppress(overflow_check);
  PRAGMA suppress(elaboration_check);
  PRAGMA suppress(storage_check);
 
BEGIN
  tk_lspkg_65.accepting_task.start_program;
  tk_lspkg_65.accepting_task.stop_program;   
end tk_ls_task_65 ;
