with zg_glob1;
use  zg_glob1;
with zg_glob2;
with zg_glob3;
with zg_glob4;
with calendar;
use  calendar;
--with system;                    --+ tcl 7 April 1992, PCR #100 part 3
                                  --+    A "WITH SYSTEM;" may be necessary on
                                  --+    some compilers to use PRAGMA PRIORITY
                                  --+    even though Ada Interpretation
                                  --+    AI-00197 clearly states that it should
                                  --+    NOT be required.

pragma elaborate(zg_glob1, zg_glob2, zg_glob3, zg_glob4 );

PACKAGE tk_lspkg_67 IS
  type evaluation_array_type is 
       array( zg_glob1.integer16'(1)..zg_glob1.integer16'(5) ) 
       of zg_glob1.integer16;
  first_evaluation_order , evaluation_order : evaluation_array_type ;
  type arrival_array_type is 
       array( zg_glob1.integer16'(1)..zg_glob1.integer16'(5) ) 
       of zg_glob1.integer16;
  first_arrival_order , arrival_order : arrival_array_type ;
  first_time_flag : boolean;
  consistent : boolean; 
  dummy : zg_glob1.integer16 ;

  task accepting_task is
    pragma priority( zg_glob1.priority_1 );
    entry start_program;
    entry stop_program; 
    entry wait_1;
    entry wait_2;
    entry wait_3;
    entry wait_4;
    entry wait_5;
  end accepting_task;

  task task_1 is 
    pragma priority( zg_glob1.priority_4 );
    entry sync;
    entry stop_program;
  end task_1 ;

  task task_2 is 
    pragma priority( zg_glob1.priority_3 );
    entry sync;
    entry stop_program;
  end task_2 ;

  task task_3 is 
    pragma priority( zg_glob1.priority_6 );
    entry sync;
    entry stop_program;
  end task_3 ;

  task task_4 is 
    pragma priority( zg_glob1.priority_2 );
    entry sync;
    entry stop_program;
  end task_4 ;

  task task_5 is 
    pragma priority( zg_glob1.priority_5 );
    entry sync;
    entry stop_program;
  end task_5 ;

  guard1_count : zg_glob1.integer16 ; 
  guard2_count : zg_glob1.integer16 ; 
  guard3_count : zg_glob1.integer16 ; 
  guard4_count : zg_glob1.integer16 ; 
  guard5_count : zg_glob1.integer16 ; 

  function guard1 return boolean;
  function guard2 return boolean;
  function guard3 return boolean;
  function guard4 return boolean;
  function guard5 return boolean;
end tk_lspkg_67 ;

--------------------------------------------------------------------------------

package body tk_lspkg_67 is 

  PRAGMA suppress(access_check);
  PRAGMA suppress(discriminant_check);
  PRAGMA suppress(index_check);
  PRAGMA suppress(length_check);
  PRAGMA suppress(range_check);
  PRAGMA suppress(division_check);
  PRAGMA suppress(overflow_check);
  PRAGMA suppress(elaboration_check);
  PRAGMA suppress(storage_check);

function value_to_return ( temp : zg_glob1.integer16 ) return boolean is
  bool_temp : boolean;
begin

  for i in evaluation_order'range
  loop
    if evaluation_order(i) = 0 then
       evaluation_order(i) := temp ;
       exit;
    end if;
  end loop;

  bool_temp := true ;
  for i in arrival_order'range
  loop
    if arrival_order(i) = temp then
      bool_temp := false ; exit;
    end if;
  end loop;  
  return bool_temp ;
end value_to_return;

function guard1 return boolean is
begin
  guard1_count := guard1_count + 1;
  return value_to_return(1);
end guard1;

function guard2 return boolean is
begin
  guard2_count := guard2_count + 1;
  return value_to_return(2);
end guard2;

function guard3 return boolean is
begin
  guard3_count := guard3_count + 1;
  return value_to_return(3);
end guard3;

function guard4 return boolean is
begin
  guard4_count := guard4_count + 1;
  return value_to_return(4);
end guard4;

function guard5 return boolean is
begin
  guard5_count := guard5_count + 1;
  return value_to_return(5);
end guard5;

task body accepting_task is
begin

  accept start_program;


--------------------------------------------------------------------------------
  -- Test Name         :  TK_LS_TASK_67
  -- Prime Purpose     :  This problem determines the order of evaluation of 
  --                      guard expressions in a SELECT statement. It will 
  --                      report whether the system evaluates all the guards
  --                      as ancillary information. An optimizing system 
  --                      might want to avoid evaluating all the guard 
  --                      expressions as soon as it finds one open alternative 
  --                      which has a task queued on its ACCEPT, although 
  --                      this might not be legal Ada unless the system can 
  --                      verify that none of the expressions which might be 
  --                      omitted have side effects which should have been 
  --                      performed. The test makes the determination by 
  --                      including functions in the guard expressions which,
  --                      as a side effect, record the order in which they were
  --                      called.
  -- Optimization      :
  -- Related tests     :  
  -- Author            :  Thomas C. Leavitt
  -- Reviewer          :  B. Decker-Lindsey - 3 March 1992
  -- Date              :  December 1991
  -- Source            :  CECOM recommendation from "Final Report - Evaluation
  --                      of the ACEC Benchmark Suite for Real-Time
  --                      Applications" July 1990
  -- Dependencies      :  Requires support for 6 levels of priority
  -- Other Information :  It is possible that a system might evaluate guard
  --                      expressions containing function calls in a 
  --                      different order than it evaluates guard expressions
  --                      not containing function calls. The results of this
  --                      test may not be completely reliable.
  -- Revisions         :  PCR #100
  -- Author            :  Thomas C. Leavitt
  -- Reviewer          :  Julie Leastman - 15 April 1992
  -- Date              :  7 April 1992
  -- Changes           :  Commented out reference to WITH SYSTEM with comments 
  --                      stating that some systems may require it to use a 
  --                      PRAGMA PRIORITY directive. Removed unneeded WITH
  --                      clauses from the context of procedure TK_LS_TASK_67
  --                      and corrected the following grammar from:
  --                      "System does evaluates" to "System does evaluate".

-------------------------------------------------------------------------------

  first_time_flag := false;
  consistent := true;
  zg_glob2.put_test_name("tk_ls_task_67");
  pragma include("startime");
    task_1.sync;
    task_2.sync;
    task_3.sync;
    task_4.sync;
    task_5.sync;

    guard1_count := 0; 
    guard2_count := 0; 
    guard3_count := 0; 
    guard4_count := 0; 
    guard5_count := 0; 

    arrival_order    := ( others => 0 );

    for i in evaluation_order'range -- accept each task
    loop
      evaluation_order := ( others => 0 );
      select
      when guard1 => accept wait_1 do arrival_order(i) := 1; end;
      or
      when guard2 => accept wait_2 do arrival_order(i) := 2; end;
      or
      when guard3 => accept wait_3 do arrival_order(i) := 3; end;
      or
      when guard4 => accept wait_4 do arrival_order(i) := 4; end;
      or
      when guard5 => accept wait_5 do arrival_order(i) := 5; end;
      end select;
    end loop;

    if first_time_flag = false then
       first_time_flag := true;
       first_arrival_order := arrival_order;
       first_evaluation_order := evaluation_order;
    end if;

    if false then
      if first_arrival_order /= arrival_order then
        zg_glob2.put_comment(" arrival_order is ( ");
        zg_glob2.put_value( arrival_order(1),1 );
        for i in 2..arrival_order'last
        loop
          zg_glob2.put_comment( ", ");
          zg_glob2.put_value( arrival_order(i),1 );
        end loop;
        zg_glob2.put_comment(" ) ");
        zg_glob2.put_new_line;
      end if;
      if first_evaluation_order /= evaluation_order then
        zg_glob2.put_comment(" evaluation_order is ( ");
        zg_glob2.put_value( evaluation_order(1),1 );
        for i in 2..evaluation_order'last
        loop
          zg_glob2.put_comment( ", ");
          zg_glob2.put_value( evaluation_order(i),1 );
        end loop;
        zg_glob2.put_comment(" ) ");
        zg_glob2.put_new_line;
      end if;
    end if;

    consistent := consistent and first_arrival_order = arrival_order;       
  pragma include("stoptime0");
   zg_glob2.put_comment_optional(
      "   Determine order in which guard expressions are evaluated. ");
   zg_glob2.put_new_line_optional;
  PRAGMA include("stoptime2");

  if guard1_count = 5 and 
     guard2_count = 5 and 
     guard3_count = 5 and 
     guard4_count = 5 and 
     guard5_count = 5 
  then
     zg_glob2.put_ancillary(
        " >>> TK_LS_TASK_67 System does evaluate all guards ");
--+ tcl 7 April 1992, PCR #100 part 4 
--+ change "System does evaluates" to "System does evaluate"
     zg_glob2.put_new_line;
  else
     zg_glob2.put_ancillary(
        " >>> TK_LS_TASK_67 System does NOT evaluate all guards ");
     zg_glob2.put_new_line;
  end if;

  if false then            -- for debugging
      zg_glob2.put_comment("  evaluation_order( ");
      zg_glob2.put_value(  evaluation_order(1),1 );
      for i in 2 .. evaluation_order'last
      loop
        zg_glob2.put_comment( ", ");
        zg_glob2.put_value( evaluation_order(i),1 );
      end loop;
      zg_glob2.put_comment(" ) ");
      zg_glob2.put_new_line;
  end if;

  if consistent then   
    if    arrival_order = ( 1, 2, 3, 4, 5 ) then
      zg_glob2.put_ancillary(
        " >>> TK_LS_TASK_67 System selects tasks in lexical order ");
      zg_glob2.put_new_line;
    elsif arrival_order(1) = ( arrival_order(5) mod 5 ) + 1 and
          arrival_order(2) = ( arrival_order(1) mod 5 ) + 1 and
          arrival_order(3) = ( arrival_order(2) mod 5 ) + 1 and
          arrival_order(4) = ( arrival_order(3) mod 5 ) + 1 and
          arrival_order(5) = ( arrival_order(4) mod 5 ) + 1 then
      zg_glob2.put_ancillary(
        " >>> TK_LS_TASK_67 System selects tasks in permuted lexical order");
      zg_glob2.put_new_line;
    elsif arrival_order = ( 5, 4, 3, 2, 1 ) then
      zg_glob2.put_ancillary(
        " >>> TK_LS_TASK_67 System selects tasks in reverse lexical order");
      zg_glob2.put_new_line;
    elsif arrival_order = ( 2, 3, 1, 5, 4 ) then
      zg_glob2.put_ancillary(
        " >>> TK_LS_TASK_67 System selects tasks in arrival order-FIFO");
      zg_glob2.put_new_line;
    elsif arrival_order = ( 4, 5, 1, 3, 2 ) then
      zg_glob2.put_ancillary(
        " >>> TK_LS_TASK_67 System selects tasks in reverse arrival" &
        " order-LIFO" );
      zg_glob2.put_new_line;
    elsif arrival_order = ( 3, 5, 1, 2, 4 ) then
      zg_glob2.put_ancillary(
        " >>> TK_LS_TASK_67 System selects tasks in priority order");
      zg_glob2.put_new_line;
    else  
      zg_glob2.put_ancillary(
        " >>> TK_LS_TASK_67 System selects tasks in unanticipated order");
      zg_glob2.put_new_line;
      zg_glob2.put_ancillary(
        " >>> TK_LS_TASK_67 arrival_order( ");
      zg_glob2.put_value(  arrival_order(1),1 );
      for i in 2 .. arrival_order'last
      loop
        zg_glob2.put_comment( ", ");
        zg_glob2.put_value( arrival_order(i),1 );
      end loop;
      zg_glob2.put_comment(" ) ");
      zg_glob2.put_new_line;
     end if;
  else
    zg_glob2.put_ancillary(
      " >>> TK_LS_TASK_67 System selects tasks in inconsistent order");
    zg_glob2.put_new_line;
    zg_glob2.put_ancillary(
      " >>> TK_LS_TASK_67 Last observed order of selection is ( ");
    zg_glob2.put_value(  arrival_order(1),1 );
    for i in 2 .. arrival_order'last
    loop
      zg_glob2.put_comment( ", ");
      zg_glob2.put_value( arrival_order(i),1 );
    end loop;
    zg_glob2.put_comment(" ) ");
    zg_glob2.put_new_line;
  end if;

  task_1.stop_program;
  task_2.stop_program;
  task_3.stop_program;
  task_4.stop_program;
  task_5.stop_program;

  accept stop_program;

end accepting_task;
 
task body task_5 is
begin
  loop
    select
      accept stop_program; 
      exit;                     -- exit loop if shut down task cleanly
    or
      accept sync;
      dummy:=1;
      select 
        accepting_task.wait_5;
        zg_glob2.put_comment("  ACCEPTING_TASK error immediate rendezvous ");
        zg_glob2.put_new_line;
      else
        accepting_task.wait_5;
      end select;
    end select;
    zg_glob4.proc0;
    delay duration(zg_glob1.zero);
  end loop;
end task_5 ;

task body task_4 is
begin
  loop
    select
      accept stop_program; 
      exit;                     -- exit loop if shut down task cleanly
    or
      accept sync;
      dummy:=1;
      select 
        accepting_task.wait_4;
        zg_glob2.put_comment("   ACCEPTING_TASK error immediate rendezvous ");
        zg_glob2.put_new_line;
      else
        accepting_task.wait_4;
      end select;
    end select;
    zg_glob4.proc0;
    delay duration(zg_glob1.zero);
  end loop;
end task_4 ;

task body task_3 is
begin
  loop
    select
      accept stop_program; 
      exit;                     -- exit loop if shut down task cleanly
    or
      accept sync;
      dummy:=1;
      select 
        accepting_task.wait_3;
        zg_glob2.put_comment("   ACCEPTING_TASK error immediate rendezvous ");
        zg_glob2.put_new_line;
      else
        accepting_task.wait_3;
      end select;
    end select;
    zg_glob4.proc0;
    delay duration(zg_glob1.zero);
  end loop;
end task_3 ;

task body task_2 is
begin
  loop
    select
      accept stop_program; 
      exit;                     -- exit loop if shut down task cleanly
    or
      accept sync;
      dummy:=1;
      select 
        accepting_task.wait_2;
        zg_glob2.put_comment("   ACCEPTING_TASK error immediate rendezvous ");
        zg_glob2.put_new_line;
      else
        accepting_task.wait_2;
      end select;
    end select;
    zg_glob4.proc0;
    delay duration(zg_glob1.zero);
  end loop;
end task_2 ;

task body task_1 is
begin
  loop
    select
      accept stop_program; 
      exit;                     -- exit loop if shut down task cleanly
    or
      accept sync;
      dummy:=1;
      select 
        accepting_task.wait_1;
        zg_glob2.put_comment("   ACCEPTING_TASK error immediate rendezvous ");
        zg_glob2.put_new_line;
      else
        accepting_task.wait_1;
      end select;
    end select;
    zg_glob4.proc0;
    delay duration(zg_glob1.zero);
  end loop;
end task_1 ;

end tk_lspkg_67 ;

--------------------------------------------------------------------------------
           
--with zg_glob1;                    --+ tcl 7 April 1992, PCR #100 part 3
--use  zg_glob1;                    --+ tcl 7 April 1992, PCR #100 part 3
--with zg_glob2;                    --+ tcl 7 April 1992, PCR #100 part 3
--with zg_glob3;                    --+ tcl 7 April 1992, PCR #100 part 3
--with zg_glob4;                    --+ tcl 7 April 1992, PCR #100 part 3
--with calendar;                    --+ tcl 7 April 1992, PCR #100 part 3
--use  calendar;                    --+ tcl 7 April 1992, PCR #100 part 3
--with system;                      --+ tcl 7 April 1992, PCR #100 part 3
with tk_lspkg_67 ;
 
PROCEDURE tk_ls_task_67 IS
  PRAGMA suppress(access_check);
  PRAGMA suppress(discriminant_check);
  PRAGMA suppress(index_check);
  PRAGMA suppress(length_check);
  PRAGMA suppress(range_check);
  PRAGMA suppress(division_check);
  PRAGMA suppress(overflow_check);
  PRAGMA suppress(elaboration_check);
  PRAGMA suppress(storage_check);
BEGIN
  tk_lspkg_67.accepting_task.start_program;
  tk_lspkg_67.accepting_task.stop_program;   
end tk_ls_task_67 ;
