-- This file contains the following ACES tests:
-- tk_rq_to_prot_01    Time to call an entry of a protected object
-- tk_rq_to_prot_02    Time to call a task entry that 
--                      requeues to the same entry of the same object as 
--                      in tk_rq_to_prot_01

with ZG_Glob1;
use  ZG_Glob1;
with ZG_Glob2;
use  ZG_Glob2;
with ZG_Glob3;  
use  ZG_Glob3;
with ZG_Glob4;
use  ZG_Glob4;

with Calendar;
use  Calendar;

procedure tk_rq03 is

  pragma Suppress(Access_Check);
  pragma Suppress(Discriminant_Check);
  pragma Suppress(Index_Check);
  pragma Suppress(Length_Check);
  pragma Suppress(Range_Check);
  pragma Suppress(Division_Check);
  pragma Suppress(Overflow_Check);
  pragma Suppress(Elaboration_Check);
  pragma Suppress(Storage_Check);

  task type Actor is
    entry Start;
    entry Do_Work;
    entry Pass_Work_On;
    entry Get_Count ( The_Count : out ZG_Glob1.FLOAT6 );
    entry Stop;
  end Actor;

  Passing_Task  : Actor;

  protected type Object is
    entry Start;
    entry Pass_Work_On;
    entry Do_Work;
    function Count_Value return ZG_Glob1.FLOAT6;
  private
    Active : BOOLEAN := False;
    Count : ZG_Glob1.FLOAT6 := 0.0;
  end Object;

  Working_Object : Object;

  task Competitor is
    entry Start;
    entry Stop;
  end Competitor;

  task body Actor is
    Finished : BOOLEAN;
    Count    : ZG_Glob1.FLOAT6 := 0.0;
    use ZG_Glob1;
  begin

    accept Start;

    DO_TWICE:
    for I in 1 .. 2 loop
      Finished := False;
      ACT:
      while not Finished loop
        select
          accept Do_Work do
            Count := Count + 1.0;
            if Count >= ZG_Glob4.XX then
              Count := 0.0;
            end if;
          end Do_Work;
        or
          accept Pass_Work_On do
            if not ZG_Glob3.Boolean_False then
              requeue Working_Object.Do_Work;
            end if;
          end Pass_Work_On;
        or
          accept Stop do
            Finished := True;
          end Stop;
        end select;
      end loop ACT;
  
      accept Get_Count ( The_Count : out ZG_Glob1.FLOAT6 ) do
        The_Count := Count;
      end Get_Count;
  
    end loop DO_TWICE;

  end Actor;

  task body Competitor is
  begin
    accept Start;
    if ZG_Glob3.Boolean_False then
      ZG_Glob4.YY := Working_Object.Count_Value;
      if ZG_Glob4.YY < 0.0 then
        Passing_Task.Get_Count ( ZG_Glob4.YY );
      end if;
      Passing_Task.Pass_Work_On;
      Working_Object.Do_Work;
    end if;
    accept Stop;
  end Competitor;

  protected body Object is

    entry Start when not Active is
    begin
      Active := True;
    end Start;

    entry Do_Work when Active is
      use ZG_Glob1;
    begin   -- Do_Work
      Count := Count + 1.0;
      if Count >= ZG_Glob4.XX then
        Count := 0.0;
      end if;
    end Do_Work;

    entry Pass_Work_On when Active is
    begin    -- Pass_Work_On
      if not ZG_Glob3.Boolean_False then
        requeue Working_Object.Do_Work;
      end if;
    end Pass_Work_On;

    function Count_Value return ZG_Glob1.FLOAT6 is
    begin
      return Count;
    end Count_Value;

  end Object;

begin -- tk_rq01

  ZG_Glob4.XX := 1.0E9;

  Passing_Task.Start;
  Competitor.Start;
  Working_Object.Start;

  TK_RQ_TO_PROT_01:
  declare
  --
  -- Test Name         :  tk_rq_to_prot_01

  -- Prime Purpose     :  Measure the performance of a call to a
  --                      task entry
  -- Optimization      :  
  -- Related tests     :  tk_rq_to_prot_02
  -- Author            :  Phil Brashear
  -- Reviewer          :  
  -- Date              :  18 OCT 95
  -- Source            :  
  -- Dependencies      :  This is an Ada 95 - specific test
  -- Other Information :
  -- Revisions         :

    Skip_Output  : exception;

  begin  -- tk_rq_to_prot_01
    ZG_Glob2.Put_Test_Name ( "tk_rq_to_prot_01" );

    pragma Include ("startime");

    Working_Object.Do_Work;

    pragma Include ("stoptime0");

    ZG_Glob2.Put_Comment_Optional
      ( "Working_Task.Do_Work;  -- Directly call entry Do_Work" );
    ZG_Glob2.Put_New_Line_Optional;

    pragma Include ( "Stoptime2" );

    Passing_Task.Stop;

    if not ZG_Glob3.Boolean_False then          -- Always true
      Passing_Task.Get_Count ( ZG_Glob4.YY );
      if ZG_Glob4.YY >= 0.0 then                -- Always true
        ZG_Glob4.YY := Working_Object.Count_Value;
      end if;
    end if;

    if ZG_Glob4.YY <= -1.0 then              -- Always false
      ZG_Glob2.Put_Value ( ZG_Glob4.YY );
    end if;

  end TK_RQ_TO_PROT_01;

  TK_RQ_TO_PROT_02:   -- Actual Requeue
  --
  -- Test Name         :  tk_rq_to_prot_02

  -- Prime Purpose     :  Measure the performance of a requeue from a
  --                      task entry to an entry of a protected object
  -- Optimization      :  
  -- Related tests     :  tk_rq_to_prot_01
  -- Author            :  Phil Brashear
  -- Reviewer          :  
  -- Date              :  18 OCT 95
  -- Source            :  
  -- Dependencies      :  This is an Ada 95 - specific test
  -- Other Information :
  -- Revisions         :

  begin
    ZG_Glob2.Put_Test_Name("tk_rq_to_prot_02");

    pragma Include("startime");
    -- Code to time begins below

    Passing_Task.Pass_Work_On;

    -- Code to time ends above
    pragma Include("stoptime0");

    ZG_Glob2.Put_Comment_Optional
      ( "Passing_Task.Pass_Work_On;   -- Requeue to entry Do_Work" );
    ZG_Glob2.Put_New_Line_Optional;

    pragma Include ( "Stoptime2" );

    Passing_Task.Stop;

    if not ZG_Glob3.Boolean_False then          -- Always true
      Passing_Task.Get_Count ( ZG_Glob4.YY );
      if ZG_Glob4.YY >= 0.0 then                -- Always true
        ZG_Glob4.YY := Working_Object.Count_Value;
      end if;
    end if;

    if ZG_Glob4.YY <= -1.0 then              -- Always false
      ZG_Glob2.Put_Value ( ZG_Glob4.YY );
    end if;

  end TK_RQ_TO_PROT_02;

  Competitor.Stop;

end tk_rq03;
