with zg_glob1;
use  zg_glob1;
with zg_glob2;
with zg_glob3;
with zg_glob4;
with calendar;
use  calendar;

procedure tk_rz_task_no_parameters is

pragma suppress(access_check);
pragma suppress(discriminant_check);
pragma suppress(index_check);
pragma suppress(length_check);
pragma suppress(range_check);
pragma suppress(division_check);
pragma suppress(overflow_check);
pragma suppress(elaboration_check);
pragma suppress(storage_check);

pragma OPTIMIZE (TIME);

   -- Accepts a simple rendezvous (one with no parameters).

   run_tot : zg_glob1.integer16 := 0 ;

   procedure NO_OP (I : zg_glob1.integer16 ; J : in out  zg_glob1.integer16 ) is
   begin
      if not zg_glob3.boolean_false then
         J := 0;
      else  -- Never executed.
         J := I - J - 1;
      end if;
      RUN_TOT := J;
   exception
      when NUMERIC_ERROR | CONSTRAINT_ERROR =>  -- Never executed.
         raise PROGRAM_ERROR ;
   end NO_OP;

   task SIMPLE_ACCEPTOR is
      entry STARTUP;
      entry E;
      entry FINISH;
   end SIMPLE_ACCEPTOR;

   -- Caller in a simple rendezvous.

   task SIMPLE_CALLER is
      entry STARTUP;
      entry FINISH ; 
   end SIMPLE_CALLER;

   task body SIMPLE_ACCEPTOR is
   begin
      loop
         accept STARTUP;
         for I in zg_glob1.integer16'(1) .. zg_glob1.integer16'(1000) 
         loop
           NO_OP (I, RUN_TOT);
           accept E 
           do
             if zg_glob3.boolean_false then 
               zg_glob4.die;                -- for consistency
             end if;
           end;
         end loop;
         accept FINISH;
      end loop;
   end SIMPLE_ACCEPTOR;

  task body SIMPLE_CALLER is
  begin
    loop
      accept STARTUP;
      SIMPLE_ACCEPTOR.STARTUP;
      for i in zg_glob1.integer16'(1) .. zg_glob1.integer16'(1000) 
      loop
        NO_OP (I, RUN_TOT);
        SIMPLE_ACCEPTOR.E;
        RUN_TOT := I;
      end loop;
      accept FINISH ;
    end loop;
   end SIMPLE_CALLER;

begin           -- of main program

  -- Test Name         :  tk_rz_task_no_parameters 
  -- Prime Purpose     :  First of 4 tests which explore the effect of passing 
  --                      increasing numbers of parameters. This test passes
  --                      no parameters. 
  -- Optimization      :  
  -- Related tests     :  tk_rz_task_one_parameter, tk_rz_task_ten_parameters,
  --                      tk_rz_task_hundred_parameter 
  -- Author            :  Thomas C. Leavitt
  -- Reviewer          :  B. Decker-Lindsey  -  1 April 1992
  -- Date              :  19 March 1992
  -- Source            :  AES Test TL15, first case
  -- Dependencies      :  
  -- Other Information :  

   zg_glob2.put_test_name("tk_rz_task_no_parameters ");

   pragma include("startime");
     SIMPLE_CALLER.STARTUP;
     SIMPLE_CALLER.FINISH ;
     SIMPLE_ACCEPTOR.FINISH;
   pragma include("stoptime0");
     zg_glob2.put_comment_optional(
     "   First of 4 tests which explore the effect of varying numbers of");
     zg_glob2.put_new_line_optional;
     zg_glob2.put_comment_optional(
     "   parameters on rendezvous speed. This test passes no parameters.");
     zg_glob2.put_new_line_optional;
     zg_glob2.put_comment_optional(
     "   Test performs 1000 simple rendezvous plus 3 for setup. ");
   pragma include ("stoptime2");

   abort simple_caller, simple_acceptor;

end tk_rz_task_no_parameters ;
