with zg_glob1;
use  zg_glob1;
with zg_glob2;
with zg_glob3;
with zg_glob4;
with calendar;
use  calendar;

procedure tk_rz_task_one_parameter is

pragma suppress(access_check);
pragma suppress(discriminant_check);
pragma suppress(index_check);
pragma suppress(length_check);
pragma suppress(range_check);
pragma suppress(division_check);
pragma suppress(overflow_check);
pragma suppress(elaboration_check);
pragma suppress(storage_check);

pragma OPTIMIZE (TIME);

   -- Accepts a simple rendezvous (one with no parameters).

   run_tot : zg_glob1.integer16 := 0 ;

   procedure NO_OP (I : zg_glob1.integer16 ; J : in out  zg_glob1.integer16 ) is
   begin
      if not zg_glob3.boolean_false then
         J := 0;
      else  -- Never executed.
         J := I - J - 1;
      end if;
      RUN_TOT := J;
   exception
      when NUMERIC_ERROR | CONSTRAINT_ERROR =>  -- Never executed.
         raise PROGRAM_ERROR ;
   end NO_OP;

   task ACCEPTOR is
      entry STARTUP;
      entry E ( I : zg_glob1.integer16 );
      entry FINISH;
   end ACCEPTOR;

   -- Caller in a simple rendezvous.

   task CALLER is
      entry STARTUP;
      entry FINISH ; 
   end CALLER;

   task body ACCEPTOR is
   begin
      loop
         accept STARTUP;
         for I in zg_glob1.integer16'(1) .. zg_glob1.integer16'(1000) 
         loop
           NO_OP (I, RUN_TOT);
           accept E ( i : zg_glob1.integer16 ) 
           do
             if zg_glob3.boolean_false then 
               zg_glob4.ii := i ;
               zg_glob4.proci1 ( zg_glob4.ii );
               zg_glob4.die;                -- for consistency
             end if;
           end;
         end loop;
         accept FINISH;
      end loop;
   end ACCEPTOR;

  task body CALLER is
  begin
    loop
      accept STARTUP;
      ACCEPTOR.STARTUP;
      for i in zg_glob1.integer16'(1) .. zg_glob1.integer16'(1000) 
      loop
        NO_OP (I, RUN_TOT);
        ACCEPTOR.E ( i );
        RUN_TOT := I;
      end loop;
      accept FINISH ;
    end loop;
   end CALLER;

begin           -- of main program

  -- Test Name         :  tk_rz_task_one_parameter 
  -- Prime Purpose     :  Second of 4 tests which explore the effect of passing 
  --                      increasing numbers of parameters. This test passes
  --                      one integer parameter. 
  -- Optimization      :  
  -- Related tests     :  tk_rz_task_no_parameter, tk_rz_task_ten_parameters,
  --                      tk_rz_task_hundred_parameter 
  -- Author            :  Thomas C. Leavitt
  -- Reviewer          :  B. Decker-Lindsey  -  1 April 1992
  -- Date              :  19 March 1992
  -- Source            :  AES Test TL15, second case
  -- Dependencies      :  
  -- Other Information :  

   zg_glob2.put_test_name("tk_rz_task_one_parameter ");

   pragma include("startime");
     CALLER.STARTUP;
     CALLER.FINISH ;
     ACCEPTOR.FINISH;
   pragma include("stoptime0");
     zg_glob2.put_comment_optional(
     "   Second of 4 tests which explore the effect of varying numbers of");
     zg_glob2.put_new_line_optional;
     zg_glob2.put_comment_optional(
     "   parameters on rendezvous speed. This test passes one parameter.");
     zg_glob2.put_new_line_optional;
     zg_glob2.put_comment_optional(
     "   Test performs 1000 simple rendezvous plus 3 for setup. ");
   pragma include ("stoptime2");

   abort caller, acceptor;

end tk_rz_task_one_parameter ;
