WITH zg_glob1;
USE  zg_glob1;
WITH zg_glob2;
USE  zg_glob2;
WITH zg_glob3;  
USE  zg_glob3;
WITH zg_glob4;
USE  zg_glob4;

WITH calendar;
USE  calendar;
 
PROCEDURE TK_RZ_Entry_Access_02 IS

  PRAGMA suppress(access_check);
  PRAGMA suppress(discriminant_check);
  PRAGMA suppress(index_check);
  PRAGMA suppress(length_check);
  PRAGMA suppress(range_check);
  PRAGMA suppress(division_check);
  PRAGMA suppress(overflow_check);
  PRAGMA suppress(elaboration_check);
  PRAGMA suppress(storage_check);

BEGIN
    --
    -- Test Name         :  TK_RZ_Entry_Access_02
    -- Prime Purpose     :  Measure overhead to read and change object value 
    --				with 5 tasks when object is protected by a 
    --				monitor
    -- Optimization      :
    -- Related tests     :  PT_PO_Prot_Access_02/3, TK_RZ_Entry_Access_03
    -- Author            :  Daniel M. Roy
    -- Reviewer          :  Phil Brashear
    -- Date              :  January 15, 1995
    -- Source            :
    -- Dependencies      :
    -- Other Information :
    -- Revisions         :

 The_Test:
  DECLARE

    The_Value	: float6;

    Task Object is
	entry Write (Value : in float6);
	entry Read (Value : out float6);
    end Object;

    task type Modify is
    end Modify;
    type Mod_Bunch is array (1..5) of Modify;

    task body Object is
	State	: float6;
    begin -- Object
	loop
	    select
	        accept Write (Value : in float6) do
	    	    State := Value * ZG_Glob4.One;
		end Write;
	    or
		accept Read (Value : out float6) do
		    Value := State;
		end Read;
	    or
		terminate;
	    end select;
	end loop;
    end Object;

    task body Modify is
    begin -- Modify
      for i in 1 .. 200 loop
        Object.Read (The_Value);
        Object.Write (The_Value - 0.0125);
        if The_Value < -1.0E7 then
          The_Value := ZG_Glob4.One;
        end if;
      end loop;
    end Modify;

 BEGIN -- The_Test
    Object.Write (Zg_Glob4.One);	-- Not timed 
    ----------------------------------------------------------------------------
    zg_glob2.put_test_name("TK_RZ_Entry_Access_02");
    PRAGMA include("startime");
    -- Code to time begins below
    declare
        The_Bunch : Mod_Bunch;
    begin
        null;
    end;
    -- Code to time ends above
    PRAGMA include("stoptime0");
    zg_glob2.put_comment_optional("   for I in 1 .. 1000 loop"); 
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional("     Object.Read (The_Value);"); 
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional("     Object.Write (The_Value - 0.5);"); 
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional("   end loop;" );
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional(
		"   -- Object protected by monitor, modified from 5 tasks");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional(
		"   -- cf PT_PO_Prot_Access_02/3, TK_RZ_Entry_Access_03");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_comment_optional(
		"   -- Ignore this(fooling optimizer): ");
    Zg_Glob2.Put_Value (The_Value);
    zg_glob2.put_new_line_optional;
    PRAGMA include("stoptime2");
    ----------------------------------------------------------------------------
  END The_Test;
  
END TK_RZ_Entry_Access_02 ;  

