--+ tcl 30-oct-89 changed in PQT to correct implementation dependency
-- tk_ss_task_54_mod inadequate task STORAGE_SIZE specified by static expression

WITH zg_glob1;
USE  zg_glob1;
WITH zg_glob2;
USE  zg_glob2;
WITH zg_glob3;
USE  zg_glob3;
WITH zg_glob4;

with CALENDAR;
use CALENDAR;

procedure tk_ss_task_54_mod  is

  --  Note that there are no suppression pragmas.  Storage_checking is required

  ILLEGAL_ADA1, ILLEGAL_ADA2, ILLEGAL_ADA3, ILLEGAL_ADA4 : exception;

  ERROR_FLAG : BOOLEAN := FALSE;

  task type SIMPLE_TASK is
    entry SYNC;     --+ tcl 30-oct-89 
  end SIMPLE_TASK;
  for SIMPLE_TASK'STORAGE_SIZE use INTEGER'(1);

  task body SIMPLE_TASK is
    BIG : array ( zg_glob1.integer16'(1) .. zg_glob1.integer16'(10000) ) 
			of zg_glob1.integer16;
  begin
                       --+ tcl 30-oct-89 
                       --  Added ACCEPT statement
    accept SYNC;
    zg_glob4.PROCI1 ( BIG (zg_glob4.LL) );  -- should never get past accept
    ERROR_FLAG := TRUE;  -- if the task body is ever executed it's BIG error
                       -- No exception handler in body. System should raise 
                       -- STORAGE_ERROR or TASKING_ERROR in the declarative
                       -- region and propagate it in the frame of the caller
  end SIMPLE_TASK;

begin


  --
  -- Test Name         :  tk_ss_task_54_mod 
  -- Prime Purpose     :  To measure the time required when a task specifies 
  --                      an inadequate storage_size with a static expression. 
  --                      The resulting exception is handled with a null 
  --                      statement.
  -- LRM Version       :  Revision A
  -- Optimization      :  
  -- Related tests     :  
  -- Author            :  Thomas C. Leavitt
  -- Reviewer          :  Kermit E. Terrell
  -- Date              :  
  -- Source            :  
  -- Dependencies      :  
  -- Other Information :  
  --

  ----------------------------------------------------------------------------
  zg_glob2.put_test_name ("tk_ss_task_54_mod");

  pragma INCLUDE ("startime");

  begin
    declare
      ATTEMPTED_TASK : SIMPLE_TASK;
    begin
                               --+ tcl 30-oct-89 added rendezvous
      ATTEMPTED_TASK.SYNC; -- tasking error may be raised here if the
                               -- system does not elaborate the body of 
                               -- simple task until it is activated (not just
                               -- created).  Attempted_task will not be callable
                               -- so a TASKING_ERROR exception raised here 
                               -- could be acceptable behavior
      raise ILLEGAL_ADA1;  -- Should never get here,              
                               -- to do so means that size clause or 
                               -- resulting storage_error was ignored.
    exception
      when TASKING_ERROR =>
        raise ; -- re-raise it.
      when others =>  -- Should not get here.                           
                           -- Appropriate handler is next declare block out  
        raise ILLEGAL_ADA2;
    end;

    if ERROR_FLAG then
      raise ILLEGAL_ADA1; -- execution should never reach here!      
    end if;

  exception
    when ILLEGAL_ADA1 | ILLEGAL_ADA2 | ILLEGAL_ADA3 | ILLEGAL_ADA4 =>
      raise ;            -- SHOULD NOT GET HERE                      
    when STORAGE_ERROR | TASKING_ERROR => -- SHOULD GET HERE         
      null;
    when CONSTRAINT_ERROR | NUMERIC_ERROR | PROGRAM_ERROR =>
      raise ILLEGAL_ADA4;-- SHOULD NOT GET HERE                      
    when others =>       -- SHOULD NOT GET HERE                      
      raise ILLEGAL_ADA3;-- implementation defined exception message 
  end;

  pragma INCLUDE ("stoptime0");

  zg_glob2.put_comment_optional
	("   inadequate task STORAGE_SIZE using static expression");
  zg_glob2.put_new_line_optional;

  pragma INCLUDE ("stoptime2");
  ----------------------------------------------------------------------------

exception
  when ILLEGAL_ADA1 =>
    zg_glob2.put_comment 
	("   inadequate task STORAGE_SIZE using static expression");
    zg_glob2.put_new_line;
    zg_glob2.put_comment 
	("   ERROR !! THIS IMPLEMENTATION ENTERED TASK WHEN THERE ");
    zg_glob2.put_new_line;
    zg_glob2.put_comment 
	("   IS NOT ENOUGH SPACE - IGNORING STORAGE_SIZE !! ");
    zg_glob2.put_new_line;
    zg_glob2.put_error_code ( zg_glob1.ERR_AT_EXECUTION_TIME );

  when ILLEGAL_ADA2 =>
    zg_glob2.put_comment 
	("   inadequate task STORAGE_SIZE using static expression");
    zg_glob2.put_new_line;
    zg_glob2.put_comment 
	("   WRONG EXCEPTION HANDLER ENTERED !! ");
    zg_glob2.put_new_line;
    zg_glob2.put_error_code ( zg_glob1.ERR_AT_EXECUTION_TIME );

  when ILLEGAL_ADA3 =>
    zg_glob2.put_comment 
	("   inadequate task STORAGE_SIZE using static expression");
    zg_glob2.put_new_line;
    zg_glob2.put_comment 
	("   System RAISED implementation defined exception ");
    zg_glob2.put_new_line;
    zg_glob2.put_error_code ( zg_glob1.ERR_AT_EXECUTION_TIME );

  when ILLEGAL_ADA4 =>
    zg_glob2.put_comment 
	("   inadequate task STORAGE_SIZE using static expression");
    zg_glob2.put_new_line;
    zg_glob2.put_comment ("   System RAISED wrong exception !! ");
    zg_glob2.put_new_line;
    zg_glob2.put_error_code ( zg_glob1.ERR_AT_EXECUTION_TIME );

  when others =>
    zg_glob2.put_comment 
	("   inadequate task STORAGE_SIZE using static expression");
    zg_glob2.put_new_line;
    zg_glob2.put_comment ("   HANDLER FOR OTHERS ENTERED !! ");
    zg_glob2.put_new_line;
    zg_glob2.put_error_code ( zg_glob1.ERR_AT_EXECUTION_TIME );

end tk_ss_task_54_mod ;
