--tcl 10-oct-89 review, moved to separate package
--    to inhibit possible procedure inlining
 
WITH zg_glob1;
USE  zg_glob1;
WITH zg_glob4;
USE  zg_glob4;

PACKAGE pkg01_xh_eh01 IS
  excpt : EXCEPTION;
  FUNCTION ctr(unc_arr : IN string) RETURN string;
  FUNCTION append(unc_arr1 : IN string; unc_arr2 : IN string) RETURN string;
END pkg01_xh_eh01;
 
PACKAGE BODY pkg01_xh_eh01 IS
 
  FUNCTION ctr(unc_arr : IN string) RETURN string IS
  BEGIN
    zg_glob4.ii := zg_glob4.ii + 1;
    IF zg_glob4.ii > 1 THEN
      RAISE excpt;
    END IF;
 
    RETURN unc_arr;
 
  END ctr;
 
  FUNCTION append(unc_arr1 : IN string; unc_arr2 : IN string) RETURN string IS
  BEGIN
    RETURN unc_arr1 & unc_arr2;
  END append;
 
END pkg01_xh_eh01;

WITH zg_glob1;
USE  zg_glob1;
WITH zg_glob2;
USE  zg_glob2;
WITH zg_glob3;
USE  zg_glob3;
WITH zg_glob4;
USE  zg_glob4;

WITH calendar;
USE  calendar;
WITH pkg01_xh_eh01;
USE  pkg01_xh_eh01;
 
PROCEDURE xh_eh_funcexcp IS

  vec1 : string(1 .. 7);
  vec2 : string(1 .. 6);

BEGIN
  -- Verify that the program was not compiled with checking suppressed

    DECLARE
      SUBTYPE mytype IS zg_glob1.integer16 RANGE 15000 .. 15001;
      a : mytype;
    BEGIN
      a := mytype(zg_glob4.ei);     -- not in range, ei has value "1"
                           -- this assignment should raise CONSTRAINT_ERROR
 
      zg_glob2.put_comment("   The system is NOT performing range checks. ");
      zg_glob2.put_new_line;
      zg_glob2.put_comment(
            "   This is either an implementation error or the program was ");
      zg_glob2.put_new_line;
      zg_glob2.put_comment(
            "   improperly compiled with options requesting suppression of ");
      zg_glob2.put_new_line;
      zg_glob2.put_comment("   constraint checking. ");
      zg_glob2.put_new_line(2);
      zg_glob2.put_comment(
             "   User should verify compiler option settings and recompile if");
      zg_glob2.put_new_line;
      zg_glob2.put_comment("   needed. ");
      zg_glob2.put_new_line(2);
 
      zg_glob2.put_test_name("xh_eh_funcexcp");
      zg_glob2.put_comment(
              "   ERROR - system is suppressing all range checking ");
      zg_glob2.put_new_line;
      zg_glob2.put_error_code(zg_glob1.err_at_execution_time);
      zg_glob1.bool := false;
    EXCEPTION
      WHEN constraint_error =>
        zg_glob1.bool := true;
    END;
 
    IF NOT zg_glob1.bool THEN   -- if system is suppressing range checking
      raise program_error; 
    END IF;                     -- no point in proceeding

  vec1 := "Hi     ";
  vec2 := "ACES  ";
  
  --
  -- Test Name       :  xh_eh_funcexcp
  -- Prime Purpose   :  To measure the time associated with cleaning up the
  --                    stack when an exception is raised during nested 
  --                    function calls.
  -- LRM Version     :  A
  -- Optimization    :
  -- Related tests   :
  -- Author          :  D. Jecha
  -- Reviewer        :  T. Leavitt, 10-oct-89
  -- Date            :
  -- Source          :
  -- Dependencies    :
  -- Other Information :  None.
  --
   
  ----------------------------------------------------------------------------
  PRAGMA include("startime");
 
  BEGIN
    zg_glob4.ii := 0;
    zg_glob4.string10 := pkg01_xh_eh01.append(pkg01_xh_eh01.ctr(vec1),
         pkg01_xh_eh01.ctr(vec2));
    -- The above statement should raise the EXCPT exception
    -- This point should never be reached.  Die raises a
    -- PROGRAM_ERROR exception.
    zg_glob4.die;
 
  EXCEPTION
    WHEN pkg01_xh_eh01.excpt =>
      NULL;
  END;
 
  PRAGMA include("stoptime0");
  zg_glob2.put_test_name("xh_eh_funcexcp");
  zg_glob2.put_comment_optional(
  "   BEGIN ");
  zg_glob2.put_new_line_optional;
  zg_glob2.put_comment_optional(
  "     ii := 0; ");
  zg_glob2.put_new_line_optional;
  zg_glob2.put_comment_optional(
  "     string10 := Append ( Ctr ( vec1 ), Ctr ( vec2 )); ");
  zg_glob2.put_new_line_optional;
  zg_glob2.put_comment_optional(
  "     -- Second call on Ctr (counted in ii) will raise user defined ");
  zg_glob2.put_new_line_optional;
  zg_glob2.put_comment_optional(
  "     -- exception EXCPT, forcing entry into handler ");
  zg_glob2.put_new_line_optional;
  zg_glob2.put_comment_optional(
  "     Die;  -- This point will never be reached  ");
  zg_glob2.put_new_line_optional;
  zg_glob2.put_comment_optional(
  "           -- Die will raise a program_error when called ");
  zg_glob2.put_new_line_optional;
  zg_glob2.put_comment_optional(
  "   EXCEPTION ");
  zg_glob2.put_new_line_optional;
  zg_glob2.put_comment_optional(
  "     WHEN EXCPT =>  null;  -- this will be reached ");
  zg_glob2.put_new_line_optional;
  zg_glob2.put_comment_optional(
  "   END; ");
  zg_glob2.put_new_line_optional;
  zg_glob2.put_comment_optional(
  "   -- Purpose of test is to measure the time associated with ");
  zg_glob2.put_new_line_optional;
  zg_glob2.put_comment_optional(
  "   -- cleaning up the stack when an exception is raised during ");
  zg_glob2.put_new_line_optional;
  zg_glob2.put_comment_optional(
  "   -- a nested function call.  A poor implementation may leave ");
  zg_glob2.put_new_line_optional;
  zg_glob2.put_comment_optional(
  "   -- garbage on the stack associated with function calls and actual ");
  zg_glob2.put_new_line_optional;
  zg_glob2.put_comment_optional(
  "   -- parameters. Space could be exhausted and program fail.");
  zg_glob2.put_new_line_optional;
  PRAGMA include("stoptime2");
  ----------------------------------------------------------------------------
     
EXCEPTION
  WHEN program_error =>     
    -- Die was called indicating that program did not work properly
    zg_glob2.put_test_name("xh_eh_funcexcp");
    zg_glob2.put_comment_optional(
  "   --raised PROGRAM_ERROR -  user-defined exception was not raised");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_error_code(zg_glob1.err_at_execution_time); 
          --tcl 10-oct-89, review
 
  WHEN storage_error =>                          -- Storage exhausted
    zg_glob2.put_test_name("xh_eh_funcexcp");
    zg_glob2.put_comment_optional(
  "   --raised STORAGE_ERROR -  does not reclaim stack space");
    zg_glob2.put_new_line_optional;
    zg_glob2.put_error_code(zg_glob1.err_at_execution_time);    
         --tcl 10-oct-89, review

END xh_eh_funcexcp ;  
