WITH zg_glob1;
USE  zg_glob1;
WITH zg_glob2;
USE  zg_glob2;
WITH zg_glob3;
USE  zg_glob3;
WITH zg_glob4;
USE  zg_glob4;

WITH calendar;
USE  calendar;
 
PROCEDURE xh_er_except_raise_01 IS

  PRAGMA suppress(access_check);
  PRAGMA suppress(discriminant_check);
  PRAGMA suppress(index_check);
  PRAGMA suppress(length_check);
  --pragma suppress(range_check);  
  --for xh_er_except_raise_01 provokes range error
  PRAGMA suppress(division_check);
  PRAGMA suppress(overflow_check);
  PRAGMA suppress(elaboration_check);
  PRAGMA suppress(storage_check);

BEGIN
  --
  -- Test Name         :  xh_er_except_raise_01
  -- Prime Purpose     :  Raise range constraint, process exception.
  -- LRM Version       :  Revision A
  -- Optimization      :
  -- Related tests     :
  -- Author            :  Thomas C. Leavitt
  -- Reviewer          :  Kermit E. Terrell
  -- Date              :
  -- Source            :
  -- Dependencies      :
  -- Other Information :
  --

    DECLARE
      SUBTYPE mytype IS zg_glob1.integer16 RANGE 15000 .. 15001;
      a : mytype;
    BEGIN
      a := mytype(zg_glob4.ei);     -- not in range, ei has value "1"
                           -- this assignment should raise CONSTRAINT_ERROR
 
      zg_glob2.put_comment("   The system is NOT performing range checks. ");
      zg_glob2.put_new_line;
      zg_glob2.put_comment(
            "   This is either an implementation error or the program was ");
      zg_glob2.put_new_line;
      zg_glob2.put_comment(
            "   improperly compiled with options requesting suppression of ");
      zg_glob2.put_new_line;
      zg_glob2.put_comment("   constraint checking. ");
      zg_glob2.put_new_line(2);
      zg_glob2.put_comment(
             "   User should verify compiler option settings and recompile if");
      zg_glob2.put_new_line;
      zg_glob2.put_comment("   needed. ");
      zg_glob2.put_new_line(2);
 
      zg_glob2.put_test_name("xh_er_except_raise_01");
      zg_glob2.put_comment(
              "   ERROR - system is suppressing all range checking ");
      zg_glob2.put_new_line;
      zg_glob2.put_error_code(zg_glob1.err_at_execution_time);
      zg_glob1.bool := false;
    EXCEPTION
      WHEN constraint_error =>
        zg_glob1.bool := true;
    END;
 
    IF NOT zg_glob1.bool THEN   -- if system is suppressing range checking
      raise program_error; 
    END IF;                     -- no point in proceeding


  zg_glob1.bool := false;

  ---------------------------------------------------------------------------
  zg_glob2.put_test_name("xh_er_except_raise_01");
  PRAGMA include("startime");
  DECLARE
    SUBTYPE small_int IS zg_glob1.integer16 RANGE 1 .. 5;
    i : small_int;
  BEGIN
    zg_glob4.ii := zg_glob4.ei - 1;        -- ii:=0; out of range
    BEGIN
      i := zg_glob4.ii;           -- should raise exception
    EXCEPTION
      WHEN OTHERS =>
        zg_glob4.ii := 1;
        i := 1;
    END;
                                  --+ KET 9/14/89 modified to correct Eilers #1
    IF zg_glob1.bool THEN
      zg_glob4.proci1(i);
    END IF;
  END;
  PRAGMA include("stoptime0");
  zg_glob2.put_comment_optional(
  "   --Raise and handle constraint error by attempting to assign an");
  zg_glob2.put_new_line_optional;
  zg_glob2.put_comment_optional(
  "   --out-of-range variable to a subtype");
  zg_glob2.put_new_line_optional;
  PRAGMA include("stoptime2");
  ---------------------------------------------------------------------------

END xh_er_except_raise_01 ;  
