WITH xh_pepkg01;
USE  xh_pepkg01;

WITH zg_glob1;
USE  zg_glob1;
WITH zg_glob2;
USE  zg_glob2;
WITH zg_glob3;
USE  zg_glob3;
WITH zg_glob4;
USE  zg_glob4;

WITH calendar;
USE  calendar;

PACKAGE pkg01_xh_pe01 IS
  pragma Elaborate_Body;  -- Required by Ada 95.  PCR 2.32   DEC 94
 
                                  --  no objects or types are exported
END pkg01_xh_pe01;
 
PACKAGE BODY pkg01_xh_pe01 IS
  
BEGIN
 
  PRAGMA include("inittime"); --timing loop must be initialized here
 
  ------------   test to ensure that program was not compiled with all
  ------------   constraint checking suppressed
    DECLARE
      SUBTYPE mytype IS zg_glob1.integer16 RANGE 15000 .. 15001;
      a : mytype;
    BEGIN
      a := mytype(zg_glob4.ei);     -- not in range, ei has value "1"
                           -- this assignment should raise CONSTRAINT_ERROR
 
      zg_glob2.put_comment("   The system is NOT performing range checks. ");
      zg_glob2.put_new_line;
      zg_glob2.put_comment(
            "   This is either an implementation error or the program was ");
      zg_glob2.put_new_line;
      zg_glob2.put_comment(
            "   improperly compiled with options requesting suppression of ");
      zg_glob2.put_new_line;
      zg_glob2.put_comment("   constraint checking. ");
      zg_glob2.put_new_line(2);
      zg_glob2.put_comment(
            "   User should verify compiler option settings and recompile if");
      zg_glob2.put_new_line;
      zg_glob2.put_comment("   needed. ");
      zg_glob2.put_new_line(2);
 
      zg_glob2.put_test_name("xh_pe_prgma_sup_elab_ck_01");
      zg_glob2.put_comment(
              "   ERROR - system is suppressing all range checking ");
      zg_glob2.put_new_line;
      zg_glob2.put_error_code(zg_glob1.err_at_execution_time);
      zg_glob1.bool := false;
    EXCEPTION
      WHEN constraint_error =>
        zg_glob1.bool := true;
    END;
 
    IF NOT zg_glob1.bool THEN   -- if system is suppressing range checking
      raise program_error; 
    END IF;                     -- no point in proceeding
 
  --
  -- Test Name         :  xh_pe_prgma_sup_elab_ck_01
  -- Prime Purpose     :  These sets of tests examine system performance on
  --                      test problems (compiled without suppression) which
  --                      contain calls on subprograms declared in external
  --                      procedures.  They are designed to test if the system
  --                      optimized some (any) of the elaboration checking code
  --                      required to verify that the package body has been
  --                      elaborated before subprograms in it are called.
  --                      Because the calls are in a conditional statement, an
  --                      optimizing compiler cannot move the elaboration check
  --                      code out of the timing loop. This problem calls one
  --                      procedure five times in a row.  A good compiler would
  --                      only generate code to perform one test.
  -- LRM Version       :  A
  -- Optimization      :
  -- Related tests     :  xh_pe_prgma_sup_elab_ck_*
  -- Author            :  Thomas C. Leavitt
  -- Reviewer          :  Kermit E. Terrell
  -- Date              :
  -- Source            :
  -- Dependencies      :
  -- Other Information :  
  --
  
  ----------------------------------------------------------------------------
  zg_glob2.put_test_name("xh_pe_prgma_sup_elab_ck_01");
  PRAGMA include("startime");
   -- define a frame to make it necessary to retain the elaboration checking
   -- code within the timing loop. Make calls on the procedures conditional 
   -- so that it will not be a valid optimization to verify xh_pepkg01
   -- at the beginning of the main program.
  BEGIN
    IF zg_glob4.ll /= zg_glob4.mm THEN    -- always true
      xh_pepkg01.elab_01;
      xh_pepkg01.elab_01;
      xh_pepkg01.elab_01;
      xh_pepkg01.elab_01;
      xh_pepkg01.elab_01;
    END IF;
  EXCEPTION
    WHEN program_error =>
      zg_glob4.die;
  END;
  PRAGMA include("stoptime0");
  zg_glob2.put_comment_optional(
  "   --5 calls on one procedure in an external package ");
  zg_glob2.put_new_line_optional;
  zg_glob2.put_comment_optional(
  "   --This may include multiple calls on a procedure in a package which");
  zg_glob2.put_new_line_optional;
  zg_glob2.put_comment_optional(
  "   --does not suppress elaboration checking. A non-optimizing system");
  zg_glob2.put_new_line_optional;
  zg_glob2.put_comment_optional(
  "   --might call on a routine to verify the elaboration of package body");
  zg_glob2.put_new_line_optional;
  zg_glob2.put_comment_optional(
  "   --before every call on a subprogram defined in an external package.  A");
  zg_glob2.put_new_line_optional;
  zg_glob2.put_comment_optional(
  "   --smart compiler would determine it could verify correctness using");
  zg_glob2.put_new_line_optional;
  zg_glob2.put_comment_optional(
  "   --fewer checks --- redundant calls on the checking code can be");
  zg_glob2.put_new_line_optional;
  zg_glob2.put_comment_optional(
  "   --eliminated, and checking code could be moved out of loops.  Calls ");
  zg_glob2.put_new_line_optional;
  zg_glob2.put_comment_optional(
  "   --are conditional so that it would not be trivially optimized by loop");
  zg_glob2.put_new_line_optional;
  zg_glob2.put_comment_optional(
  "   --invariant motion out of the timing loop, with conditional execution,");
  zg_glob2.put_new_line_optional;
  zg_glob2.put_comment_optional(
  "   --a system which moved test out of loop would raise exception for a");
  zg_glob2.put_new_line_optional;
  zg_glob2.put_comment_optional(
  "   --statement which might not be executed at all. Note problems are ");
  zg_glob2.put_new_line_optional;
  zg_glob2.put_comment_optional(
  "   --executed in a library package body, so an optimizing compiler can't");
  zg_glob2.put_new_line_optional;
  zg_glob2.put_comment_optional(
  "   --assume that all referenced library packages have been elaborated. ");
  zg_glob2.put_new_line_optional;
  zg_glob2.put_comment_optional(
  "   --Also, all library packages have bodies and initialization code. ");
  zg_glob2.put_new_line_optional;
  PRAGMA include("stoptime2");
  ----------------------------------------------------------------------------
 
END pkg01_xh_pe01;
----------------------------------
 
WITH pkg01_xh_pe01;
USE  pkg01_xh_pe01;
PRAGMA elaborate(pkg01_xh_pe01);
 
PROCEDURE xh_pe_prgma_sup_elab_ck_01 IS
BEGIN
  NULL;  -- the test problem is contained in the package body
END xh_pe_prgma_sup_elab_ck_01;
