WITH xh_pepkg07;
USE  xh_pepkg07;

WITH zg_glob1;
USE  zg_glob1;
WITH zg_glob2;
USE  zg_glob2;
WITH zg_glob3;
USE  zg_glob3;
WITH zg_glob4;
USE  zg_glob4;

WITH calendar;
USE  calendar;

PACKAGE pkg01_xh_pe03 IS
  pragma Elaborate_Body;  -- Required by Ada 95.  PCR 2.32   DEC 94
 
                                  --  no objects or types are exported
END pkg01_xh_pe03;
 
PACKAGE BODY pkg01_xh_pe03 IS
  
BEGIN
 
  PRAGMA include("inittime"); --timing loop must be initialized here
 
  ------------   test to ensure that program was not compiled with all
  ------------   constraint checking suppressed
    DECLARE
      SUBTYPE mytype IS zg_glob1.integer16 RANGE 15000 .. 15001;
      a : mytype;
    BEGIN
      a := mytype(zg_glob4.ei);     -- not in range, ei has value "1"
                           -- this assignment should raise CONSTRAINT_ERROR
 
      zg_glob2.put_comment("   The system is NOT performing range checks. ");
      zg_glob2.put_new_line;
      zg_glob2.put_comment(
            "   This is either an implementation error or the program was ");
      zg_glob2.put_new_line;
      zg_glob2.put_comment(
            "   improperly compiled with options requesting suppression of ");
      zg_glob2.put_new_line;
      zg_glob2.put_comment("   constraint checking. ");
      zg_glob2.put_new_line(2);
      zg_glob2.put_comment(
            "   User should verify compiler option settings and recompile if");
      zg_glob2.put_new_line;
      zg_glob2.put_comment("   needed. ");
      zg_glob2.put_new_line(2);
 
      zg_glob2.put_test_name("xh_pe_prgma_sup_elab_ck_03");
      zg_glob2.put_comment(
              "   ERROR - system is suppressing all range checking ");
      zg_glob2.put_new_line;
      zg_glob2.put_error_code(zg_glob1.err_at_execution_time);
      zg_glob1.bool := false;
    EXCEPTION
      WHEN constraint_error =>
        zg_glob1.bool := true;
    END;
 
    IF NOT zg_glob1.bool THEN   -- if system is suppressing range checking
      raise program_error; 
    END IF;                     -- no point in proceeding
 
  --
  -- Test Name         :  xh_pe_prgma_sup_elab_ck_03
  -- Prime Purpose     :  This is one of the elab* set of test problems. This
  --                      problem calls the same procedure four times in a row
  --                      unconditionally and then tests the condition and 
  --                      calls it one more time.  An optimizing compiler 
  --                      could move the elaboration checking code out of the
  --                      timing loop.
  -- LRM Version       :  A
  -- Optimization      :
  -- Related tests     :  xh_pe_prgma_sup_elab_ck_*
  -- Author            :  Thomas C. Leavitt
  -- Reviewer          :  Kermit E. Terrell
  -- Date              :
  -- Source            :
  -- Dependencies      :
  -- Other Information :  
  --
  
  ----------------------------------------------------------------------------
  zg_glob2.put_test_name("xh_pe_prgma_sup_elab_ck_03");
  PRAGMA include("startime");
  BEGIN
       xh_pepkg07.elab_07;
       xh_pepkg07.elab_07;
       xh_pepkg07.elab_07;
       xh_pepkg07.elab_07;
    IF zg_glob4.ll /= zg_glob4.mm THEN    -- always true
       xh_pepkg07.elab_07;
    END IF;
  EXCEPTION
    WHEN program_error =>
      zg_glob4.die;
  END;
  PRAGMA include("stoptime0");
  zg_glob2.put_comment_optional(
  "   --5 calls on same procedure; 4 unconditional. ");
  zg_glob2.put_new_line_optional;
  zg_glob2.put_comment_optional(
  "   --One call on checking code is sufficient ");
  zg_glob2.put_new_line_optional;
  PRAGMA include("stoptime2");
  ----------------------------------------------------------------------------
 
END pkg01_xh_pe03;
----------------------------------
 
WITH pkg01_xh_pe03;
USE  pkg01_xh_pe03;
PRAGMA elaborate(pkg01_xh_pe03);
 
PROCEDURE xh_pe_prgma_sup_elab_ck_03 IS
BEGIN
  NULL;    -- the actual test problem is contained in the package body
END xh_pe_prgma_sup_elab_ck_03;
