WITH xh_pepkg08;
USE  xh_pepkg08;

WITH zg_glob1;
USE  zg_glob1;
WITH zg_glob2;
USE  zg_glob2;
WITH zg_glob3;
USE  zg_glob3;
WITH zg_glob4;
USE  zg_glob4;

WITH calendar;
USE  calendar;
  
PACKAGE pkg01_xh_pe04 IS
  pragma Elaborate_Body;  -- Required by Ada 95.  PCR 2.32   DEC 94
 
                                  --  no objects or types are exported
END pkg01_xh_pe04;
 
PACKAGE BODY pkg01_xh_pe04 IS
  
BEGIN
 
  PRAGMA include("inittime"); --timing loop must be initialized here
 
  ------------   test to ensure that program was not compiled with all
  ------------   constraint checking suppressed
    DECLARE
      SUBTYPE mytype IS zg_glob1.integer16 RANGE 15000 .. 15001;
      a : mytype;
    BEGIN
      a := mytype(zg_glob4.ei);     -- not in range, ei has value "1"
                           -- this assignment should raise CONSTRAINT_ERROR
 
      zg_glob2.put_comment("   The system is NOT performing range checks. ");
      zg_glob2.put_new_line;
      zg_glob2.put_comment(
            "   This is either an implementation error or the program was ");
      zg_glob2.put_new_line;
      zg_glob2.put_comment(
            "   improperly compiled with options requesting suppression of ");
      zg_glob2.put_new_line;
      zg_glob2.put_comment("   constraint checking. ");
      zg_glob2.put_new_line(2);
      zg_glob2.put_comment(
            "   User should verify compiler option settings and recompile if");
      zg_glob2.put_new_line;
      zg_glob2.put_comment("   needed. ");
      zg_glob2.put_new_line(2);
 
      zg_glob2.put_test_name("xh_pe_prgma_sup_elab_ck_04");
      zg_glob2.put_comment(
              "   ERROR - system is suppressing all range checking ");
      zg_glob2.put_new_line;
      zg_glob2.put_error_code(zg_glob1.err_at_execution_time);
      zg_glob1.bool := false;
    EXCEPTION
      WHEN constraint_error =>
        zg_glob1.bool := true;
    END;
 
    IF NOT zg_glob1.bool THEN   -- if system is suppressing range checking
      raise program_error; 
    END IF;                     -- no point in proceeding
 
  --
  -- Test Name         :  xh_pe_prgma_sup_elab_ck_04
  -- Prime Purpose     :  This is one of the elab* set of test problems. This
  --                      problem conditionally calls on five different
  --                      procedures defined in the same package. Similar to
  --                      xh_pe_prgma_sup_elab_ck_01, an optimizing compiler 
  --                      could share the checking code for the package body 
  --                      elaboration.
  -- LRM Version       :  A
  -- Optimization      :
  -- Related tests     :  xh_pe_prgma_sup_elab_ck_*
  -- Author            :  Thomas C. Leavitt
  -- Reviewer          :  Kermit E. Terrell
  -- Date              :
  -- Source            :
  -- Dependencies      :
  -- Other Information :  
  --
  
  ----------------------------------------------------------------------------
  zg_glob2.put_test_name("xh_pe_prgma_sup_elab_ck_04");
  PRAGMA include("startime");
  BEGIN
    IF zg_glob4.ll /= zg_glob4.mm THEN    -- always true
      xh_pepkg08.elab_08;
      xh_pepkg08.elab_09;
      xh_pepkg08.elab_10;
      xh_pepkg08.elab_11;
      xh_pepkg08.elab_12;
    END IF;
  EXCEPTION
    WHEN program_error =>
      zg_glob4.die;
  END;
  PRAGMA include("stoptime0");
  zg_glob2.put_comment_optional(
  "   --5 calls on different procedures in same external package ");
  zg_glob2.put_new_line_optional;
  zg_glob2.put_comment_optional(
  "   --could share the elaboration checking code. ");
  zg_glob2.put_new_line_optional;
  PRAGMA include("stoptime2");
  ----------------------------------------------------------------------------
 
END pkg01_xh_pe04;
----------------------------------
 
WITH pkg01_xh_pe04;
USE  pkg01_xh_pe04;
PRAGMA elaborate(pkg01_xh_pe04);
 
PROCEDURE xh_pe_prgma_sup_elab_ck_04 IS
BEGIN
  NULL;    -- the actual test problem is contained in the package body
END xh_pe_prgma_sup_elab_ck_04;                            
