WITH xh_pepkg09;
USE  xh_pepkg09;

WITH zg_glob1;
USE  zg_glob1;
WITH zg_glob2;
USE  zg_glob2;
WITH zg_glob3;
USE  zg_glob3;
WITH zg_glob4;
USE  zg_glob4;

WITH calendar;
USE  calendar;

PRAGMA elaborate(xh_pepkg09);
  
PACKAGE pkg01_xh_pe05 IS
  pragma Elaborate_Body;  -- Required by Ada 95.  PCR 2.32   DEC 94
 
                                  --  no objects or types are exported
END pkg01_xh_pe05;
 
PACKAGE BODY pkg01_xh_pe05 IS
  
BEGIN
 
  PRAGMA include("inittime"); --timing loop must be initialized here
 
  ------------   test to ensure that program was not compiled with all
  ------------   constraint checking suppressed
    DECLARE
      SUBTYPE mytype IS zg_glob1.integer16 RANGE 15000 .. 15001;
      a : mytype;
    BEGIN
      a := mytype(zg_glob4.ei);     -- not in range, ei has value "1"
                           -- this assignment should raise CONSTRAINT_ERROR
 
      zg_glob2.put_comment("   The system is NOT performing range checks. ");
      zg_glob2.put_new_line;
      zg_glob2.put_comment(
            "   This is either an implementation error or the program was ");
      zg_glob2.put_new_line;
      zg_glob2.put_comment(
            "   improperly compiled with options requesting suppression of ");
      zg_glob2.put_new_line;
      zg_glob2.put_comment("   constraint checking. ");
      zg_glob2.put_new_line(2);
      zg_glob2.put_comment(
            "   User should verify compiler option settings and recompile if");
      zg_glob2.put_new_line;
      zg_glob2.put_comment("   needed. ");
      zg_glob2.put_new_line(2);
 
      zg_glob2.put_test_name("xh_pe_prgma_sup_elab_ck_05");
      zg_glob2.put_comment(
              "   ERROR - system is suppressing all range checking ");
      zg_glob2.put_new_line;
      zg_glob2.put_error_code(zg_glob1.err_at_execution_time);
      zg_glob1.bool := false;
    EXCEPTION
      WHEN constraint_error =>
        zg_glob1.bool := true;
    END;
 
    IF NOT zg_glob1.bool THEN   -- if system is suppressing range checking
      raise program_error; 
    END IF;                     -- no point in proceeding
 
  --
  -- Test Name         :  xh_pe_prgma_sup_elab_ck_05
  -- Prime Purpose     :  This problem calls one procedure five times in a row
  --                      within a conditional statement.  The procedure is in
  --                      a package which has a PRAGMA ELABORATE specified,
  --                      permitting an optimizing compiler to omit checking
  --                      code because the PRAGMA guarantees prior elaboration.
  -- LRM Version       :  A
  -- Optimization      :
  -- Related tests     :  xh_pe_prgma_sup_elab_ck_*
  -- Author            :  Thomas C. Leavitt
  -- Reviewer          :  Kermit E. Terrell
  -- Date              :
  -- Source            :
  -- Dependencies      :
  -- Other Information :  
  --
  
  ----------------------------------------------------------------------------
  zg_glob2.put_test_name("xh_pe_prgma_sup_elab_ck_05");
  PRAGMA include("startime");
  BEGIN
    IF zg_glob4.ll /= zg_glob4.mm THEN    -- always true
      xh_pepkg09.elab_13;
      xh_pepkg09.elab_13;
      xh_pepkg09.elab_13;
      xh_pepkg09.elab_13;
      xh_pepkg09.elab_13;
    END IF;
  EXCEPTION
    WHEN program_error =>
      zg_glob4.die;
  END;
  PRAGMA include("stoptime0");
  zg_glob2.put_comment_optional(
  "   --5 calls on one procedure in an external package which has had a");
  zg_glob2.put_new_line_optional;
  zg_glob2.put_comment_optional(
  "   --PRAGMA ELABORATE specified, making an explicit check for elaborate ");
  zg_glob2.put_new_line_optional;
  zg_glob2.put_comment_optional(
  "   --code superfluous.  cf xh_pe_prgma_sup_elab_ck_01 ");
  zg_glob2.put_new_line_optional;
  PRAGMA include("stoptime2");
  ----------------------------------------------------------------------------
 
END pkg01_xh_pe05;
----------------------------------
 
WITH pkg01_xh_pe05;
USE  pkg01_xh_pe05;
PRAGMA elaborate(pkg01_xh_pe05);
 
PROCEDURE xh_pe_prgma_sup_elab_ck_05 IS
BEGIN
  NULL;    -- the actual test problem is contained in the package body
END xh_pe_prgma_sup_elab_ck_05;                            
