WITH zg_glob1;
USE  zg_glob1;
WITH zg_glob2;
USE  zg_glob2;
WITH zg_glob3;
USE  zg_glob3;
WITH zg_glob4;
USE  zg_glob4;

WITH calendar;
USE  calendar;
 
PROCEDURE xh_pr_prgma_sup_range_ck_02 IS

-- There is no pragma suppress (index_check) or (range_check) here since 
-- the purpose of these series of tests is to evaluate cost of checking.
 
  PRAGMA suppress(access_check);
  PRAGMA suppress(discriminant_check);
  -------------------------------------------pragma suppress(index_check);
  PRAGMA suppress(length_check);
  -------------------------------------------pragma suppress(range_check);
  PRAGMA suppress(division_check);
  PRAGMA suppress(overflow_check);
  PRAGMA suppress(elaboration_check);
  PRAGMA suppress(storage_check);

BEGIN
  --
  -- Test Name         :  xh_pr_prgma_sup_range_ck_02
  -- Prime Purpose     :  fetch from 1D array with subscript range checking,
  --                      using constant subscript
  -- LRM Version       :  Revision A
  -- Optimization      :  folding - should check literal range at compile
  --                      time
  -- Related tests     :  op_fs_fold_simp_03
  -- Author            :  Thomas C. Leavitt
  -- Reviewer          :  Thomas M. Lee
  -- Date              :  October 12, 1989
  -- Source            :
  -- Dependencies      :
  -- Other Information :
  --
 
  ------------   test to ensure that program was not compiled with all
  ------------   constraint checking suppressed

    DECLARE
      SUBTYPE mytype IS zg_glob1.integer16 RANGE 15000 .. 15001;
      a : mytype;
    BEGIN
      a := mytype(zg_glob4.ei);     -- not in range, ei has value "1"
                           -- this assignment should raise CONSTRAINT_ERROR
 
      zg_glob2.put_comment("   The system is NOT performing range checks. ");
      zg_glob2.put_new_line;
      zg_glob2.put_comment(
            "   This is either an implementation error or the program was ");
      zg_glob2.put_new_line;
      zg_glob2.put_comment(
            "   improperly compiled with options requesting suppression of ");
      zg_glob2.put_new_line;
      zg_glob2.put_comment("   constraint checking. ");
      zg_glob2.put_new_line(2);
      zg_glob2.put_comment(
             "   User should verify compiler option settings and recompile if");
      zg_glob2.put_new_line;
      zg_glob2.put_comment("   needed. ");
      zg_glob2.put_new_line(2);
 
      zg_glob2.put_test_name("xh_pr_prgma_sup_range_ck_02");
      zg_glob2.put_comment(
              "   ERROR - system is suppressing all range checking ");
      zg_glob2.put_new_line;
      zg_glob2.put_error_code(zg_glob1.err_at_execution_time);
      zg_glob1.bool := false;
    EXCEPTION
      WHEN constraint_error =>
        zg_glob1.bool := true;
    END;
 
    IF NOT zg_glob1.bool THEN   -- if system is suppressing range checking
      raise program_error; 
    END IF;                     -- no point in proceeding
  
  ----------------------------------------------------------------------------
  zg_glob2.put_test_name("xh_pr_prgma_sup_range_ck_02");
  PRAGMA include("startime");
  zg_glob4.ii := zg_glob4.i1(1);
  PRAGMA include("stoptime0");
  zg_glob2.put_comment_optional(
  "   ii:=i1(1);  --with checking, cf op_fs_fold_simp_03");
  zg_glob2.put_new_line_optional;
  PRAGMA include("stoptime2");
  ----------------------------------------------------------------------------

END xh_pr_prgma_sup_range_ck_02;  
