WITH zg_glob1;
USE  zg_glob1;
WITH zg_glob2;
USE  zg_glob2;
WITH zg_glob3;
USE  zg_glob3;
WITH zg_glob4;
USE  zg_glob4;

WITH calendar;
USE  calendar;
 
PROCEDURE xh_pr_prgma_sup_range_ck_14 IS

-- no suppression pragmas in this program, checks range constraints

BEGIN
  --
  -- Test Name       :  xh_pr_prgma_sup_range_ck_14
  -- Prime Purpose   :  Assign out of range dynamic expression. Compare with
  --                    op_df_data_flo_03_a through op_df_data_flo_06. This 
  --                    needs explicit checking.
  -- Optimization    :
  -- Related tests   :  op_df_data_flo_03_a, op_df_data_flo_04_a, 
  --                    op_df_data_flo_05, op_df_data_flo_06
  -- Author          :  Thomas C. Leavitt
  -- Reviewer        :  Thomas M. Lee
  -- Date            :  October 18, 1989
  -- Source          :
  -- Dependencies    :
  -- Other Information :
  -- Revisions         :
  -- Auther            :  T. Leavitt
  -- Reviewer          :  
  -- Date              :  11 Jan 1993
  -- Change            :  Changed the name of this test for ACES Version 1
  --                   :  because the change to ZG_GLOB4.PROC0 to inhibit
  --                   :  optimization makes the test incomparable with
  --                   :  earlier versions of the problem. 
  --

  ------------   test to ensure that program was not compiled with all
  ------------   constraint checking suppressed

    DECLARE
      SUBTYPE mytype IS zg_glob1.integer16 RANGE 15000 .. 15001;
      a : mytype;
    BEGIN
      a := mytype(zg_glob4.ei);     -- not in range, ei has value "1"
                           -- this assignment should raise CONSTRAINT_ERROR
 
      zg_glob2.put_comment("   The system is NOT performing range checks. ");
      zg_glob2.put_new_line;
      zg_glob2.put_comment(
            "   This is either an implementation error or the program was ");
      zg_glob2.put_new_line;
      zg_glob2.put_comment(
            "   improperly compiled with options requesting suppression of ");
      zg_glob2.put_new_line;
      zg_glob2.put_comment("   constraint checking. ");
      zg_glob2.put_new_line(2);
      zg_glob2.put_comment(
             "   User should verify compiler option settings and recompile if");
      zg_glob2.put_new_line;
      zg_glob2.put_comment("   needed. ");
      zg_glob2.put_new_line(2);
 
      zg_glob2.put_test_name("xh_pr_prgma_sup_range_ck_14");
      zg_glob2.put_comment(
              "   ERROR - system is suppressing all range checking ");
      zg_glob2.put_new_line;
      zg_glob2.put_error_code(zg_glob1.err_at_execution_time);
      zg_glob1.bool := false;
    EXCEPTION
      WHEN constraint_error =>
        zg_glob1.bool := true;
    END;
 
    IF NOT zg_glob1.bool THEN   -- if system is suppressing range checking
      raise program_error; 
    END IF;                     -- no point in proceeding
 

  zg_glob4.ii := 25 * zg_glob4.ei;   -- setup value which will be out of range
  ----------------------------------------------------------------------------
  zg_glob2.put_test_name("xh_pr_prgma_sup_range_ck_14");
  PRAGMA include("startime");
  BEGIN
    zg_glob4.ri := zg_glob4.ii;            -- out of range
    zg_glob4.die;
  EXCEPTION
    WHEN constraint_error =>
      zg_glob4.proc0;
  END;
  PRAGMA include("stoptime0");
  zg_glob2.put_comment_optional(
  "   --dynamic testing of range constraint test which fails");
  zg_glob2.put_new_line_optional;
  PRAGMA include("stoptime2");
  ----------------------------------------------------------------------------

END xh_pr_prgma_sup_range_ck_14 ;  
