with zg_glob1;
use  zg_glob1;
with zg_glob2;
with zg_glob3;
with zg_glob4;
with calendar;
use  calendar;

procedure tk_ls_task_77 is

pragma suppress(access_check);
pragma suppress(discriminant_check);
pragma suppress(index_check);
pragma suppress(length_check);
pragma suppress(range_check);
pragma suppress(division_check);
pragma suppress(overflow_check);
pragma suppress(elaboration_check);
pragma suppress(storage_check);

pragma OPTIMIZE (TIME);

   delay1, delay2, delay3 : duration := 0.0 ;

   last_selected_alternative : zg_glob1.integer16 range 0..4 := 0;
   first_alternative : zg_glob1.integer16 := 0;
   second_alternative : zg_glob1.integer16 := 0;
  
   first_order, second_order, third_order, cyclic_order, reverse_cyclic_order
           : boolean := false ;

   procedure order( calling_alternative : zg_glob1.integer16 ) is
   begin
     if    first_alternative = 0 
     then
        first_alternative := calling_alternative; 
     elsif second_alternative = 0 
     then
        second_alternative := calling_alternative; 
        first_order  := first_alternative = 1 and second_alternative = 1 ;
        second_order := first_alternative = 2 and second_alternative = 2 ;
        third_order  := first_alternative = 3 and second_alternative = 3 ;
        cyclic_order := (first_alternative + 1) mod 3 =
                         second_alternative mod 3 ;
        reverse_cyclic_order 
                     := (second_alternative + 1) mod 3 =
                         first_alternative mod 3 ;
     elsif first_order  and calling_alternative /= 1 then
        first_order := false;
     elsif second_order and calling_alternative /= 2 then
        second_order := false;
     elsif third_order  and calling_alternative /= 3 then
        third_order := false;
     elsif cyclic_order and (last_selected_alternative + 1) mod 3 /= 
                             calling_alternative mod 3 
     then 
        cyclic_order := false ;
     elsif reverse_cyclic_order and (calling_alternative + 1) mod 3 /= 
                             last_selected_alternative mod 3 
     then 
        reverse_cyclic_order := false ;
     end if;
     last_selected_alternative := calling_alternative ;
   end order ;

   task SELECTION is   -- Work horse
      entry START;
      entry FINISH;
   end SELECTION;

   task body SELECTION is

   begin

      loop 
         accept START;

         select
            delay DELAY1;   -- zero delay 
            order(1); 
         or
            delay DELAY2;   -- zero delay 
            order(2) ;
         or
            delay DELAY3;   -- zero delay 
            order(3) ;
         or
            accept START;   -- Never selected
         end select;

         accept FINISH ;
     end loop;
      
   end SELECTION;

begin
  -- Test Name         :  tk_ls_task_77
  -- Prime Purpose     :  This test determines in what order the system chooses 
  --                      open DELAY alternatives in a SELECT statement when 
  --                      several such alternatives could be selected.  Any
  --                      programs designed to exploit this knowledge gained
  --                      through experimentation will have reduced 
  --                      portability (perhaps the next release of the compiler
  --                      will change the algorithm used without notice) and
  --                      risk unpleasant surprises. The treatment of 
  --                      SELECT statements with multiple DELAY alternatives 
  --                      is not guaranteed to be the same in all cases.
  --                      The selection made in the case tested in this test 
  --                      problem may not be followed in all other cases. This
  --                      test problem uses zero DELAY statements to avoid
  --                      confounding results with task scheduling and noise
  --                      in the clock.
  -- Optimization      :
  -- Related tests     :  
  -- Author            :  Thomas C. Leavitt
  -- Reviewer          :  B. Decker-Lindsey  -  1 April 1992
  -- Date              :  12 March 1992
  -- Source            :  AES Test TL13
  -- Dependencies      :  
  -- Other Information :  

   zg_glob2.put_test_name("tk_ls_task_77");
 
   pragma include("startime");
     last_selected_alternative := 0;
     first_alternative := 0;
     second_alternative := 0;
     for i in 1..100 
     loop
        if zg_glob3.boolean_false then
           delay1 := delay1 - 1.0;
           delay2 := delay2 - 2.0;
           delay3 := delay3 - 3.0;
           zg_glob4.proc1( zg_glob1.float6(delay1) );
           zg_glob4.proc1( zg_glob1.float6(delay2) );
           zg_glob4.proc1( zg_glob1.float6(delay3) );
        end if;
        selection.start;
        selection.finish;
     end loop;
   pragma include ("stoptime0");
   zg_glob2.put_comment_optional(
   "   This test determines in what order the system chooses open DELAY ");
   zg_glob2.put_new_line_optional;
   zg_glob2.put_comment_optional(
   "   alternatives in a SELECT statement when several such alternatives ");
   zg_glob2.put_new_line_optional;
   zg_glob2.put_comment_optional(
   "   could be selected.  Any programs designed to exploit this knowledge ");
   zg_glob2.put_new_line_optional;
   zg_glob2.put_comment_optional(
   "   gained through experimentation will have reduced portability (perhaps");
   zg_glob2.put_new_line_optional;
   zg_glob2.put_comment_optional(
   "   the next release of the compiler will change the algorithm used ");
   zg_glob2.put_new_line_optional;
   zg_glob2.put_comment_optional(
   "   without notice) and risk unpleasant surprises in that the treatment ");
   zg_glob2.put_new_line_optional;
   zg_glob2.put_comment_optional(
   "   of SELECT statements with multiple DELAY alternatives is not guaranteed");
   zg_glob2.put_new_line_optional;
   zg_glob2.put_comment_optional(
   "   to be the same in all cases - the selection made in the case tested ");
   zg_glob2.put_new_line_optional;
   zg_glob2.put_comment_optional(
   "   in this test problem may not be followed in all other cases. ");
   pragma include ("stoptime2");
  
 
   zg_glob2.put_ancillary(" >>> tk_ls_task_77 ");
   if    first_order then
      zg_glob2.put_comment(
        "DELAY alternative selected was always the first in lexical order.");
   elsif second_order then
      zg_glob2.put_comment(
        "DELAY alternative selected was always the second in lexical order.");
   elsif third_order then
      zg_glob2.put_comment(
        "DELAY alternative selected was always the last in lexical order.");
   elsif cyclic_order then
      zg_glob2.put_comment(
        "DELAY alternatives were selected in cyclic order.");
   elsif reverse_cyclic_order then
      zg_glob2.put_comment(
        "DELAY alternatives were selected in reverse cyclic order.");
   else
      zg_glob2.put_comment(
        "DELAY alternatives were selected in undetermined order.");
   end if;
   zg_glob2.put_new_line;

   abort selection;

end tk_ls_task_77 ;
